<?php
// feed.php - API endpoint to fetch reviews for the social media feed

// Set error handling to suppress HTML output
ini_set('display_errors', 0);
error_reporting(E_ALL);

// Function to handle errors as JSON
function returnError($message, $statusCode = 500) {
    http_response_code($statusCode);
    header("Content-Type: application/json");
    echo json_encode(['error' => $message]);
    exit;
}

// Set headers
header("Content-Type: application/json");
header("Access-Control-Allow-Origin: *");
header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");

// Use a try-catch for all code
try {
    // Get pagination parameters
    $page = isset($_GET['page']) ? intval($_GET['page']) : 1;
    $limit = isset($_GET['limit']) ? intval($_GET['limit']) : 10;
    
    // Validate parameters
    if ($page < 1) $page = 1;
    if ($limit < 1) $limit = 10;
    if ($limit > 50) $limit = 50; // Cap at 50 items per page
    
    // Calculate offset
    $offset = ($page - 1) * $limit;
    
    // Connect to SQLite database
    $dbPath = '/home/phantomales/TastingTrail/Producer.db';
    
    // Check if database file exists
    if (!file_exists($dbPath)) {
        returnError("Database file not found at: $dbPath");
    }
    
    // Connect to the database
    $db = new SQLite3($dbPath);
    
    if (!$db) {
        returnError("Failed to open database: " . SQLite3::lastErrorMsg());
    }
    
    // Query to get recent reviews with venue details
    $sql = "
        SELECT 
            r.ReviewID, 
            r.ProducerID, 
            r.UserID, 
            r.Rating, 
            r.ReviewText,
            r.PhotoID,
            r.CreatedAt,
            p.CompanyName,
            p.DBA,
            p.City,
            p.State,
            p.Street,
            p.Zip
        FROM ProducerReviews r
        JOIN ProducerList_Prod p ON r.ProducerID = p.ProducerID
        ORDER BY r.CreatedAt DESC
        LIMIT :limit OFFSET :offset
    ";
    
    $stmt = $db->prepare($sql);
    if (!$stmt) {
        returnError("Failed to prepare statement: " . $db->lastErrorMsg());
    }
    
    $stmt->bindValue(':limit', $limit, SQLITE3_INTEGER);
    $stmt->bindValue(':offset', $offset, SQLITE3_INTEGER);
    
    $result = $stmt->execute();
    if (!$result) {
        returnError("Failed to execute query: " . $db->lastErrorMsg());
    }
    
    // Fetch review results
    $reviews = [];
    while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
        // Format any fields that need formatting
        $reviewId = $row['ReviewID'];
        
        // Query to get product reviews for this review
        $productSql = "
            SELECT 
                pr.ProductID,
                pr.Rating,
                pr.QuickTake,
                pr.PhotoID,
                p.Name,
                p.Category,
                p.Style
            FROM ProductReviews pr
            JOIN Products p ON pr.ProductID = p.ProductID
            WHERE pr.ReviewID = :reviewId
        ";
        
        $productStmt = $db->prepare($productSql);
        if (!$productStmt) {
            returnError("Failed to prepare product statement: " . $db->lastErrorMsg());
        }
        
        $productStmt->bindValue(':reviewId', $reviewId, SQLITE3_INTEGER);
        $productResult = $productStmt->execute();
        
        if (!$productResult) {
            returnError("Failed to execute product query: " . $db->lastErrorMsg());
        }
        
        // Fetch product results
        $products = [];
        while ($productRow = $productResult->fetchArray(SQLITE3_ASSOC)) {
            // Add photo path if it exists
            if (!empty($productRow['PhotoID'])) {
                // Check if it's already a full path
                if (stripos($productRow['PhotoID'], '/') === false) {
                    $productRow['PhotoID'] = '/uploads/photos/product/' . $productRow['PhotoID'];
                }
            }
            
            $products[] = $productRow;
        }
        
        // Add photo path if it exists
        if (!empty($row['PhotoID'])) {
            // Check if it's already a full path
            if (stripos($row['PhotoID'], '/') === false) {
                $row['PhotoID'] = '/uploads/photos/venue/' . $row['PhotoID'];
            }
        }
        
        // Add products to the review
        $row['Products'] = $products;
        $row['ProductCount'] = count($products);
        
        $reviews[] = $row;
    }
    
    // Return JSON response
    echo json_encode([
        'reviews' => $reviews,
        'count' => count($reviews),
        'page' => $page,
        'limit' => $limit,
        'hasMore' => count($reviews) >= $limit, // If we got as many as we asked for, there might be more
        'timestamp' => date('Y-m-d H:i:s')
    ], JSON_NUMERIC_CHECK);
    
    // Close database connection
    $db->close();
    
} catch (Exception $e) {
    returnError("Caught exception: " . $e->getMessage());
} catch (Error $e) {
    returnError("Caught error: " . $e->getMessage());
}
?>