<?php
// nearby-producers.php

// Set error handling to suppress HTML output
ini_set('display_errors', 0);
error_reporting(E_ALL);

// Function to handle errors as JSON
function returnError($message, $statusCode = 500)
{
    http_response_code($statusCode);
    header("Content-Type: application/json");
    echo json_encode(['error' => $message]);
    exit;
}

// Set headers before any output
header("Content-Type: application/json");
header("Access-Control-Allow-Origin: *");

// Use a try-catch for all code
try {
    // Get parameters
    $latitude = isset($_GET['latitude']) ? floatval($_GET['latitude']) : 0;
    $longitude = isset($_GET['longitude']) ? floatval($_GET['longitude']) : 0;
    $radius = isset($_GET['radius']) ? floatval($_GET['radius']) : 10;
    $query = isset($_GET['query']) ? $_GET['query'] : '';

    // Validate parameters
    if ($latitude == 0 || $longitude == 0) {
        returnError('Latitude and longitude are required', 400);
    }

    // Connect to SQLite database with correct path
    $dbPath = '/home/phantomales/TastingTrail/Producer.db';

    // Check if database file exists
    if (!file_exists($dbPath)) {
        returnError("Database file not found at: $dbPath");
    }

    // Connect to the database
    $db = new SQLite3($dbPath);

    if (!$db) {
        returnError("Failed to open database: " . SQLite3::lastErrorMsg());
    }

    // Create search term for LIKE query
    $searchTerm = '%' . SQLite3::escapeString($query) . '%';

    // Calculate distance using Haversine formula
    // Update field names to match new schema
    $sql = "
        SELECT *
        FROM (
            SELECT
                p.ProducerID, 
                p.CompanyName, 
                p.DBA, 
                p.Street, 
                p.City, 
                p.State, 
                p.Zip,
                p.GeoLat, 
                p.GeoLong,
                p.Phone,
                p.WebSite,
                p.HoursText AS Hours,
                p.TastingRoom,
                p.Tours,
                p.AppointmentRequired AS Appointments,
                p.Accessible,
                p.Social,
                (6371 * acos(cos(radians(:lat)) * cos(radians(p.GeoLat)) * cos(radians(p.GeoLong) - radians(:lon)) + sin(radians(:lat)) * sin(radians(p.GeoLat)))) AS distance
            FROM ProducerList_Prod p
            WHERE (p.CompanyName LIKE :search OR p.DBA LIKE :search OR :search = '%')
        ) AS subquery
        WHERE distance < :radius
        ORDER BY distance
        LIMIT 15
    ";

    // Prepare and execute the statement
    $stmt = $db->prepare($sql);
    if (!$stmt) {
        returnError("Failed to prepare statement: " . $db->lastErrorMsg());
    }

    $stmt->bindValue(':lat', $latitude, SQLITE3_FLOAT);
    $stmt->bindValue(':lon', $longitude, SQLITE3_FLOAT);
    $stmt->bindValue(':search', $searchTerm, SQLITE3_TEXT);
    $stmt->bindValue(':radius', $radius, SQLITE3_FLOAT);

    $result = $stmt->execute();
    if (!$result) {
        returnError("Failed to execute query: " . $db->lastErrorMsg());
    }

    // Fetch results
    $producers = [];
    while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
        // Process Social field safely
        $fb = '';
        $ig = '';
        $x = '';

        if (!empty($row['Social'])) {
            // Handle potential JSON issues in the Social field
            try {
                $social = json_decode($row['Social'], true);
                if (is_array($social)) {
                    $fb = isset($social['facebook']) ? $social['facebook'] : '';
                    $ig = isset($social['instagram']) ? $social['instagram'] : '';
                    $x = isset($social['twitter']) ? $social['twitter'] : '';
                }
            } catch (Exception $e) {
                // Silently handle JSON parsing errors
            }
        }

        // Add social media fields to output
        $row['FB'] = $fb;
        $row['IG'] = $ig;
        $row['X'] = $x;

        // Remove the Social field from output to avoid JSON issues
        unset($row['Social']);

        // Add rating if available
        $row['Rating'] = 0;

        try {
            $ratingsStmt = $db->prepare("SELECT AVG(AverageRating) as Rating FROM ProducerRatings WHERE ProducerID = :producerId");
            if ($ratingsStmt) {
                $ratingsStmt->bindValue(':producerId', $row['ProducerID'], SQLITE3_INTEGER);
                $ratingsResult = $ratingsStmt->execute();
                if ($ratingsResult) {
                    $ratingRow = $ratingsResult->fetchArray(SQLITE3_ASSOC);
                    if ($ratingRow && isset($ratingRow['Rating'])) {
                        $row['Rating'] = floatval($ratingRow['Rating']);
                    }
                }
            }
        } catch (Exception $e) {
            // Silently handle rating lookup errors
        }

        $producers[] = $row;
    }

    // Return JSON response
    echo json_encode(['producers' => $producers], JSON_NUMERIC_CHECK);

    // Close database connection
    $db->close();

} catch (Exception $e) {
    returnError("Caught exception: " . $e->getMessage());
} catch (Error $e) {
    returnError("Caught error: " . $e->getMessage());
}
?>