<?php
// producers-search.php - Updated version to handle all search types properly with new DB schema

// Set error handling
ini_set('display_errors', 0);
error_reporting(E_ALL);

// Function to handle errors as JSON
function returnError($message, $statusCode = 500)
{
    http_response_code($statusCode);
    header("Content-Type: application/json");
    echo json_encode(['error' => $message]);
    exit;
}

// Set headers before any output
header("Content-Type: application/json");
header("Access-Control-Allow-Origin: *");
header("Cache-Control: max-age=3600");

try {
    // Get parameters with isset checks
    $latitude = isset($_GET['latitude']) ? floatval($_GET['latitude']) : 0;
    $longitude = isset($_GET['longitude']) ? floatval($_GET['longitude']) : 0;
    $radius = isset($_GET['radius']) ? floatval($_GET['radius']) : 50;
    $query = isset($_GET['query']) ? $_GET['query'] : '';
    $nationwide = isset($_GET['nationwide']) && $_GET['nationwide'] === 'true';
    $state = isset($_GET['state']) ? strtoupper(trim($_GET['state'])) : '';

    // Validate parameters
    if ($latitude == 0 || $longitude == 0) {
        returnError('Latitude and longitude are required', 400);
    }

    // Cap radius for local searches
    if (!$nationwide && empty($state)) {
        $radius = min($radius, 160.934); // ~100 miles
    }

    // Database path
    $dbPath = '/home/phantomales/TastingTrail/Producer.db';
    if (!file_exists($dbPath)) {
        $dbPath = dirname(__DIR__) . '/Producer.db';
        if (!file_exists($dbPath)) {
            returnError("Database file not found", 500);
        }
    }

    // Connect to SQLite
    $db = new SQLite3($dbPath);
    if (!$db) {
        returnError("Failed to open database: " . SQLite3::lastErrorMsg());
    }

    // Prepare search term - default to wildcard if empty
    $searchTerm = !empty($query) ? '%' . SQLite3::escapeString($query) . '%' : '%';

    // Base query for all search types (always includes distance calculation)
    // Updated field names to match the new schema
    $baseQuery = "
        SELECT
            p.ProducerID,
            p.CompanyName,
            p.DBA,
            p.Street,
            p.City,
            p.State,
            p.Zip,
            p.GeoLat,
            p.GeoLong,
            p.Phone,
            p.WebSite,
            p.HoursText AS Hours,
            p.TastingRoom,
            p.Tours,
            p.AppointmentRequired AS Appointments,
            p.Accessible,
            p.Social,
            p.Type,
            p.FlightDetails,
            p.Description,
            (6371 * acos(cos(radians(:lat)) * cos(radians(p.GeoLat)) * cos(radians(p.GeoLong) - radians(:lon)) + sin(radians(:lat)) * sin(radians(p.GeoLat)))) AS distance
        FROM ProducerList_Prod p
        WHERE (p.CompanyName LIKE :search OR p.DBA LIKE :search OR :search = '%')
    ";

    // Add state filter if provided
    if (!empty($state)) {
        $baseQuery .= " AND p.State = :state";
    }

    // Complete query with different sorting based on search type
    if (!empty($query)) {
        // Name-based search (with or without location/state)
        if ($nationwide || !empty($state)) {
            // Nationwide or state search - no distance filter
            $sql = "
                SELECT * FROM ($baseQuery)
                ORDER BY 
                    CASE 
                        WHEN CompanyName = :exactMatch OR DBA = :exactMatch THEN 0
                        WHEN CompanyName LIKE :startMatch OR DBA LIKE :startMatch THEN 1
                        ELSE 2
                    END, 
                    distance 
                LIMIT 100
            ";
        } else {
            // Local name+location search - filter by distance
            $sql = "
                SELECT * FROM ($baseQuery)
                WHERE distance < :radius
                ORDER BY 
                    CASE 
                        WHEN CompanyName = :exactMatch OR DBA = :exactMatch THEN 0
                        WHEN CompanyName LIKE :startMatch OR DBA LIKE :startMatch THEN 1
                        ELSE 2
                    END, 
                    distance 
                LIMIT 100
            ";
        }
    } else {
        // Location-only search - simple radius filter
        $sql = "
            SELECT * FROM ($baseQuery)
            WHERE distance < :radius
            ORDER BY distance 
            LIMIT 50
        ";
    }

    // Prepare statement
    $stmt = $db->prepare($sql);
    if (!$stmt) {
        returnError("Failed to prepare statement: " . $db->lastErrorMsg());
    }

    // Bind parameters that are used in all queries
    $stmt->bindValue(':lat', $latitude, SQLITE3_FLOAT);
    $stmt->bindValue(':lon', $longitude, SQLITE3_FLOAT);
    $stmt->bindValue(':search', $searchTerm, SQLITE3_TEXT);

    // Bind radius for queries that use it
    if (!$nationwide && empty($state)) {
        $stmt->bindValue(':radius', $radius, SQLITE3_FLOAT);
    } else if (!empty($query) && !$nationwide && !empty($state)) {
        // This is for state search with name that still uses radius
        $stmt->bindValue(':radius', 1000, SQLITE3_FLOAT); // Just use a very large radius
    }

    // Bind parameters for name searches
    if (!empty($query)) {
        $exactMatch = SQLite3::escapeString($query);
        $startMatch = SQLite3::escapeString($query) . '%';
        $stmt->bindValue(':exactMatch', $exactMatch, SQLITE3_TEXT);
        $stmt->bindValue(':startMatch', $startMatch, SQLITE3_TEXT);
    }

    // Bind state parameter for state-based searches
    if (!empty($state)) {
        $stmt->bindValue(':state', $state, SQLITE3_TEXT);
    }

    // Execute query
    $result = $stmt->execute();
    if (!$result) {
        returnError("Failed to execute query: " . $db->lastErrorMsg());
    }

    // Fetch results
    $producers = [];
    while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
        // Extract social media information from the JSON in Social field
        $fb = '';
        $ig = '';
        $x = '';

        if (!empty($row['Social'])) {
            // Handle potential JSON issues in the Social field
            try {
                $social = json_decode($row['Social'], true);
                if (is_array($social)) {
                    $fb = isset($social['facebook']) ? $social['facebook'] : '';
                    $ig = isset($social['instagram']) ? $social['instagram'] : '';
                    $x = isset($social['twitter']) ? $social['twitter'] : '';
                }
            } catch (Exception $e) {
                // Silently handle JSON parsing errors
            }
        }

        // Add social media fields to output and remove Social field
        $row['FB'] = $fb;
        $row['IG'] = $ig;
        $row['X'] = $x;
        unset($row['Social']);

        // Ensure numeric types
        $row['ProducerID'] = intval($row['ProducerID']);
        $row['GeoLat'] = floatval($row['GeoLat']);
        $row['GeoLong'] = floatval($row['GeoLong']);
        $row['distance'] = floatval($row['distance']);

        // Get average rating from ProducerRatings
        $row['Rating'] = 0;
        $row['ReviewCount'] = 0;

        try {
            $ratingsStmt = $db->prepare("
                SELECT AVG(AverageRating) as Rating, SUM(ReviewCount) as ReviewCount 
                FROM ProducerRatings 
                WHERE ProducerID = :producerId
            ");

            if ($ratingsStmt) {
                $ratingsStmt->bindValue(':producerId', $row['ProducerID'], SQLITE3_INTEGER);
                $ratingsResult = $ratingsStmt->execute();

                if ($ratingsResult) {
                    $ratingsRow = $ratingsResult->fetchArray(SQLITE3_ASSOC);
                    if ($ratingsRow) {
                        $row['Rating'] = isset($ratingsRow['Rating']) ? floatval($ratingsRow['Rating']) : 0;
                        $row['ReviewCount'] = isset($ratingsRow['ReviewCount']) ? intval($ratingsRow['ReviewCount']) : 0;
                    }
                }
            }
        } catch (Exception $e) {
            // Silently handle rating lookup errors
        }

        // Get top products for this producer (limit to 3)
        $row['TopProducts'] = [];

        try {
            $productsStmt = $db->prepare("
                SELECT p.ProductID, p.Name, p.Category, p.Style, AVG(pr.Rating) as Rating, COUNT(pr.Rating) as ReviewCount
                FROM Products p
                LEFT JOIN ProductReviews pr ON p.ProductID = pr.ProductID
                WHERE p.ProducerID = :producerId AND p.IsActive = 1
                GROUP BY p.ProductID
                ORDER BY Rating DESC, ReviewCount DESC
                LIMIT 3
            ");

            if ($productsStmt) {
                $productsStmt->bindValue(':producerId', $row['ProducerID'], SQLITE3_INTEGER);
                $productsResult = $productsStmt->execute();

                if ($productsResult) {
                    while ($productRow = $productsResult->fetchArray(SQLITE3_ASSOC)) {
                        // Ensure numeric types for product data
                        $productRow['ProductID'] = intval($productRow['ProductID']);
                        $productRow['Rating'] = floatval($productRow['Rating']);
                        $productRow['ReviewCount'] = intval($productRow['ReviewCount']);

                        $row['TopProducts'][] = $productRow;
                    }
                }
            }
        } catch (Exception $e) {
            // Silently handle product lookup errors
        }

        $row['distanceMiles'] = round($row['distance'] / 1.60934, 1);
        $producers[] = $row;
    }

    // Return JSON with numeric check option
    echo json_encode(['producers' => $producers], JSON_NUMERIC_CHECK);

    // Close database
    $db->close();

} catch (Exception $e) {
    returnError("Caught exception: " . $e->getMessage());
} catch (Error $e) {
    returnError("Caught error: " . $e->getMessage());
}
?>