<?php
// search-products.php - Searches for products from a specific producer

// Set error handling to suppress HTML output
ini_set('display_errors', 0);
error_reporting(E_ALL);

// Function to handle errors as JSON
function returnError($message, $statusCode = 500) {
    http_response_code($statusCode);
    header("Content-Type: application/json");
    echo json_encode(['error' => $message]);
    exit;
}

// Set headers before any output
header("Content-Type: application/json");
header("Access-Control-Allow-Origin: *");

// Use a try-catch for all code
try {
    // Get parameters with isset checks
    $producerId = isset($_GET['producerId']) ? intval($_GET['producerId']) : 0;
    $query = isset($_GET['query']) ? $_GET['query'] : '';

    // Validate parameters
    if ($producerId == 0) {
        returnError('Producer ID is required', 400);
    }

    // Connect to SQLite database with correct path
    $dbPath = '/home/phantomales/TastingTrail/Producer.db';
    
    // Check if database file exists
    if (!file_exists($dbPath)) {
        returnError("Database file not found at: $dbPath");
    }
    
    // Connect to the database
    $db = new SQLite3($dbPath);
    
    if (!$db) {
        returnError("Failed to open database: " . SQLite3::lastErrorMsg());
    }
    
    // Create search term for LIKE query
    $searchTerm = '%' . SQLite3::escapeString($query) . '%';
    
    // Query to find products for this producer with ratings
    $sql = "
        SELECT 
            p.ProductID,
            p.ProducerID,
            p.Name,
            p.Category,
            p.Style,
            p.ABV,
            p.Description,
            p.IsActive,
            COALESCE(AVG(pr.Rating), 0) AS AverageRating,
            COUNT(pr.Rating) AS ReviewCount
        FROM Products p
        LEFT JOIN ProductReviews pr ON p.ProductID = pr.ProductID
        WHERE p.ProducerID = :producerId
        AND (p.Name LIKE :search OR p.Category LIKE :search OR p.Style LIKE :search OR :search = '')
        AND p.IsActive = 1
        GROUP BY p.ProductID
        ORDER BY p.Name
    ";
    
    // Prepare and execute the statement
    $stmt = $db->prepare($sql);
    if (!$stmt) {
        returnError("Failed to prepare statement: " . $db->lastErrorMsg());
    }
    
    $stmt->bindValue(':producerId', $producerId, SQLITE3_INTEGER);
    $stmt->bindValue(':search', $searchTerm, SQLITE3_TEXT);
    
    $result = $stmt->execute();
    if (!$result) {
        returnError("Failed to execute query: " . $db->lastErrorMsg());
    }
    
    // Fetch results
    $products = [];
    while ($row = $result->fetchArray(SQLITE3_ASSOC)) {
        // Ensure numeric values are properly typed
        $row['ProductID'] = intval($row['ProductID']);
        $row['ProducerID'] = intval($row['ProducerID']);
        $row['ABV'] = floatval($row['ABV']);
        $row['AverageRating'] = floatval($row['AverageRating']);
        $row['ReviewCount'] = intval($row['ReviewCount']);
        $row['IsActive'] = intval($row['IsActive']);
        
        $products[] = $row;
    }
    
    // Return JSON response with numeric check option
    echo json_encode(['products' => $products], JSON_NUMERIC_CHECK);
    
    // Close database connection
    $db->close();
    
} catch (Exception $e) {
    returnError("Caught exception: " . $e->getMessage());
} catch (Error $e) {
    returnError("Caught error: " . $e->getMessage());
}
?>