<?php
// submit-review.php - Handles review submission and stores in database
// Updated for new database schema

// Set error handling to suppress HTML output
ini_set('display_errors', 0);
error_reporting(E_ALL);

// Function to handle errors as JSON
function returnError($message, $statusCode = 500) {
    http_response_code($statusCode);
    header("Content-Type: application/json");
    echo json_encode(['error' => $message]);
    exit;
}

// Function to return success response
function returnSuccess($data = []) {
    header("Content-Type: application/json");
    echo json_encode(array_merge(['success' => true], $data), JSON_NUMERIC_CHECK);
    exit;
}

// Function to get the current user ID
// This will be enhanced later when user authentication is implemented
function getCurrentUserId() {
    // In the future, this will check for logged-in users
    // For now, return a default guest ID
    return 1; // Default guest user ID
}

// Only accept POST requests
if ($_SERVER['REQUEST_METHOD'] !== 'POST') {
    returnError('Only POST method is allowed', 405);
}

// Set headers
header("Content-Type: application/json");
header("Access-Control-Allow-Origin: *");

// Use a try-catch for all code
try {
    // Connect to SQLite database
    $dbPath = '/home/phantomales/TastingTrail/Producer.db';
    
    // Check if database file exists
    if (!file_exists($dbPath)) {
        returnError("Database file not found at: $dbPath");
    }
    
    // Connect to the database
    $db = new SQLite3($dbPath);
    
    if (!$db) {
        returnError("Failed to open database: " . SQLite3::lastErrorMsg());
    }
    
    // Get POST data
    $producerId = isset($_POST['producerId']) ? intval($_POST['producerId']) : 0;
    $venueName = isset($_POST['venueName']) ? $_POST['venueName'] : '';
    $venueAddress = isset($_POST['venueAddress']) ? $_POST['venueAddress'] : '';
    $latitude = isset($_POST['latitude']) ? floatval($_POST['latitude']) : 0;
    $longitude = isset($_POST['longitude']) ? floatval($_POST['longitude']) : 0;
    $isNewVenue = isset($_POST['isNewVenue']) ? ($_POST['isNewVenue'] === 'true' || $_POST['isNewVenue'] === '1') : false;
    
    $rating = isset($_POST['rating']) ? floatval($_POST['rating']) : 0;
    $review = isset($_POST['review']) ? $_POST['review'] : '';
    
    $productCount = isset($_POST['productCount']) ? intval($_POST['productCount']) : 0;
    
    // Get the current user ID
    $userId = getCurrentUserId();
    
    // Validate required fields
    if ($venueName === '') {
        returnError('Venue name is required', 400);
    }
    
    if ($latitude === 0 || $longitude === 0) {
        returnError('Location coordinates are required', 400);
    }
    
    if ($rating <= 0 || $rating > 5) {
        returnError('Rating must be between 1 and 5', 400);
    }
    
    // Start transaction
    $db->exec('BEGIN TRANSACTION');
    
    try {
        // If it's a new venue, insert it into the producer table
        if ($isNewVenue) {
            // Current timestamp for created/updated fields
            $now = date('Y-m-d H:i:s');
            
            // Create default social media JSON
            $socialJson = json_encode([
                'facebook' => '',
                'instagram' => '',
                'twitter' => ''
            ]);
            
            // Insert into producer list - modify fields to match new schema
            $sql = "
                INSERT INTO ProducerList_Prod (
                    CompanyName, 
                    DBA, 
                    Street, 
                    GeoLat, 
                    GeoLong,
                    Social,
                    AppointmentRequired,
                    TastingRoom,
                    CreatedAt,
                    UpdatedAt
                ) VALUES (
                    :companyName, 
                    :dba, 
                    :street, 
                    :geoLat, 
                    :geoLong,
                    :social, 
                    'N',
                    'Y',
                    :createdAt,
                    :updatedAt
                )
            ";
            
            $stmt = $db->prepare($sql);
            if (!$stmt) {
                throw new Exception("Failed to prepare statement: " . $db->lastErrorMsg());
            }
            
            $stmt->bindValue(':companyName', $venueName, SQLITE3_TEXT);
            $stmt->bindValue(':dba', $venueName, SQLITE3_TEXT); // Use same name for DBA
            $stmt->bindValue(':street', $venueAddress, SQLITE3_TEXT);
            $stmt->bindValue(':geoLat', $latitude, SQLITE3_FLOAT);
            $stmt->bindValue(':geoLong', $longitude, SQLITE3_FLOAT);
            $stmt->bindValue(':social', $socialJson, SQLITE3_TEXT);
            $stmt->bindValue(':createdAt', $now, SQLITE3_TEXT);
            $stmt->bindValue(':updatedAt', $now, SQLITE3_TEXT);
            
            $result = $stmt->execute();
            if (!$result) {
                throw new Exception("Failed to insert producer: " . $db->lastErrorMsg());
            }
            
            // Get the new producer ID
            $producerId = $db->lastInsertRowID();
        }
        
        // Save the review in the ProducerReviews table (new schema)
        $now = date('Y-m-d H:i:s');
        $sql = "
            INSERT INTO ProducerReviews (
                ProducerID, 
                UserID, 
                Rating, 
                ReviewText, 
                CreatedAt,
                UpdatedAt
            ) VALUES (
                :producerId, 
                :userId, 
                :rating, 
                :review, 
                :createdAt,
                :updatedAt
            )
        ";
        
        $stmt = $db->prepare($sql);
        if (!$stmt) {
            throw new Exception("Failed to prepare statement: " . $db->lastErrorMsg());
        }
        
        $stmt->bindValue(':producerId', $producerId, SQLITE3_INTEGER);
        $stmt->bindValue(':userId', $userId, SQLITE3_INTEGER); // Use the current user ID
        $stmt->bindValue(':rating', $rating, SQLITE3_FLOAT);
        $stmt->bindValue(':review', $review, SQLITE3_TEXT);
        $stmt->bindValue(':createdAt', $now, SQLITE3_TEXT);
        $stmt->bindValue(':updatedAt', $now, SQLITE3_TEXT);
        
        $result = $stmt->execute();
        if (!$result) {
            throw new Exception("Failed to insert review: " . $db->lastErrorMsg());
        }
        
        $reviewId = $db->lastInsertRowID();
        
        // Update or create entry in ProducerRatings table
        $sql = "
            SELECT RatingID, AverageRating, ReviewCount FROM ProducerRatings 
            WHERE ProducerID = :producerId AND Source = 'user'
        ";
        $stmt = $db->prepare($sql);
        $stmt->bindValue(':producerId', $producerId, SQLITE3_INTEGER);
        $result = $stmt->execute();
        $ratingRow = $result->fetchArray(SQLITE3_ASSOC);
        
        if ($ratingRow) {
            // Update existing rating
            $newCount = $ratingRow['ReviewCount'] + 1;
            $newAvg = (($ratingRow['AverageRating'] * $ratingRow['ReviewCount']) + $rating) / $newCount;
            
            $sql = "
                UPDATE ProducerRatings
                SET AverageRating = :avgRating, ReviewCount = :reviewCount, LastUpdated = :lastUpdated
                WHERE RatingID = :ratingId
            ";
            $stmt = $db->prepare($sql);
            $stmt->bindValue(':avgRating', $newAvg, SQLITE3_FLOAT);
            $stmt->bindValue(':reviewCount', $newCount, SQLITE3_INTEGER);
            $stmt->bindValue(':lastUpdated', $now, SQLITE3_TEXT);
            $stmt->bindValue(':ratingId', $ratingRow['RatingID'], SQLITE3_INTEGER);
        } else {
            // Insert new rating
            $sql = "
                INSERT INTO ProducerRatings (
                    ProducerID, Source, AverageRating, ReviewCount, LastUpdated
                ) VALUES (
                    :producerId, 'user', :avgRating, 1, :lastUpdated
                )
            ";
            $stmt = $db->prepare($sql);
            $stmt->bindValue(':producerId', $producerId, SQLITE3_INTEGER);
            $stmt->bindValue(':avgRating', $rating, SQLITE3_FLOAT);
            $stmt->bindValue(':lastUpdated', $now, SQLITE3_TEXT);
        }
        
        $result = $stmt->execute();
        if (!$result) {
            throw new Exception("Failed to update ratings: " . $db->lastErrorMsg());
        }
        
        // Handle establishment photo - store PhotoID in reviews
        if (isset($_FILES['establishmentPhoto']) && $_FILES['establishmentPhoto']['error'] === UPLOAD_ERR_OK) {
            $photoFileName = 'venue_' . $producerId . '_' . time() . '.jpg';
            $uploadDir = 'uploads/photos/venue/';
            $fullPath = $uploadDir . $photoFileName;
            
            // Create directory if it doesn't exist
            if (!file_exists($uploadDir)) {
                mkdir($uploadDir, 0755, true);
            }
            
            // Move uploaded file
            if (move_uploaded_file($_FILES['establishmentPhoto']['tmp_name'], $fullPath)) {
                // Save photo ID in the review (use the file path as the ID)
                $sql = "UPDATE ProducerReviews SET PhotoID = :photoId WHERE ReviewID = :reviewId";
                $stmt = $db->prepare($sql);
                $stmt->bindValue(':photoId', $fullPath, SQLITE3_TEXT);
                $stmt->bindValue(':reviewId', $reviewId, SQLITE3_INTEGER);
                $stmt->execute();
            }
        }
        
        // Process each product review
        if ($productCount > 0 && isset($_POST['product'])) {
            foreach ($_POST['product'] as $index => $product) {
                $productId = isset($product['id']) ? intval($product['id']) : 0;
                $productName = isset($product['name']) ? $product['name'] : '';
                $category = isset($product['category']) ? $product['category'] : '';
                $style = isset($product['style']) ? $product['style'] : '';
                $productRating = isset($product['rating']) ? floatval($product['rating']) : 0;
                $quickTake = isset($product['quickTake']) ? $product['quickTake'] : '';
                $isNewProduct = isset($product['isNewProduct']) && ($product['isNewProduct'] === 'true' || $product['isNewProduct'] === '1');
                
                // If it's a new product, insert it into the Products table
                if ($isNewProduct) {
                    $sql = "
                        INSERT INTO Products (
                            ProducerID, Name, Category, Style, IsActive, CreatedAt, UpdatedAt
                        ) VALUES (
                            :producerId, :name, :category, :style, 1, :createdAt, :updatedAt
                        )
                    ";
                    
                    $stmt = $db->prepare($sql);
                    $stmt->bindValue(':producerId', $producerId, SQLITE3_INTEGER);
                    $stmt->bindValue(':name', $productName, SQLITE3_TEXT);
                    $stmt->bindValue(':category', $category, SQLITE3_TEXT);
                    $stmt->bindValue(':style', $style, SQLITE3_TEXT);
                    $stmt->bindValue(':createdAt', $now, SQLITE3_TEXT);
                    $stmt->bindValue(':updatedAt', $now, SQLITE3_TEXT);
                    
                    $stmt->execute();
                    $productId = $db->lastInsertRowID();
                }
                
                // Insert product review into ProductReviews table
                $sql = "
                    INSERT INTO ProductReviews (
                        ReviewID, ProductID, Rating, QuickTake, UserID
                    ) VALUES (
                        :reviewId, :productId, :rating, :quickTake, :userId
                    )
                ";
                
                $stmt = $db->prepare($sql);
                $stmt->bindValue(':reviewId', $reviewId, SQLITE3_INTEGER);
                $stmt->bindValue(':productId', $productId, SQLITE3_INTEGER);
                $stmt->bindValue(':rating', $productRating, SQLITE3_FLOAT);
                $stmt->bindValue(':quickTake', $quickTake, SQLITE3_TEXT);
                $stmt->bindValue(':userId', $userId, SQLITE3_INTEGER); // Add UserID
                
                $stmt->execute();
                $productReviewId = $db->lastInsertRowID();
                
                // Handle product photo
                if (isset($_FILES["product{$index}photo"]) && $_FILES["product{$index}photo"]['error'] === UPLOAD_ERR_OK) {
                    $photoFileName = 'product_' . $productId . '_' . time() . '.jpg';
                    $uploadDir = 'uploads/photos/product/';
                    $fullPath = $uploadDir . $photoFileName;
                    
                    // Create directory if it doesn't exist
                    if (!file_exists($uploadDir)) {
                        mkdir($uploadDir, 0755, true);
                    }
                    
                    // Move uploaded file
                    if (move_uploaded_file($_FILES["product{$index}photo"]['tmp_name'], $fullPath)) {
                        // Just log the file path for now
                        error_log("Saved product photo: $fullPath for product ID: $productId");
                        
                        // In the future, you might want to store this path in a column
                        // For example, if you add a PhotoPath column to Products table:
                        /*
                        $sql = "
                            UPDATE Products 
                            SET PhotoPath = :photoPath, UpdatedAt = :updatedAt
                            WHERE ProductID = :productId
                        ";
                        
                        $stmt = $db->prepare($sql);
                        $stmt->bindValue(':photoPath', $fullPath, SQLITE3_TEXT);
                        $stmt->bindValue(':updatedAt', $now, SQLITE3_TEXT);
                        $stmt->bindValue(':productId', $productId, SQLITE3_INTEGER);
                        $stmt->execute();
                        */
                    }
                }
            }
        }
        
        // Commit the transaction
        $db->exec('COMMIT');
        
        // Return success response
        returnSuccess([
            'reviewId' => $reviewId,
            'timestamp' => $now,
            'producerId' => $producerId,
            'productCount' => $productCount
        ]);
        
    } catch (Exception $e) {
        // Rollback the transaction on error
        $db->exec('ROLLBACK');
        throw $e;
    }
    
} catch (Exception $e) {
    returnError("Caught exception: " . $e->getMessage());
} catch (Error $e) {
    returnError("Caught error: " . $e->getMessage());
}
?>