﻿// Main app.js file for Craft Review PWA

// Initialize the application when DOM is fully loaded
document.addEventListener('DOMContentLoaded', initApp);

function initApp() {
    // Set up event listeners
    setupFormListeners();
    setupStylesBasedOnCategory();

    // Check if we're online and update UI accordingly
    updateOnlineStatus();
    window.addEventListener('online', updateOnlineStatus);
    window.addEventListener('offline', updateOnlineStatus);
}

function setupFormListeners() {
    const reviewForm = document.getElementById('reviewForm');

    if (reviewForm) {
        reviewForm.addEventListener('submit', (e) => {
            e.preventDefault();

            // Validate the form
            if (validateForm()) {
                // Submit the form data
                submitReview();
            }
        });
    }
}

function validateForm() {
    // Get required fields for establishment
    const locationInput = document.getElementById('locationSearch');
    const establishmentPhoto = document.getElementById('establishmentPhoto');
    const establishmentReview = document.getElementById('establishmentReview');
    const establishmentRating = document.getElementById('establishmentRating');

    // Validation flags
    let isValid = true;
    let errorMessage = '';

    // Check location
    if (!locationInput.value.trim() || !document.getElementById('latitude').value) {
        isValid = false;
        errorMessage += 'Please select a valid location\n';
        locationInput.classList.add('error');
    } else {
        locationInput.classList.remove('error');
    }

    // Check establishment photo
    if (!establishmentPhoto.files || establishmentPhoto.files.length === 0) {
        isValid = false;
        errorMessage += 'Please add an establishment photo\n';
        document.getElementById('establishmentPhotoUpload').classList.add('error');
    } else {
        document.getElementById('establishmentPhotoUpload').classList.remove('error');
    }

    // Check establishment review
    if (!establishmentReview.value.trim()) {
        isValid = false;
        errorMessage += 'Please enter a brief review of the establishment\n';
        establishmentReview.classList.add('error');
    } else {
        establishmentReview.classList.remove('error');
    }

    // Get the number of products
    const tastingCount = parseInt(document.getElementById('tastingCount').value);

    // Validate each product
    for (let i = 1; i <= tastingCount; i++) {
        const productName = document.getElementById(`productName${i}`);
        const productPhoto = document.getElementById(`productPhoto${i}`);
        const productRating = document.getElementById(`productRating${i}`);

        // Check product name
        if (!productName.value.trim()) {
            isValid = false;
            errorMessage += `Please enter a name for Product ${i}\n`;
            productName.classList.add('error');
        } else {
            productName.classList.remove('error');
        }

        // Check product photo (optional but encouraged)
        if (!productPhoto.files || productPhoto.files.length === 0) {
            // Just add a warning class instead of error
            document.getElementById(`productPhotoUpload${i}`).classList.add('warning');
        } else {
            document.getElementById(`productPhotoUpload${i}`).classList.remove('warning');
        }
    }

    // Display error message if validation fails
    if (!isValid) {
        alert('Please fix the following errors:\n' + errorMessage);
    }

    return isValid;
}

async function submitReview() {
    // Show loading state
    const submitButton = document.getElementById('submitReview');
    const originalButtonText = submitButton.textContent;
    submitButton.textContent = 'Submitting...';
    submitButton.disabled = true;

    try {
        // Get the number of products
        const tastingCount = parseInt(document.getElementById('tastingCount').value);

        // Collect product data
        const products = [];
        for (let i = 1; i <= tastingCount; i++) {
            const productPhoto = document.getElementById(`productPhoto${i}`).files[0];

            products.push({
                name: document.getElementById(`productName${i}`).value,
                category: document.getElementById(`category${i}`).value,
                style: document.getElementById(`style${i}`).value,
                rating: parseFloat(document.getElementById(`productRating${i}`).value),
                quickTake: document.getElementById(`quickTake${i}`).value,
                photoId: productPhoto ? await savePhoto(productPhoto, `product${i}`) : null
            });
        }

        // Gather form data
        const formData = {
            establishment: {
                location: {
                    name: document.getElementById('locationSearch').value,
                    latitude: document.getElementById('latitude').value,
                    longitude: document.getElementById('longitude').value
                },
                review: document.getElementById('establishmentReview').value,
                rating: parseFloat(document.getElementById('establishmentRating').value),
                photoId: await savePhoto(document.getElementById('establishmentPhoto').files[0], 'establishment')
            },
            products: products,
            reviewDate: new Date().toISOString(),
            userId: 'current-user-id' // This would be replaced with actual auth
        };

        // Save review data
        const reviewId = await saveReview(formData);

        // Show success message
        alert('Your review has been submitted successfully!');

        // Reset the form
        document.getElementById('reviewForm').reset();
        resetPhotoPreviewsAndCounts();

        // Redirect to the review detail page (when implemented)
        // window.location.href = `/review/${reviewId}`;
    } catch (error) {
        console.error('Error submitting review:', error);
        alert('There was an error submitting your review. Please try again.');
    } finally {
        // Reset button state
        submitButton.textContent = originalButtonText;
        submitButton.disabled = false;
    }
}

function resetPhotoPreviewsAndCounts() {
    // Reset establishment elements
    document.getElementById('establishmentPhotoPreview').innerHTML = '';
    document.getElementById('establishmentReviewCount').textContent = '0';
    document.getElementById('establishmentRatingValue').textContent = '3.0';
    document.getElementById('selectedLocation').textContent = '';

    // Reset all product elements
    const productContainer = document.getElementById('productReviewsContainer');
    const productForms = productContainer.querySelectorAll('.product-review');

    // Remove all but the first product form
    for (let i = 1; i < productForms.length; i++) {
        productContainer.removeChild(productForms[i]);
    }

    // Reset first product form
    const firstProductForm = productForms[0];
    if (firstProductForm) {
        firstProductForm.querySelector('.product-name').value = '';
        firstProductForm.querySelector('.product-category').value = '';
        firstProductForm.querySelector('.product-style').innerHTML = '<option value="">Select a style</option>';
        firstProductForm.querySelector('.product-rating').value = '3';
        firstProductForm.querySelector('#productRatingValue1').textContent = '3.0';
        firstProductForm.querySelector('.product-quick-take').value = '';
        firstProductForm.querySelector('#quickTakeCount1').textContent = '0';
        firstProductForm.querySelector(`#productPhotoPreview1`).innerHTML = '';
    }

    // Reset tasting count to 1
    document.getElementById('tastingCount').value = '1';
}

function setupStylesBasedOnCategory() {
    // Set up for first product initially
    setupCategoryListener(1);
}

function setupCategoryListener(index) {
    const categorySelect = document.getElementById(`category${index}`);
    const styleSelect = document.getElementById(`style${index}`);

    if (categorySelect && styleSelect) {
        // Define styles for each category
        const styleOptions = {
            'Beer': ['IPA', 'Stout', 'Pilsner', 'Lager', 'Wheat Beer', 'Sour', 'Porter', 'Pale Ale', 'Other'],
            'Wine': ['Red', 'White', 'Rosé', 'Sparkling', 'Dessert', 'Fortified', 'Orange', 'Natural', 'Other'],
            'Mead': ['Traditional', 'Melomel', 'Metheglin', 'Braggot', 'Cyser', 'Pyment', 'Other'],
            'Cider': ['Dry', 'Semi-Dry', 'Sweet', 'Perry', 'Fruit', 'Hopped', 'Spiced', 'Other'],
            'Distilled Spirit': ['Whiskey', 'Bourbon', 'Rum', 'Vodka', 'Gin', 'Tequila', 'Brandy', 'Liqueur', 'Other'],
            'Seltzer': ['Fruit', 'Berry', 'Citrus', 'Tropical', 'Herbal', 'Unflavored', 'Other'],
            'Kombucha': ['Original', 'Fruit', 'Flower', 'Herb', 'Spiced', 'Other'],
            'Other': ['Non-Alcoholic', 'Coffee', 'Tea', 'Mixed Drink', 'Cocktail', 'Other']
        };

        // Update styles when category changes
        categorySelect.addEventListener('change', () => {
            // Clear existing options
            styleSelect.innerHTML = '<option value="">Select a style</option>';

            // Get selected category
            const selectedCategory = categorySelect.value;

            // Add new options if category is selected
            if (selectedCategory && styleOptions[selectedCategory]) {
                styleOptions[selectedCategory].forEach(style => {
                    const option = document.createElement('option');
                    option.value = style;
                    option.textContent = style;
                    styleSelect.appendChild(option);
                });
            }
        });
    }
}

function updateOnlineStatus() {
    const status = navigator.onLine ? 'online' : 'offline';

    if (status === 'offline') {
        // Display offline notification
        showNotification('You are currently offline. Your reviews will be saved and uploaded when you reconnect.', 'warning');
    } else {
        // Check for pending uploads when back online
        checkPendingUploads();
    }
}

function showNotification(message, type = 'info') {
    // Create notification element if it doesn't exist
    let notification = document.querySelector('.notification');

    if (!notification) {
        notification = document.createElement('div');
        notification.className = 'notification';
        document.body.appendChild(notification);
    }

    // Set notification type class
    notification.className = `notification ${type}`;
    notification.textContent = message;

    // Show notification
    notification.style.display = 'block';

    // Auto hide after 5 seconds
    setTimeout(() => {
        notification.style.display = 'none';
    }, 5000);
}

// This function would check for any reviews saved while offline
function checkPendingUploads() {
    // This would be implemented with the DB functionality
    // to sync any pending reviews with the server
    console.log('Checking for pending uploads...');

    // Example implementation:
    // getPendingReviews().then(reviews => {
    //     if (reviews.length > 0) {
    //         showNotification(`Uploading ${reviews.length} pending reviews...`, 'info');
    //         uploadPendingReviews(reviews);
    //     }
    // });
}