// db.js - Handles data storage and retrieval

// Initialize IndexedDB for local storage
let db;
const DB_NAME = 'craft_review_db';
const DB_VERSION = 1;
const REVIEWS_STORE = 'reviews';
const PENDING_STORE = 'pending_uploads';

// Initialize the database when the page loads
document.addEventListener('DOMContentLoaded', initDatabase);

function initDatabase() {
    const request = indexedDB.open(DB_NAME, DB_VERSION);

    request.onerror = (event) => {
        console.error('Database error:', event.target.error);
        showNotification('Could not open local database. Some features may not work properly.', 'error');
    };

    request.onsuccess = (event) => {
        db = event.target.result;
        console.log('Database opened successfully');
    };

    request.onupgradeneeded = (event) => {
        const db = event.target.result;

        // Create object stores
        if (!db.objectStoreNames.contains(REVIEWS_STORE)) {
            const reviewsStore = db.createObjectStore(REVIEWS_STORE, { keyPath: 'id', autoIncrement: true });
            reviewsStore.createIndex('userId', 'userId', { unique: false });
            reviewsStore.createIndex('locationId', 'location.id', { unique: false });
            reviewsStore.createIndex('reviewDate', 'reviewDate', { unique: false });
        }

        if (!db.objectStoreNames.contains(PENDING_STORE)) {
            const pendingStore = db.createObjectStore(PENDING_STORE, { keyPath: 'id', autoIncrement: true });
            pendingStore.createIndex('timestamp', 'timestamp', { unique: false });
        }
    };
}

// Function to save a review locally
async function saveReview(reviewData) {
    return new Promise((resolve, reject) => {
        if (!db) {
            reject(new Error('Database not initialized'));
            return;
        }

        // Generate a unique ID for the review
        reviewData.id = `review-${Date.now()}-${Math.floor(Math.random() * 10000)}`;

        // Add timestamp for when this review was created
        reviewData.timestamp = Date.now();

        try {
            const transaction = db.transaction([REVIEWS_STORE, PENDING_STORE], 'readwrite');
            const reviewsStore = transaction.objectStore(REVIEWS_STORE);
            const pendingStore = transaction.objectStore(PENDING_STORE);

            // Add to reviews store
            const reviewRequest = reviewsStore.add(reviewData);

            reviewRequest.onsuccess = () => {
                console.log('Review saved locally');

                // If we're offline, add to pending uploads
                if (!navigator.onLine) {
                    pendingStore.add({
                        type: 'review',
                        data: reviewData,
                        timestamp: Date.now()
                    });
                    console.log('Review added to pending uploads queue');
                } else {
                    // If we're online, simulate sending to server
                    simulateServerUpload(reviewData)
                        .then(() => console.log('Review uploaded to server'))
                        .catch(err => {
                            console.error('Server upload failed:', err);
                            // Add to pending uploads if server upload fails
                            pendingStore.add({
                                type: 'review',
                                data: reviewData,
                                timestamp: Date.now()
                            });
                        });
                }

                resolve(reviewData.id);
            };

            reviewRequest.onerror = (event) => {
                console.error('Error saving review:', event.target.error);
                reject(event.target.error);
            };

            transaction.oncomplete = () => {
                console.log('Transaction completed');
            };

            transaction.onerror = (event) => {
                console.error('Transaction error:', event.target.error);
                reject(event.target.error);
            };
        } catch (error) {
            console.error('Error in transaction:', error);
            reject(error);
        }
    });
}

// Function to get all reviews
async function getReviews() {
    return new Promise((resolve, reject) => {
        if (!db) {
            reject(new Error('Database not initialized'));
            return;
        }

        try {
            const transaction = db.transaction(REVIEWS_STORE, 'readonly');
            const reviewsStore = transaction.objectStore(REVIEWS_STORE);
            const request = reviewsStore.getAll();

            request.onsuccess = () => {
                resolve(request.result);
            };

            request.onerror = (event) => {
                console.error('Error getting reviews:', event.target.error);
                reject(event.target.error);
            };
        } catch (error) {
            console.error('Error in transaction:', error);
            reject(error);
        }
    });
}

// Function to get a specific review
async function getReview(reviewId) {
    return new Promise((resolve, reject) => {
        if (!db) {
            reject(new Error('Database not initialized'));
            return;
        }

        try {
            const transaction = db.transaction(REVIEWS_STORE, 'readonly');
            const reviewsStore = transaction.objectStore(REVIEWS_STORE);
            const request = reviewsStore.get(reviewId);

            request.onsuccess = () => {
                if (request.result) {
                    resolve(request.result);
                } else {
                    reject(new Error('Review not found'));
                }
            };

            request.onerror = (event) => {
                console.error('Error getting review:', event.target.error);
                reject(event.target.error);
            };
        } catch (error) {
            console.error('Error in transaction:', error);
            reject(error);
        }
    });
}

// Function to get pending uploads
async function getPendingUploads() {
    return new Promise((resolve, reject) => {
        if (!db) {
            reject(new Error('Database not initialized'));
            return;
        }

        try {
            const transaction = db.transaction(PENDING_STORE, 'readonly');
            const pendingStore = transaction.objectStore(PENDING_STORE);
            const request = pendingStore.getAll();

            request.onsuccess = () => {
                resolve(request.result);
            };

            request.onerror = (event) => {
                console.error('Error getting pending uploads:', event.target.error);
                reject(event.target.error);
            };
        } catch (error) {
            console.error('Error in transaction:', error);
            reject(error);
        }
    });
}

// Function to remove a pending upload after it's successfully sent to server
async function removePendingUpload(id) {
    return new Promise((resolve, reject) => {
        if (!db) {
            reject(new Error('Database not initialized'));
            return;
        }

        try {
            const transaction = db.transaction(PENDING_STORE, 'readwrite');
            const pendingStore = transaction.objectStore(PENDING_STORE);
            const request = pendingStore.delete(id);

            request.onsuccess = () => {
                console.log('Pending upload removed');
                resolve();
            };

            request.onerror = (event) => {
                console.error('Error removing pending upload:', event.target.error);
                reject(event.target.error);
            };
        } catch (error) {
            console.error('Error in transaction:', error);
            reject(error);
        }
    });
}

// Simulate uploading to a server
function simulateServerUpload(data) {
    return new Promise((resolve, reject) => {
        // In a real implementation, this would be an actual API call
        console.log('Simulating upload to server:', data);

        // Simulate network latency
        setTimeout(() => {
            // 95% success rate in this simulation
            if (Math.random() < 0.95) {
                resolve({ success: true, id: `server-${Date.now()}` });
            } else {
                reject(new Error('Simulated server error'));
            }
        }, 1200);
    });
}

// Function to process pending uploads (sync with server)
async function syncPendingUploads() {
    if (!navigator.onLine) {
        console.log('Cannot sync pending uploads while offline');
        return;
    }

    try {
        const pendingUploads = await getPendingUploads();

        if (pendingUploads.length === 0) {
            console.log('No pending uploads to sync');
            return;
        }

        console.log(`Found ${pendingUploads.length} pending uploads to sync`);

        // Process each pending upload
        for (const upload of pendingUploads) {
            try {
                await simulateServerUpload(upload.data);
                await removePendingUpload(upload.id);
                console.log('Successfully synced upload:', upload.id);
            } catch (error) {
                console.error('Failed to sync upload:', upload.id, error);
                // We'll leave the upload in the pending store to try again later
            }
        }
    } catch (error) {
        console.error('Error syncing pending uploads:', error);
    }
}
