// Global variables for maps and markers
let map = null;
let discoverMap = null;
let tourMap = null;
let singleProducerMap = null;
let userLocationMarker = null;
let producerMarkers = null;
let producers = [];
let selectedProducers = [];

// Initialize on DOM load
document.addEventListener('DOMContentLoaded', function() {
    console.log('DOM fully loaded - setting up event listeners');
    
    // Set up main choice buttons
    setupChoiceEvents();
    
    // Set up other UI components when needed
    setupBackButtons();
    setupRadiusSlider();
    setupSearchButtons();
    setupLocationButtons();
    setupSingleProducerSearch(); // This function needs to run properly
});

// Setup choice section event listeners
function setupChoiceEvents() {
    const findSingleButton = document.getElementById('findSingleProducer');
    const planMultiStopButton = document.getElementById('planMultiStopTour');
    
    if (findSingleButton) {
        console.log('Find Single Producer button found');
        findSingleButton.onclick = function() {
            console.log('Single producer button clicked');
            showSingleProducerSection();
        };
    } else {
        console.error('Find Single Producer button not found');
    }
    
    if (planMultiStopButton) {
        console.log('Plan Multi-Stop Tour button found');
        planMultiStopButton.onclick = function() {
            console.log('Multi-stop tour button clicked');
            showMultiStopTourSection();
        };
    } else {
        console.error('Plan Multi-Stop Tour button not found');
    }
}

// Setup back buttons
function setupBackButtons() {
    const backButtons = document.querySelectorAll('.back-button');
    console.log('Found ' + backButtons.length + ' back buttons');
    
    backButtons.forEach(function(button) {
        button.onclick = function() {
            console.log('Back button clicked');
            showChoiceSection();
        };
    });
}

// Setup radius slider
function setupRadiusSlider() {
    const slider = document.getElementById('radiusSlider');
    const value = document.getElementById('radiusValue');
    if (slider && value) {
        slider.addEventListener('input', function() {
            value.textContent = slider.value;
        });
    }
}

// Setup search buttons
function setupSearchButtons() {
    const multiSearchButton = document.getElementById('searchButton');
    if (multiSearchButton) {
        multiSearchButton.onclick = function() {
            console.log('Multi search button clicked');
            searchProducers();
        };
    }
    
    const clearSelectionButton = document.getElementById('clearSelectionButton');
    if (clearSelectionButton) {
        clearSelectionButton.onclick = function() {
            console.log('Clear selection button clicked');
            clearSelection();
        };
    }
    
    const createTourButton = document.getElementById('createTourButton');
    if (createTourButton) {
        createTourButton.onclick = function() {
            console.log('Create tour button clicked');
            createTour();
        };
    }
    
    const backToResultsButton = document.getElementById('backToResultsButton');
    if (backToResultsButton) {
        backToResultsButton.onclick = function() {
            console.log('Back to results button clicked');
            backToResults();
        };
    }
    
    const shareTourButton = document.getElementById('shareTourButton');
    if (shareTourButton) {
        shareTourButton.onclick = function() {
            console.log('Share tour button clicked');
            showShareOptions();
        };
    }
}

// Setup location buttons
// Replace the current setupLocationButtons() function with this:
function setupLocationButtons() {
    console.log('Setting up location buttons');
    
    // Setup get current location button
    const getCurrentLocationButton = document.getElementById('getCurrentLocation');
    if (getCurrentLocationButton) {
        getCurrentLocationButton.onclick = function() {
            console.log('Get current location button clicked');
            getUserLocation();
        };
    }
    
    // Setup single producer location button
    const getSingleProducerLocationButton = document.getElementById('getSingleProducerLocation');
    if (getSingleProducerLocationButton) {
        getSingleProducerLocationButton.onclick = function() {
            console.log('Get single producer location button clicked');
            getSingleProducerLocation();
        };
    }
}

// Add this function after the setupLocationButtons() function
function setupLocationSelectionUI() {
    console.log('Setting up location selection UI');
    
    // Get all radio buttons
    const locationMethods = document.querySelectorAll('input[name="locationMethod"]');
    
    // Add event listeners to radio buttons
    locationMethods.forEach(function(method) {
        method.addEventListener('change', function() {
            showLocationInputByType(this.value);
        });
    });
    
    // Setup search location button
    const searchLocationButton = document.getElementById('searchLocationButton');
    if (searchLocationButton) {
        searchLocationButton.addEventListener('click', function() {
            const locationSearch = document.getElementById('locationSearch');
            if (locationSearch && locationSearch.value.trim()) {
                // Geocode the entered location
                geocodeLocation(locationSearch.value.trim());
            } else {
                showNotification('Please enter a location', 'warning');
            }
        });
    }
    
    // Initial setup - show the default method (address)
    // By default, the 'address' option should be checked in the HTML
    const checkedMethod = document.querySelector('input[name="locationMethod"]:checked');
    if (checkedMethod) {
        showLocationInputByType(checkedMethod.value);
    } else {
        // If none is checked, default to address
        const addressRadio = document.getElementById('locationMethodAddress');
        if (addressRadio) {
            addressRadio.checked = true;
            showLocationInputByType('address');
        }
    }
}
// Add this function after setupLocationSelectionUI()
function showLocationInputByType(type) {
    console.log('Showing location input type:', type);
    
    // Hide all input methods first
    const inputMethods = document.querySelectorAll('.location-input-method');
    inputMethods.forEach(function(method) {
        method.style.display = 'none';
    });
    
    // Show the selected method
    const selectedMethod = document.getElementById('locationInput' + type.charAt(0).toUpperCase() + type.slice(1));
    if (selectedMethod) {
        selectedMethod.style.display = 'block';
    }
    
    // For all location types, ensure the map is visible and properly sized
    // This ensures the map is initialized regardless of which option is selected
    setTimeout(function() {
        initializeMultiStopMap();
    }, 100);
}

// Add this function after showLocationInputByType()
function initializeMultiStopMap(forceInit = false) {
    console.log('Initializing multi-stop map, force:', forceInit);
    
    // Check if map container exists and is visible
    const mapContainer = document.getElementById('map');
    if (!mapContainer) {
        console.error('Map container not found');
        return;
    }
    
    // If map already exists but is not properly sized, invalidate size
    if (map && !forceInit) {
        console.log('Map exists, invalidating size');
        setTimeout(function() {
            map.invalidateSize();
        }, 100);
        return;
    }
    
    // If we need to create a new map or force reinitialize
    console.log('Creating new map instance');
    setTimeout(function() {
        initMap();
    }, 100);
}


// Add this function after initializeMultiStopMap()
function geocodeLocation(address) {
    console.log('Geocoding location:', address);
    showNotification('Finding location...', 'info');
    
    fetch('https://nominatim.openstreetmap.org/search?format=json&q=' + encodeURIComponent(address))
        .then(function(res) {
            if (!res.ok) throw new Error('Network error');
            return res.json();
        })
        .then(function(data) {
            if (data && data.length > 0) {
                const lat = parseFloat(data[0].lat);
                const lon = parseFloat(data[0].lon);
                
                // Update the map with the found location
                updateMapPosition(lat, lon);
                updateCoordinates(lat, lon);
                
                showNotification('Location found', 'success');
            } else {
                showNotification('Location not found', 'error');
            }
        })
        .catch(function(err) {
            console.error('Geocoding error:', err);
            showNotification('Error finding location', 'error');
        });
}

// Setup single producer search
function setupSingleProducerSearch() {
    console.log('Setting up single producer search');
    
    // Use direct approach to add the event listener
    document.addEventListener('click', function(event) {
        if (event.target && event.target.id === 'searchSingleProducerButton') {
            console.log('Search single producer button clicked via event delegation');
            searchSingleProducer();
        }
    });
    
    // Also try the direct approach for when the element exists immediately
    const searchSingleProducerButton = document.getElementById('searchSingleProducerButton');
    if (searchSingleProducerButton) {
        console.log('Search single producer button found directly');
        searchSingleProducerButton.addEventListener('click', function() {
            console.log('Search single producer button clicked directly');
            searchSingleProducer();
        });
    } else {
        console.warn('Search single producer button not found during initial setup - will use event delegation');
    }
}

// Show choice section
// Replace the current showChoiceSection() function with this:
function showChoiceSection() {
    console.log('Showing choice section');
    
    // Get references to sections
    const choiceSection = document.getElementById('choiceSection');
    const singleProducerSection = document.getElementById('singleProducerSection');
    const multiStopTourSection = document.getElementById('multiStopTourSection');
    
    // Hide/show sections
    if (choiceSection) choiceSection.style.display = 'block';
    if (singleProducerSection) singleProducerSection.style.display = 'none';
    if (multiStopTourSection) multiStopTourSection.style.display = 'none';
    
    // Reset any active maps
    resetMaps();
}

// Show single producer section
// Replace the current showSingleProducerSection() function with this:
function showSingleProducerSection() {
    console.log('Showing single producer section');
    
    // Get references to sections
    const choiceSection = document.getElementById('choiceSection');
    const singleProducerSection = document.getElementById('singleProducerSection');
    const multiStopTourSection = document.getElementById('multiStopTourSection');
    
    // Hide/show sections
    if (choiceSection) choiceSection.style.display = 'none';
    if (singleProducerSection) singleProducerSection.style.display = 'block';
    if (multiStopTourSection) multiStopTourSection.style.display = 'none';
    
    // Reset maps from other sections
    resetMaps();
    
    // Reset the single producer search form
    const nameSearch = document.getElementById('producerNameSearch');
    const locationSearch = document.getElementById('producerLocationSearch');
    
    if (nameSearch) nameSearch.value = '';
    if (locationSearch) locationSearch.value = '';
    
    // Clear any existing result
    const resultContainer = document.getElementById('singleProducerResult');
    if (resultContainer) resultContainer.style.display = 'none';
    
    // Setup the search button again, in case it was dynamically loaded
    setTimeout(function() {
        const searchButton = document.getElementById('searchSingleProducerButton');
        if (searchButton) {
            console.log('Found search button after section display');
            searchButton.onclick = function() {
                console.log('Search button clicked after section shown');
                searchSingleProducer();
            };
        } else {
            console.error('Search button not found after section display');
        }
    }, 100);
}

// Show multi-stop tour section
function showMultiStopTourSection() {
    console.log('Showing multi-stop tour section');
    
    // Get references to sections
    const choiceSection = document.getElementById('choiceSection');
    const singleProducerSection = document.getElementById('singleProducerSection');
    const multiStopTourSection = document.getElementById('multiStopTourSection');
    
    // Hide/show sections
    if (choiceSection) choiceSection.style.display = 'none';
    if (singleProducerSection) singleProducerSection.style.display = 'none';
    if (multiStopTourSection) multiStopTourSection.style.display = 'block';
    
    // Reset maps before initializing new ones
    resetMaps();
    
    // Reset form values
    const locationSearch = document.getElementById('locationSearch');
    const radiusSlider = document.getElementById('radiusSlider');
    const radiusValue = document.getElementById('radiusValue');
    
    if (locationSearch) locationSearch.value = '';
    if (radiusSlider) radiusSlider.value = 10;
    if (radiusValue) radiusValue.textContent = 10;
    
    // Hide any existing results
    const searchResults = document.querySelector('.search-results');
    const tourPlanContainer = document.getElementById('tourPlanContainer');
    
    if (searchResults) searchResults.style.display = 'none';
    if (tourPlanContainer) tourPlanContainer.style.display = 'none';
    
    // Reset hidden fields
    document.getElementById('latitude').value = '';
    document.getElementById('longitude').value = '';
    
    // Reset selected location text
    const selectedLocation = document.getElementById('selectedLocation');
    if (selectedLocation) selectedLocation.textContent = '';
    
    // Setup the location selection UI
    setupLocationSelectionUI();
    
    // Initialize the map - this is crucial - call after the section is displayed
    // Use a slightly longer timeout to ensure the DOM is fully rendered
    setTimeout(function() {
        // This ensures the map container is visible before initializing
        initializeMultiStopMap(true); // Pass true to force initialization
    }, 500);
}

// Replace the current resetMaps() function with this:
function resetMaps() {
    console.log('Resetting all maps');
    
    // If maps exist, remove them
    if (map) {
        map.remove();
        map = null;
        console.log('Main map removed');
    }
    
    if (discoverMap) {
        discoverMap.remove();
        discoverMap = null;
        console.log('Discover map removed');
    }
    
    if (tourMap) {
        tourMap.remove();
        tourMap = null;
        console.log('Tour map removed');
    }
    
    if (singleProducerMap) {
        singleProducerMap.remove();
        singleProducerMap = null;
        console.log('Single producer map removed');
    }
    
    // Reset user location marker
    userLocationMarker = null;
    console.log('All maps reset');
}

// Initialize main search map
// Replace the current initMap() function with this:
function initMap() {
    console.log('Initializing main map...');
    const mapContainer = document.getElementById('map');
    if (!mapContainer) {
        console.error('Map container not found');
        return;
    }

    try {
        // If map already exists, remove it first
        if (map) {
            map.remove();
            map = null;
        }
        
        // Create new map
        map = L.map('map').setView([33.6846, -117.8265], 13);
        
        // Create a new tile layer
        L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
            attribution: ' <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a>',
            maxZoom: 18
        }).addTo(map);
        
        console.log('Tile layer added to main map');

        // Add user location marker
        userLocationMarker = L.marker([33.6846, -117.8265], { draggable: true }).addTo(map);
        userLocationMarker.on('dragend', function(e) {
            const pos = userLocationMarker.getLatLng();
            updateCoordinates(pos.lat, pos.lng);
        });

        // Map click handler
        map.on('click', function(e) {
            userLocationMarker.setLatLng(e.latlng);
            updateCoordinates(e.latlng.lat, e.latlng.lng);
        });
        
        // Force a map refresh with timeout to ensure DOM is ready
        setTimeout(function() {
            map.invalidateSize();
            console.log('Map size invalidated');
        }, 300);
    } catch (e) {
        console.error('Error initializing map:', e);
        showNotification('Error initializing map', 'error');
    }
}


// Initialize results map
function initDiscoverMap() {
    console.log('Initializing discover map...');
    const mapContainer = document.getElementById('discoverMap');
    if (!mapContainer) {
        console.error('Discover map container not found');
        return;
    }

    try {
        // If discoverMap already exists, remove it first
        if (discoverMap) {
            discoverMap.remove();
            discoverMap = null;
        }
        
        discoverMap = L.map('discoverMap').setView([33.6846, -117.8265], 10);
        
        // Create a new tile layer
        L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
            attribution: ' <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a>',
            maxZoom: 18
        }).addTo(discoverMap);
        
        console.log('Tile layer added to discover map');
        
        producerMarkers = L.featureGroup().addTo(discoverMap);
        
        // Force a map refresh
        setTimeout(function() {
            discoverMap.invalidateSize();
        }, 100);
    } catch (e) {
        console.error('Error initializing discover map:', e);
        showNotification('Error initializing discover map', 'error');
    }
}

// Initialize tour map
function initTourMap() {
    console.log('Initializing tour map...');
    const mapContainer = document.getElementById('tourMap');
    if (!mapContainer) {
        console.log('Tour map container not found - this is normal if not on tour page');
        return;
    }

    try {
        // If tourMap already exists, remove it first
        if (tourMap) {
            tourMap.remove();
            tourMap = null;
        }
        
        tourMap = L.map('tourMap').setView([33.6846, -117.8265], 10);
        
        // Create a new tile layer
        L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
            attribution: ' <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a>',
            maxZoom: 18
        }).addTo(tourMap);
        
        console.log('Tile layer added to tour map');
        
        // Force a map refresh
        setTimeout(function() {
            tourMap.invalidateSize();
        }, 100);
    } catch (e) {
        console.error('Error initializing tour map:', e);
        showNotification('Error initializing tour map', 'error');
    }
}

// Initialize single producer map
function initSingleProducerMap(lat, lng, name) {
    console.log('Initializing single producer map...');
    const mapContainer = document.getElementById('singleProducerMap');
    if (!mapContainer) {
        console.error('Single producer map container not found');
        return;
    }

    try {
        // If singleProducerMap already exists, remove it first
        if (singleProducerMap) {
            singleProducerMap.remove();
            singleProducerMap = null;
        }
        
        singleProducerMap = L.map('singleProducerMap').setView([lat, lng], 15);
        
        // Create a new tile layer
        L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
            attribution: ' <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a>',
            maxZoom: 18
        }).addTo(singleProducerMap);
        
        console.log('Tile layer added to single producer map');
        
        // Add marker for the producer
        const marker = L.marker([lat, lng]).addTo(singleProducerMap);
        marker.bindPopup('<strong>' + name + '</strong>').openPopup();
        
        // Force a map refresh
        setTimeout(function() {
            singleProducerMap.invalidateSize();
        }, 100);
    } catch (e) {
        console.error('Error initializing single producer map:', e);
        showNotification('Error initializing producer map', 'error');
    }
}

// Get user's current location for multi-stop tour
function getUserLocation() {
    getLocation(function(latitude, longitude) {
        updateCoordinates(latitude, longitude);
        updateMapPosition(latitude, longitude);
    });
}

// Get user's current location for single producer search
function getSingleProducerLocation() {
    getLocation(function(latitude, longitude) {
        const locationSearch = document.getElementById('producerLocationSearch');
        reverseGeocode(latitude, longitude, function(address) {
            if (locationSearch) {
                locationSearch.value = address;
            }
        });
    });
}

// Common function to get user location
function getLocation(callback) {
    if (!navigator.geolocation) {
        console.error('Geolocation not supported');
        return showNotification('Geolocation not supported', 'error');
    }

    showNotification('Fetching location...', 'info');
    navigator.geolocation.getCurrentPosition(
        function(pos) {
            const latitude = pos.coords.latitude;
            const longitude = pos.coords.longitude;
            console.log('Location obtained:', { latitude, longitude });
            callback(latitude, longitude);
            showNotification('Location obtained', 'success');
        },
        function(err) {
            console.error('Geolocation error:', err);
            let msg = 'Unable to get location';
            if (err.code === 1) {
                msg = 'Location access denied. Please allow location access.';
            } else if (err.code === 2) {
                msg = 'Location unavailable. Try entering a location manually.';
            } else if (err.code === 3) {
                msg = 'Location request timed out. Please try again.';
            }
            showNotification(msg, 'error');
        },
        { enableHighAccuracy: true, timeout: 10000 }
    );
}

// State abbreviation to full name mapping
const stateMap = {
    'AL': 'Alabama',
    'AK': 'Alaska',
    'AZ': 'Arizona',
    'AR': 'Arkansas',
    'CA': 'California',
    'CO': 'Colorado',
    'CT': 'Connecticut',
    'DE': 'Delaware',
    'FL': 'Florida',
    'GA': 'Georgia',
    'HI': 'Hawaii',
    'ID': 'Idaho',
    'IL': 'Illinois',
    'IN': 'Indiana',
    'IA': 'Iowa',
    'KS': 'Kansas',
    'KY': 'Kentucky',
    'LA': 'Louisiana',
    'ME': 'Maine',
    'MD': 'Maryland',
    'MA': 'Massachusetts',
    'MI': 'Michigan',
    'MN': 'Minnesota',
    'MS': 'Mississippi',
    'MO': 'Missouri',
    'MT': 'Montana',
    'NE': 'Nebraska',
    'NV': 'Nevada',
    'NH': 'New Hampshire',
    'NJ': 'New Jersey',
    'NM': 'New Mexico',
    'NY': 'New York',
    'NC': 'North Carolina',
    'ND': 'North Dakota',
    'OH': 'Ohio',
    'OK': 'Oklahoma',
    'OR': 'Oregon',
    'PA': 'Pennsylvania',
    'RI': 'Rhode Island',
    'SC': 'South Carolina',
    'SD': 'South Dakota',
    'TN': 'Tennessee',
    'TX': 'Texas',
    'UT': 'Utah',
    'VT': 'Vermont',
    'VA': 'Virginia',
    'WA': 'Washington',
    'WV': 'West Virginia',
    'WI': 'Wisconsin',
    'WY': 'Wyoming',
    'DC': 'District of Columbia'
};

// Reverse map for looking up abbreviations from full state names
const reverseStateMap = {};
for (const [abbr, fullName] of Object.entries(stateMap)) {
    reverseStateMap[fullName.toLowerCase()] = abbr;
}

// Search for a single producer
function searchSingleProducer() {
    console.log('Searching for single producer');
    const nameSearch = document.getElementById('producerNameSearch');
    const locationSearch = document.getElementById('producerLocationSearch');
    
    const name = nameSearch ? nameSearch.value.trim() : '';
    let location = locationSearch ? locationSearch.value.trim() : '';
    
    // Check if the location is a state abbreviation or full state name
    let isStateSearch = false;
    let stateCode = '';
    
    // Check for 2-letter state code
    if (location.length === 2) {
        const upperLocation = location.toUpperCase();
        if (stateMap[upperLocation]) {
            location = stateMap[upperLocation];
            isStateSearch = true;
            stateCode = upperLocation;
            console.log('Detected state abbreviation:', { stateCode, stateName: location });
        }
    } 
    // Check for full state name
    else if (location.length > 2) {
        const lowerLocation = location.toLowerCase();
        if (reverseStateMap[lowerLocation]) {
            isStateSearch = true;
            stateCode = reverseStateMap[lowerLocation];
            console.log('Detected full state name:', { stateCode, stateName: location });
        }
    }
    
    // Validate input - require at least one search parameter
    if (!name && !location) {
        return showNotification('Please enter a producer name or location', 'warning');
    }
    
    showNotification('Searching...', 'info');
    
    // CASE 1: Location provided - get coordinates for the location
    if (location) {
        fetch('https://nominatim.openstreetmap.org/search?format=json&q=' + encodeURIComponent(location))
            .then(function(res) {
                if (!res.ok) throw new Error('Network error');
                return res.json();
            })
            .then(function(data) {
                if (data && data.length > 0) {
                    // Get coordinates for the location
                    const lat = parseFloat(data[0].lat);
                    const lon = parseFloat(data[0].lon);
                    
                    console.log('Location found:', { location, lat, lon });
                    
                    // If this is a state search, pass the state code
                    const stateParam = isStateSearch ? stateCode : '';
                    
                    // Search with both name and location
                    searchProducerWithParams(name, location, lat, lon, false, isStateSearch, stateParam);
                } else {
                    showNotification('Location not found. Try a different location.', 'error');
                }
            })
            .catch(function(err) {
                console.error('Geocoding error:', err);
                showNotification('Error finding location', 'error');
            });
    } 
    // CASE 2: Name-only search - handle as nationwide search
    else {
        console.log('Performing nationwide name-only search for:', name);
        
        // Use center of US for nationwide search
        const defaultLat = 39.8283; // Center of continental US
        const defaultLon = -98.5795;
        
        // Pass these to the search function with nationwide flag
        searchProducerWithParams(name, '', defaultLat, defaultLon, true, false, '');
    }
}

// Search for producers using name and/or location parameters
function searchProducerWithParams(name, locationName, lat, lng, nationwide, isStateSearch = false, stateCode = '') {
    console.log('searchProducerWithParams called with:', { name, locationName, lat, lng, nationwide, isStateSearch, stateCode });
    
    let apiUrl = './api/producers-search.php?';
    
    // Add search parameters
    const params = [];
    
    // Always add name parameter if provided
    if (name) {
        params.push('query=' + encodeURIComponent(name));
    }
    
    // Add location parameters
    params.push('latitude=' + lat);
    params.push('longitude=' + lng);
    
    // Handle different search types
    if (nationwide) {
        // Nationwide search - no distance filtering
        params.push('nationwide=true');
        // No radius needed for nationwide search
    } else if (isStateSearch && stateCode) {
        // State-level search - just state filter
        params.push('state=' + encodeURIComponent(stateCode));
        // No radius for state search
    } else {
        // Standard local search
        params.push('radius=50');
    }
    
    // Join all parameters
    apiUrl += params.join('&');
    
    console.log('Searching producers with URL:', apiUrl);
    
    fetch(apiUrl)
        .then(function(res) {
            if (!res.ok) {
                console.error('HTTP error status:', res.status);
                throw new Error('HTTP ' + res.status);
            }
            return res.json();
        })
        .then(function(data) {
            console.log('Producer results:', data);
            if (data.producers && data.producers.length > 0) {
                // If multiple producers found, show a selection table
                if (data.producers.length > 1) {
                    displayProducerSelectionTable(data.producers, name, locationName);
                    showNotification('Found ' + data.producers.length + ' producers', 'success');
                } else {
                    // If only one producer found, display it directly
                    displaySingleProducerResult(data.producers[0]);
                    showNotification('Found producer', 'success');
                }
            } else {
                showNotification('No producers found', 'warning');
            }
        })
        .catch(function(err) {
            console.error('Search error:', err);
            showNotification('Error searching producers: ' + err.message, 'error');
        });
}

// Display a table of producers for selection
function displayProducerSelectionTable(producers, searchName, searchLocation) {
    const resultContainer = document.getElementById('singleProducerResult');
    if (!resultContainer) {
        console.error('Single producer result container not found');
        return;
    }
    
    // Sort producers by match relevance and then by distance
    producers = sortProducersByRelevance(producers, searchName, searchLocation);
    
    // Check if this is a nationwide search
    const isNationwide = searchName && !searchLocation;
    
    // Check if we need to limit results
    const maxResultsToShow = 50;
    const totalResults = producers.length;
    let limitingResults = false;
    
    if (totalResults > maxResultsToShow) {
        limitingResults = true;
        producers = producers.slice(0, maxResultsToShow);
    }
    
    let html = `
        <div class="producer-selection">
            <h4>Found ${totalResults} Producers</h4>
            ${limitingResults ? `<p class="refine-message">Showing first ${maxResultsToShow} results. Please refine your search for more specific results.</p>` : ''}
            <p>Please select a producer to view details:</p>
            <div class="producer-selection-table">
                <div class="producer-selection-header">
                    <div class="producer-selection-name">Producer Name</div>
                    <div class="producer-selection-address">Address</div>
                    <div class="producer-selection-distance">${isNationwide ? 'State' : 'Distance'}</div>
                    <div class="producer-selection-action">Action</div>
                </div>
                <div class="producer-selection-list">
    `;
    
    producers.forEach(function(producer) {
        const name = producer.DBA || producer.CompanyName;
        const address = producer.Street + ', ' + producer.City + ', ' + producer.State + ' ' + producer.Zip;
        
        // Encode producer data for safe passing to onclick handler
        const encodedProducer = encodeURIComponent(JSON.stringify(producer));
        
        // For nationwide searches, show state instead of distance
        const distanceDisplay = isNationwide ? 
            producer.State : // Just show state for nationwide searches
            producer.distanceMiles + ' mi'; // Show distance for local searches
        
        html += `
            <div class="producer-selection-item">
                <div class="producer-selection-name">${name}</div>
                <div class="producer-selection-address">${address}</div>
                <div class="producer-selection-distance">${distanceDisplay}</div>
                <div class="producer-selection-action">
                    <button class="selection-button" onclick="selectProducer('${encodedProducer}')">Select</button>
                </div>
            </div>
        `;
    });
    
    html += `
                </div>
            </div>
            <div class="selection-actions">
                <button class="secondary-button" onclick="showSingleProducerForm()">New Search</button>
            </div>
        </div>
    `;
    
    resultContainer.innerHTML = html;
    resultContainer.style.display = 'block';
    
    // Add CSS for the selection table if not already present
    addSelectionTableStyles();
    
    // Add CSS for the refine message if not already present
    if (!document.getElementById('refine-message-styles')) {
        const styleEl = document.createElement('style');
        styleEl.id = 'refine-message-styles';
        styleEl.textContent = `
            .refine-message {
                color: #cc0000;
                font-weight: bold;
                background-color: #fff4f4;
                padding: 0.5rem;
                border-radius: 8px;
                margin-bottom: 1rem;
                border: 1px solid #ffcccc;
            }
        `;
        document.head.appendChild(styleEl);
    }
    
    // Add global function for select producer function
    window.selectProducer = function(encodedProducer) {
        try {
            const producer = JSON.parse(decodeURIComponent(encodedProducer));
            displaySingleProducerResult(producer);
        } catch (e) {
            console.error('Error parsing producer data:', e);
            showNotification('Error selecting producer', 'error');
        }
    };
    
    // Add global function for showSingleProducerForm
    window.showSingleProducerForm = function() {
        const resultContainer = document.getElementById('singleProducerResult');
        if (resultContainer) {
            resultContainer.style.display = 'none';
            resultContainer.innerHTML = '';
        }
        
        // Clear the search inputs
        const nameSearch = document.getElementById('producerNameSearch');
        const locationSearch = document.getElementById('producerLocationSearch');
        
        if (nameSearch) nameSearch.value = '';
        if (locationSearch) locationSearch.value = '';
    };
}

// Sort producers by relevance to search terms and then by distance
function sortProducersByRelevance(producers, searchName, searchLocation) {
    if (!searchName && !searchLocation) {
        // If no search terms, just sort by distance
        return producers.sort(function(a, b) {
            return a.distance - b.distance;
        });
    }
    
    // Define if this is a nationwide search (no location provided)
    const isNationwide = searchName && !searchLocation;
    
    // Assign a relevance score to each producer
    producers.forEach(function(producer) {
        let score = 0;
        const name = (producer.DBA || producer.CompanyName || '').toLowerCase();
        const location = (producer.City + ' ' + producer.State).toLowerCase();
        
        // Add points for name matches
        if (searchName) {
            const searchNameLower = searchName.toLowerCase();
            
            // Exact match - highest score
            if (name === searchNameLower) {
                score += 1000; // Much higher for nationwide searches
            }
            // Name starts with search term
            else if (name.startsWith(searchNameLower)) {
                score += 750;
            }
            // Contains search term as a whole word
            else if (new RegExp('\\b' + searchNameLower + '\\b').test(name)) {
                score += 500;
            }
            // Contains search term anywhere
            else if (name.includes(searchNameLower)) {
                score += 250;
            }
            // Contains parts of search term
            else {
                const searchParts = searchNameLower.split(/\s+/);
                searchParts.forEach(function(part) {
                    if (part.length > 2 && name.includes(part)) {
                        score += 100;
                    }
                });
            }
        }
        
        // Add points for location matches (not relevant for nationwide searches)
        if (!isNationwide && searchLocation) {
            const searchLocationLower = searchLocation.toLowerCase();
            if (location.includes(searchLocationLower)) {
                score += 30;
            }
        }
        
        // Add inverse distance as a small factor (max 10 points for closest)
        // For nationwide searches, distance is much less important
        if (producer.distance) {
            const distancePoints = isNationwide 
                ? Math.max(0, 5 - producer.distance / 1000) // Very small factor for nationwide
                : Math.max(0, 50 - producer.distance / 2);  // Bigger factor for local searches
            score += distancePoints;
        }
        
        producer.relevanceScore = score;
    });
    
    // For nationwide searches, use state grouping to show results from same state together
    if (isNationwide) {
        // First group by state
        const stateGroups = {};
        producers.forEach(function(producer) {
            const state = producer.State || 'Unknown';
            if (!stateGroups[state]) {
                stateGroups[state] = [];
            }
            stateGroups[state].push(producer);
        });
        
        // Sort each state group by relevance score
        Object.keys(stateGroups).forEach(function(state) {
            stateGroups[state].sort(function(a, b) {
                return b.relevanceScore - a.relevanceScore;
            });
        });
        
        // Combine all sorted producers
        let sortedProducers = [];
        
        // Then add all other states
        Object.keys(stateGroups).forEach(function(state) {
            sortedProducers = sortedProducers.concat(stateGroups[state]);
        });
        
        return sortedProducers;
    } else {
        // For local searches, sort by relevance score and distance
        return producers.sort(function(a, b) {
            if (b.relevanceScore !== a.relevanceScore) {
                return b.relevanceScore - a.relevanceScore;
            }
            return a.distance - b.distance;
        });
    }
}

// Display a single producer result
function displaySingleProducerResult(producer) {
    const resultContainer = document.getElementById('singleProducerResult');
    if (!resultContainer) {
        console.error('Single producer result container not found');
        return;
    }
    
    const name = producer.DBA || producer.CompanyName;
    const address = producer.Street + ', ' + producer.City + ', ' + producer.State + ' ' + producer.Zip;
    
    // Build amenities list
    let amenities = [];
    if (producer.TastingRoom === 'Y') amenities.push('Tasting Room');
    if (producer.Tours === 'Y') amenities.push('Tours');
    if (producer.Appointments === 'Y') amenities.push('Appointments');
    if (producer.Accessible === 'Y') amenities.push('Accessible');
    const amenitiesText = amenities.length > 0 ? amenities.join(', ') : 'None';
    
    // Build social links
    let socialLinks = [];
    if (producer.FB) socialLinks.push('<a href="' + formatSocialUrl('facebook', producer.FB) + '" target="_blank">Facebook</a>');
    if (producer.IG) socialLinks.push('<a href="' + formatSocialUrl('instagram', producer.IG) + '" target="_blank">Instagram</a>');
    if (producer.X) socialLinks.push('<a href="' + formatSocialUrl('twitter', producer.X) + '" target="_blank">X</a>');
    const socialText = socialLinks.length > 0 ? socialLinks.join(' | ') : '';
    
    resultContainer.innerHTML = `
        <div class="producer-detail">
            <div class="producer-detail-header">
                <div class="producer-detail-name">${name}</div>
                <div class="producer-detail-distance">${producer.distanceMiles} miles away</div>
            </div>
            
            <div class="producer-detail-address">${address}</div>
            
            <div class="producer-detail-contact">
                ${producer.Phone ? '<div class="producer-detail-phone"><strong>Phone:</strong> <a href="tel:' + producer.Phone + '">' + formatPhoneNumber(producer.Phone) + '</a></div>' : ''}
                ${producer.WebSite ? '<div class="producer-detail-website"><a href="' + formatWebsiteUrl(producer.WebSite) + '" target="_blank">Visit Website</a></div>' : ''}
            </div>
            
            <div class="producer-detail-info-section">
                ${producer.Hours ? '<div class="producer-detail-hours"><strong>Hours:</strong> ' + producer.Hours + '</div>' : ''}
                <div class="producer-detail-amenities"><strong>Amenities:</strong> ${amenitiesText}</div>
                ${socialText ? '<div class="producer-social">' + socialText + '</div>' : ''}
            </div>
            
            <div id="singleProducerMap" class="producer-detail-map"></div>
            
            <div class="actions-row">
                <button type="button" class="primary-button" onclick="getDirections(${producer.GeoLat}, ${producer.GeoLong})">Get Directions</button>
                <button type="button" class="secondary-button" onclick="useAsStartingPoint(${producer.GeoLat}, ${producer.GeoLong}, '${name.replace(/'/g, "\\'")}')">Use as Starting Point</button>
                <button type="button" class="secondary-button" onclick="showSingleProducerForm()">New Search</button>
            </div>
        </div>
    `;
    
    resultContainer.style.display = 'block';
    
    // Initialize the map with the producer location
    setTimeout(function() {
        initSingleProducerMap(producer.GeoLat, producer.GeoLong, name);
    }, 100);
    
    // Add global functions for button actions if they don't exist
    window.getDirections = window.getDirections || function(lat, lng) {
        // Try to open in Google Maps
        window.open('https://www.google.com/maps/dir/?api=1&destination=' + lat + ',' + lng, '_blank');
    };
    
    window.useAsStartingPoint = window.useAsStartingPoint || function(lat, lng, name) {
        showMultiStopTourSection();
        
        setTimeout(function() {
            // Wait for the map to initialize
            if (!map) {
                initMap();
                setTimeout(function() {
                    updateMapPosition(lat, lng);
                    updateCoordinates(lat, lng);
                    document.getElementById('locationSearch').value = name;
                }, 500);
            } else {
                updateMapPosition(lat, lng);
                updateCoordinates(lat, lng);
                document.getElementById('locationSearch').value = name;
            }
        }, 300);
    };
}

// Add CSS styles for the selection table if not already in the document
function addSelectionTableStyles() {
    if (!document.getElementById('selection-table-styles')) {
        const styleEl = document.createElement('style');
        styleEl.id = 'selection-table-styles';
        styleEl.textContent = `
            .producer-selection {
                margin-top: 1.5rem;
            }
            
            .producer-selection h4 {
                margin-bottom: 0.5rem;
                color: var(--primary-color);
            }
            
            .producer-selection-table {
                border: 1px solid #eee;
                border-radius: var(--radius);
                overflow: hidden;
                margin-top: 1rem;
            }
            
            .producer-selection-header {
                display: grid;
                grid-template-columns: 2fr 3fr 1fr 1fr;
                background-color: #f5f5f5;
                padding: 0.8rem;
                font-weight: bold;
                border-bottom: 1px solid #eee;
            }
            
            .producer-selection-list {
                max-height: 400px;
                overflow-y: auto;
            }
            
            .producer-selection-item {
                display: grid;
                grid-template-columns: 2fr 3fr 1fr 1fr;
                padding: 0.8rem;
                border-bottom: 1px solid #eee;
                align-items: center;
            }
            
            .producer-selection-item:last-child {
                border-bottom: none;
            }
            
            .producer-selection-item:hover {
                background-color: rgba(139, 69, 19, 0.05);
            }
            
            .selection-button {
                background-color: var(--primary-color);
                color: white;
                border: none;
                padding: 0.5rem 1rem;
                border-radius: var(--radius);
                cursor: pointer;
                transition: background-color 0.3s;
            }
            
            .selection-button:hover {
                background-color: var(--secondary-color);
            }
            
            .selection-actions {
                margin-top: 1rem;
                display: flex;
                justify-content: center;
            }
            
            @media (max-width: 768px) {
                .producer-selection-header,
                .producer-selection-item {
                    grid-template-columns: 2fr 3fr 1fr 1fr;
                    font-size: 0.9rem;
                }
                
                .producer-selection-address {
                    font-size: 0.8rem;
                }
            }
            
            @media (max-width: 480px) {
                .producer-selection-header,
                .producer-selection-item {
                    grid-template-columns: 1fr 1.5fr 0.6fr 0.6fr;
                    font-size: 0.8rem;
                    padding: 0.6rem;
                }
                
                .selection-button {
                    padding: 0.4rem 0.6rem;
                    font-size: 0.8rem;
                }
            }
        `;
        document.head.appendChild(styleEl);
    }
}

// Reverse geocode coordinates to address
function reverseGeocode(lat, lng, callback) {
    fetch('https://nominatim.openstreetmap.org/reverse?format=json&lat=' + lat + '&lon=' + lng + '&zoom=18&addressdetails=1')
        .then(function(res) {
            if (!res.ok) throw new Error('Network error');
            return res.json();
        })
        .then(function(data) {
            if (data.display_name) {
                const address = formatAddress(data);
                
                const locationSearch = document.getElementById('locationSearch');
                const selectedLocation = document.getElementById('selectedLocation');
                
                if (locationSearch) {
                    locationSearch.value = address;
                }
                
                if (selectedLocation) {
                    selectedLocation.textContent = 'Selected: ' + address;
                }
                
                if (callback) {
                    callback(address);
                }
            }
        })
        .catch(function(err) {
            console.error('Reverse geocoding error:', err);
        });
}

// Format address
function formatAddress(data) {
    const addr = data.address || {};
    const parts = [];
    if (addr.house_number && addr.road) parts.push(addr.house_number + ' ' + addr.road);
    else if (addr.road) parts.push(addr.road);
    if (addr.city || addr.town || addr.village) parts.push(addr.city || addr.town || addr.village);
    if (addr.state) parts.push(addr.state);
    if (addr.postcode) parts.push(addr.postcode);
    return parts.join(', ') || data.display_name;
}

// Update coordinates
function updateCoordinates(lat, lng) {
    document.getElementById('latitude').value = lat.toFixed(6);
    document.getElementById('longitude').value = lng.toFixed(6);
    
    const locationSearch = document.getElementById('locationSearch');
    if (locationSearch) {
        locationSearch.value = lat.toFixed(6) + ', ' + lng.toFixed(6);
    }
    
    const selectedLocation = document.getElementById('selectedLocation');
    if (selectedLocation) {
        selectedLocation.textContent = 'Selected: ' + lat.toFixed(6) + ', ' + lng.toFixed(6);
    }
    
    reverseGeocode(lat, lng);
}

// Update map position
function updateMapPosition(lat, lng) {
    if (map && userLocationMarker) {
        map.setView([lat, lng], 15);
        userLocationMarker.setLatLng([lat, lng]);
    }
}

// Search producers for multi-stop tour
function searchProducers() {
    const lat = document.getElementById('latitude').value;
    const lng = document.getElementById('longitude').value;
    const radius = document.getElementById('radiusSlider') ? document.getElementById('radiusSlider').value : 10;

    if (!lat || !lng) return showNotification('Select a location', 'error');

    showNotification('Searching producers...', 'info');
    
    const searchButton = document.getElementById('searchButton');
    if (searchButton) {
        searchButton.disabled = true;
    }

    // API URL
    let apiUrl = './api/producers-search.php?latitude=' + lat + '&longitude=' + lng + '&radius=' + (radius * 1.60934);
    
    console.log('Searching producers with URL:', apiUrl);
    
    fetch(apiUrl)
        .then(function(res) {
            if (!res.ok) {
                console.error('HTTP error status:', res.status);
                throw new Error('HTTP ' + res.status);
            }
            return res.json();
        })
        .then(function(data) {
            console.log('Producer results:', data);
            if (data.producers && data.producers.length > 0) {
                producers = data.producers;
                displayProducerResults(producers);
                
                const searchResults = document.querySelector('.search-results');
                if (searchResults) {
                    searchResults.style.display = 'block';
                }
                
                // Initialize discover map
                setTimeout(function() {
                    initDiscoverMap();
                    setTimeout(function() {
                        updateMapWithProducers(producers, lat, lng);
                    }, 200);
                }, 100);
                
                selectedProducers = [];
                updateSelectionCount();
                showNotification('Found ' + producers.length + ' producers', 'success');
            } else {
                showNotification('No producers found', 'warning');
            }
        })
        .catch(function(err) {
            console.error('Search error:', err);
            showNotification('Error searching producers: ' + err.message, 'error');
        })
        .finally(function() {
            const searchButton = document.getElementById('searchButton');
            if (searchButton) {
                searchButton.disabled = false;
            }
        });
}

// Display producer results for multi-stop tour
function displayProducerResults(producers) {
    const content = document.getElementById('producerListContent');
    const resultCount = document.getElementById('resultCount');
    
    if (!content) {
        console.error('Producer list content element not found');
        return;
    }
    
    content.innerHTML = '';
    
    if (resultCount) {
        resultCount.textContent = '(' + producers.length + ' found)';
    }

    // Sort producers by distance
    producers.sort(function(a, b) {
        return a.distance - b.distance;
    }).forEach(function(producer) {
        const item = document.createElement('div');
        item.className = 'producer-item';
        item.dataset.id = producer.ProducerID;

        const name = producer.DBA || producer.CompanyName;
        const address = producer.Street + ', ' + producer.City + ', ' + producer.State + ' ' + producer.Zip;
        
        // Build amenities list
        let amenities = [];
        if (producer.TastingRoom === 'Y') amenities.push('Tasting Room');
        if (producer.Tours === 'Y') amenities.push('Tours');
        if (producer.Appointments === 'Y') amenities.push('Appointments');
        if (producer.Accessible === 'Y') amenities.push('Accessible');
        const amenitiesText = amenities.length > 0 ? amenities.join(', ') : 'None';

        item.innerHTML = `
            <div class="select-col">
                <input type="checkbox" id="producer-${producer.ProducerID}" class="producer-checkbox" data-id="${producer.ProducerID}">
            </div>
            <div class="name-col">
                <div class="producer-name">${name}</div>
                <div class="producer-address">${address}</div>
                <div class="producer-amenities"><strong>Amenities:</strong> ${amenitiesText}</div>
                ${producer.Hours ? '<div class="producer-hours"><strong>Hours:</strong> ' + producer.Hours + '</div>' : ''}
            </div>
            <div class="distance-col">${producer.distanceMiles} mi</div>
        `;

        const checkbox = item.querySelector('.producer-checkbox');
        if (checkbox) {
            checkbox.addEventListener('change', function() {
                toggleProducerSelection(producer, checkbox.checked);
            });
        }

        content.appendChild(item);
    });
}

// Update map with producers
function updateMapWithProducers(producers, userLat, userLng) {
    console.log('Updating map with', producers.length, 'producers');
    
    if (!discoverMap) {
        console.error('discoverMap is not initialized');
        return;
    }
    
    try {
        if (producerMarkers) {
            producerMarkers.clearLayers();
        } else {
            producerMarkers = L.featureGroup().addTo(discoverMap);
        }

        const bounds = L.latLngBounds();
        const lat = parseFloat(userLat);
        const lng = parseFloat(userLng);

        if (lat && lng && !isNaN(lat) && !isNaN(lng)) {
            const userMarker = L.marker([lat, lng], {
                icon: L.divIcon({ className: 'user-location-marker', html: '<div class="user-marker-icon"></div>', iconSize: [20, 20] })
            }).addTo(discoverMap);
            userMarker.bindPopup('<strong>Your Location</strong>');
            bounds.extend([lat, lng]);
        }

        producers.forEach(function(producer) {
            if (producer.GeoLat && producer.GeoLong && !isNaN(producer.GeoLat) && !isNaN(producer.GeoLong)) {
                const marker = L.marker([producer.GeoLat, producer.GeoLong]);
                marker.bindPopup(`
                    <div class="map-popup">
                        <div class="popup-name">${producer.DBA || producer.CompanyName}</div>
                        <div class="popup-address">${producer.Street}, ${producer.City}, ${producer.State}</div>
                        <div class="popup-distance">${producer.distanceMiles} miles</div>
                    </div>
                `);
                producerMarkers.addLayer(marker);
                bounds.extend([producer.GeoLat, producer.GeoLong]);
            }
        });

        if (bounds.isValid()) {
            console.log('Fitting bounds to view');
            discoverMap.fitBounds(bounds, { padding: [50, 50], maxZoom: 15 });
            // Force a redraw
            setTimeout(function() {
                discoverMap.invalidateSize();
            }, 100);
        } else if (lat && lng && !isNaN(lat) && !isNaN(lng)) {
            console.log('Setting view to user location');
            discoverMap.setView([lat, lng], 10);
            setTimeout(function() {
                discoverMap.invalidateSize();
            }, 100);
        } else {
            console.log('No valid bounds or user location, using default view');
            discoverMap.setView([33.6846, -117.8265], 10);
            setTimeout(function() {
                discoverMap.invalidateSize();
            }, 100);
        }
    } catch (e) {
        console.error('Error in updateMapWithProducers:', e);
        showNotification('Error rendering map', 'error');
    }
}

// Toggle producer selection
function toggleProducerSelection(producer, isSelected) {
    const item = document.querySelector(`.producer-item[data-id="${producer.ProducerID}"]`);
    if (isSelected) {
        if (selectedProducers.length >= 8) {
            const checkbox = document.getElementById(`producer-${producer.ProducerID}`);
            if (checkbox) {
                checkbox.checked = false;
            }
            showNotification('Maximum 8 producers allowed', 'warning');
            return;
        }
        selectedProducers.push(producer);
        if (item) {
            item.classList.add('selected');
        }
    } else {
        selectedProducers = selectedProducers.filter(function(p) {
            return p.ProducerID !== producer.ProducerID;
        });
        if (item) {
            item.classList.remove('selected');
        }
    }
    updateSelectionCount();
}

// Update selection count
function updateSelectionCount() {
    const selectedCount = document.getElementById('selectedCount');
    if (selectedCount) {
        selectedCount.textContent = selectedProducers.length;
    }
    
    const clearSelectionButton = document.getElementById('clearSelectionButton');
    if (clearSelectionButton) {
        clearSelectionButton.disabled = !selectedProducers.length;
    }
    
    const createTourButton = document.getElementById('createTourButton');
    if (createTourButton) {
        createTourButton.disabled = !selectedProducers.length;
    }
}

// Clear selection
function clearSelection() {
    document.querySelectorAll('.producer-checkbox').forEach(function(cb) {
        cb.checked = false;
    });
    document.querySelectorAll('.producer-item').forEach(function(item) {
        item.classList.remove('selected');
    });
    selectedProducers = [];
    updateSelectionCount();
    showNotification('Selection cleared', 'info');
}

// Create tour
function createTour() {
    if (!selectedProducers.length) return showNotification('Select at least one producer', 'warning');
    createTourView();
    
    const searchResults = document.querySelector('.search-results');
    if (searchResults) {
        searchResults.style.display = 'none';
    }
    
    const tourPlanContainer = document.getElementById('tourPlanContainer');
    if (tourPlanContainer) {
        tourPlanContainer.style.display = 'block';
    }
    
    window.scrollTo({ top: 0, behavior: 'smooth' });
}

// Create tour view
function createTourView() {
    console.log('Creating tour view');
    const locations = document.getElementById('tourLocations');
    if (!locations) {
        console.error('Tour locations container not found');
        return;
    }
    
    locations.innerHTML = '';
    
    // Get starting location information
    const startLat = parseFloat(document.getElementById('latitude').value || 33.6846);
    const startLng = parseFloat(document.getElementById('longitude').value || -117.8265);
    const startLocationText = document.getElementById('selectedLocation').textContent || 'Your Starting Location';
    
    // Add starting point as a special item
    const startItem = document.createElement('div');
    startItem.className = 'tour-location-item starting-point';
    startItem.innerHTML = `
        <div class="tour-location-header">
            <div class="tour-location-name">Starting Point</div>
            <div class="tour-location-number-start">
                <svg class="starting-icon" xmlns="http://www.w3.org/2000/svg" viewBox="0 0 24 24" width="24" height="24">
                    <path fill="#8b4513" d="M12 0c-4.198 0-8 3.403-8 7.602 0 4.198 3.469 9.21 8 16.398 4.531-7.188 8-12.2 8-16.398 0-4.199-3.801-7.602-8-7.602zm0 11c-1.657 0-3-1.343-3-3s1.343-3 3-3 3 1.343 3 3-1.343 3-3 3z"/>
                </svg>
            </div>
        </div>
        <div class="tour-location-info">
            <div class="tour-location-address">${startLocationText}</div>
        </div>
    `;
    locations.appendChild(startItem);

    // Sort producers by distance
    const sorted = selectedProducers.sort(function(a, b) {
        return a.distance - b.distance;
    });

    sorted.forEach(function(producer, i) {
        const name = producer.DBA || producer.CompanyName;
        const address = producer.Street + ', ' + producer.City + ', ' + producer.State + ' ' + producer.Zip;
        
        // Build amenities list
        let amenities = [];
        if (producer.TastingRoom === 'Y') amenities.push('Tasting Room');
        if (producer.Tours === 'Y') amenities.push('Tours');
        if (producer.Appointments === 'Y') amenities.push('Appointments');
        if (producer.Accessible === 'Y') amenities.push('Accessible');
        const amenitiesText = amenities.length > 0 ? amenities.join(', ') : 'None';
        
        // Build social links
        let socialLinks = [];
        if (producer.FB) socialLinks.push('<a href="' + formatSocialUrl('facebook', producer.FB) + '" target="_blank">Facebook</a>');
        if (producer.IG) socialLinks.push('<a href="' + formatSocialUrl('instagram', producer.IG) + '" target="_blank">Instagram</a>');
        if (producer.X) socialLinks.push('<a href="' + formatSocialUrl('twitter', producer.X) + '" target="_blank">X</a>');
        const socialText = socialLinks.length > 0 ? socialLinks.join(' | ') : '';

        const item = document.createElement('div');
        item.className = 'tour-location-item';
        item.innerHTML = `
            <div class="tour-location-header">
                <div class="tour-location-name">${name}</div>
                <div class="tour-location-number">${i + 1}</div>
            </div>
            <div class="tour-location-info">
                <div class="tour-location-address">${address}</div>
                ${producer.Hours ? '<div class="tour-location-hours"><strong>Hours:</strong> ' + producer.Hours + '</div>' : ''}
                <div class="tour-location-amenities"><strong>Amenities:</strong> ${amenitiesText}</div>
                <div class="tour-location-contact">
                    ${producer.Phone ? '<div class="tour-location-phone"><strong>Phone:</strong> <a href="tel:' + producer.Phone + '">' + formatPhoneNumber(producer.Phone) + '</a></div>' : ''}
                    ${producer.WebSite ? '<div class="tour-location-website"><a href="' + formatWebsiteUrl(producer.WebSite) + '" target="_blank">Visit Website</a></div>' : ''}
                </div>
                ${socialText ? '<div class="producer-social">' + socialText + '</div>' : ''}
            </div>
        `;
        locations.appendChild(item);
    });

    // Initialize tour map
    setTimeout(function() {
        initTourMap();
        setTimeout(function() {
            updateTourMap();
        }, 200);
    }, 100);
}

// Update tour map
function updateTourMap() {
    console.log('Updating tour map');
    try {
        if (!tourMap) {
            console.log('Tour map not initialized, initializing now');
            initTourMap();
            return; // Will be called again after initialization
        }

        // Clear any existing layers except tile layer
        tourMap.eachLayer(function(layer) {
            if (!(layer instanceof L.TileLayer)) {
                tourMap.removeLayer(layer);
            }
        });

        const bounds = L.latLngBounds();
        const startLat = parseFloat(document.getElementById('latitude').value || 33.6846);
        const startLng = parseFloat(document.getElementById('longitude').value || -117.8265);

        // Add starting point marker
        if (startLat && startLng && !isNaN(startLat) && !isNaN(startLng)) {
            const startMarker = L.marker([startLat, startLng], {
                icon: L.divIcon({ 
                    className: 'user-location-marker', 
                    html: '<div class="user-marker-icon"></div>', 
                    iconSize: [20, 20] 
                })
            }).addTo(tourMap);
            startMarker.bindPopup('<strong>Your Starting Location</strong>');
            bounds.extend([startLat, startLng]);
        }

        // Create array of all tour points including starting point
        const allPoints = [];
        
        // Add starting point first
        allPoints.push({
            lat: startLat,
            lng: startLng,
            isStartPoint: true
        });
        
        // Add all selected producers
        selectedProducers.forEach(function(p, i) {
            if (p.GeoLat && p.GeoLong && !isNaN(p.GeoLat) && !isNaN(p.GeoLong)) {
                allPoints.push({
                    lat: p.GeoLat,
                    lng: p.GeoLong,
                    isStartPoint: false,
                    producer: p,
                    index: i
                });
                
                // Add marker for each producer
                const marker = L.marker([p.GeoLat, p.GeoLong], {
                    icon: L.divIcon({ 
                        className: 'numbered-marker', 
                        html: '<div class="marker-number">' + (i + 1) + '</div>', 
                        iconSize: [30, 30] 
                    })
                });
                marker.bindPopup(`
                    <div class="map-popup">
                        <div class="popup-name">${p.DBA || p.CompanyName}</div>
                        <div class="popup-address">${p.Street}, ${p.City}, ${p.State}</div>
                        <div class="popup-distance">${p.distanceMiles} miles</div>
                    </div>
                `);
                marker.addTo(tourMap);
                bounds.extend([p.GeoLat, p.GeoLong]);
            }
        });

        if (allPoints.length > 1) {
            // Optimize the route using 2-opt algorithm
            const optimizedPath = optimizePath(allPoints);
            
            // Extract coordinates for the polyline
            const pathCoords = optimizedPath.map(point => [point.lat, point.lng]);
            
            // Create polyline with the optimized path
            L.polyline(pathCoords, { color: '#8b4513', weight: 3 }).addTo(tourMap);
        }

        if (bounds.isValid()) {
            console.log('Fitting tour map to bounds');
            tourMap.fitBounds(bounds, { padding: [50, 50], maxZoom: 15 });
            // Force a redraw
            setTimeout(function() {
                tourMap.invalidateSize();
            }, 100);
        } else {
            console.log('No valid bounds, using default view');
            tourMap.setView([33.6846, -117.8265], 10);
            setTimeout(function() {
                tourMap.invalidateSize();
            }, 100);
        }
    } catch (e) {
        console.error('Error in updateTourMap:', e);
        showNotification('Error rendering tour map', 'error');
    }
}

// Revised optimization function
function optimizePath(points) {
    console.log('Optimizing path with nearest neighbor algorithm');
    
    // If we have fewer than 2 points (including start), no path needed
    if (points.length < 2) {
        return points;
    }
    
    // Extract the starting point - should be the first point in our path
    const startIndex = points.findIndex(p => p.isStartPoint);
    if (startIndex === -1) {
        console.error('No starting point found in points array');
        return points;
    }
    
    // Create a new ordered path starting with the starting point
    const path = [points[startIndex]];
    
    // Create a list of unvisited points (excluding starting point)
    const unvisited = points.filter(p => !p.isStartPoint);
    
    // Simple nearest neighbor algorithm
    let currentPoint = path[0];
    
    while (unvisited.length > 0) {
        // Find the nearest unvisited point
        let nearestIndex = -1;
        let minDistance = Infinity;
        
        for (let i = 0; i < unvisited.length; i++) {
            const distance = calculateDistance(
                currentPoint.lat, currentPoint.lng,
                unvisited[i].lat, unvisited[i].lng
            );
            
            if (distance < minDistance) {
                minDistance = distance;
                nearestIndex = i;
            }
        }
        
        // Add the nearest point to our path
        currentPoint = unvisited[nearestIndex];
        path.push(currentPoint);
        
        // Remove the point from unvisited
        unvisited.splice(nearestIndex, 1);
    }
    
    console.log('Path optimized with nearest neighbor');
    return path;
}

// Improved distance calculation function
function calculateDistance(lat1, lon1, lat2, lon2) {
    // Haversine formula for calculating distance between two points on Earth
    const R = 6371; // Earth's radius in km
    const dLat = (lat2 - lat1) * Math.PI / 180;
    const dLon = (lon2 - lon1) * Math.PI / 180;
    const a = 
        Math.sin(dLat/2) * Math.sin(dLat/2) +
        Math.cos(lat1 * Math.PI / 180) * Math.cos(lat2 * Math.PI / 180) * 
        Math.sin(dLon/2) * Math.sin(dLon/2);
    const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1-a));
    return R * c; // Distance in km
}

// Back to results
function backToResults() {
    const tourPlanContainer = document.getElementById('tourPlanContainer');
    if (tourPlanContainer) {
        tourPlanContainer.style.display = 'none';
    }
    
    const searchResults = document.querySelector('.search-results');
    if (searchResults) {
        searchResults.style.display = 'block';
    }
    
    const shareOptions = document.getElementById('shareOptions');
    if (shareOptions) {
        shareOptions.style.display = 'none';
    }
}

// Show share options
function showShareOptions() {
    const options = document.getElementById('shareOptions');
    if (options) {
        options.style.display = options.style.display === 'none' ? 'block' : 'none';
    }
}

// Share via SMS
function shareViaSMS() {
    const text = createShareText();
    if (navigator.share) {
        navigator.share({ title: 'My Tasting Tour', text: text })
            .catch(function(err) {
                console.error('Share error:', err);
                copyToClipboard(text);
                showNotification('Copied to clipboard', 'success');
            });
    } else {
        copyToClipboard(text);
        showNotification('Copied to clipboard', 'success');
    }
}

// Share via email
function shareViaEmail() {
    const text = createShareText();
    const subject = encodeURIComponent('My Tasting Tour');
    const body = encodeURIComponent(text);
    window.open('mailto:?subject=' + subject + '&body=' + body);
}

// Post to feed
function postToFeed() {
    const text = createShareText();
    showNotification('Posted to feed', 'success');
}

// Create share text
function createShareText() {
    let text = 'My Tasting Tour:\n\n';
    selectedProducers.sort(function(a, b) {
        return a.distance - b.distance;
    }).forEach(function(p, i) {
        const name = p.DBA || p.CompanyName;
        const address = p.Street + ', ' + p.City + ', ' + p.State + ' ' + p.Zip;
        text += (i + 1) + '. ' + name + '\n   ' + address + '\n';
        if (p.Phone) text += '   Phone: ' + formatPhoneNumber(p.Phone) + '\n';
        if (p.WebSite) text += '   Website: ' + formatWebsiteUrl(p.WebSite) + '\n';
        if (p.Hours) text += '   Hours: ' + p.Hours + '\n';
        text += '\n';
    });
    text += 'Created with Craft Review';
    return text;
}

// Copy to clipboard
function copyToClipboard(text) {
    if (navigator.clipboard) {
        navigator.clipboard.writeText(text)
            .catch(function(err) {
                console.error('Clipboard write failed:', err);
                fallbackCopyToClipboard(text);
            });
    } else {
        fallbackCopyToClipboard(text);
    }
}

// Fallback clipboard copy method
function fallbackCopyToClipboard(text) {
    const textarea = document.createElement('textarea');
    textarea.value = text;
    textarea.style.position = 'fixed';
    document.body.appendChild(textarea);
    textarea.select();
    try {
        document.execCommand('copy');
    } catch (err) {
        console.error('Fallback clipboard copy failed:', err);
    }
    document.body.removeChild(textarea);
}

// Show notification
function showNotification(message, type) {
    type = type || 'info';
    const modal = document.getElementById('notificationModal');
    if (!modal) return console.error('Notification modal not found');

    modal.className = 'notification ' + type;
    modal.textContent = message;
    modal.style.display = 'block';
    setTimeout(function() {
        modal.style.display = 'none';
    }, type === 'error' ? 5000 : 3000);
}

// Format phone number
function formatPhoneNumber(phone) {
    if (!phone) return '';
    const clean = phone.replace(/\D/g, '');
    return clean.length === 10 ? '(' + clean.slice(0, 3) + ') ' + clean.slice(3, 6) + '-' + clean.slice(6) : phone;
}

// Format website URL
function formatWebsiteUrl(url) {
    if (!url) return '';
    return url.match(/^https?:\/\//) ? url : 'https://' + url;
}

// Format social URL
function formatSocialUrl(platform, handle) {
    if (!handle) return '';
    handle = handle.replace('@', '').trim();
    const urls = {
        facebook: 'https://facebook.com/' + handle,
        instagram: 'https://instagram.com/' + handle,
        twitter: 'https://twitter.com/' + handle
    };
    return urls[platform] || 'https://' + platform + '.com/' + handle;
}