// feed.js - Social media feed functionality for Craft Review

// Global variables
let currentPage = 1;
let isLoading = false;
let hasMoreItems = true;

// Initialize feed when DOM is loaded
document.addEventListener('DOMContentLoaded', initFeed);

function initFeed() {
    console.log('Initializing feed...');

    // Load initial feed items
    loadFeedItems();

    // Set up event listeners
    setupEventListeners();

    // Set up scroll detection for infinite scroll
    setupInfiniteScroll();
}

function setupEventListeners() {
    // Refresh feed button
    const refreshButton = document.getElementById('refreshFeed');
    if (refreshButton) {
        refreshButton.addEventListener('click', () => {
            currentPage = 1;
            hasMoreItems = true;

            // Clear existing feed items
            const feedItemsContainer = document.getElementById('feedItems');
            if (feedItemsContainer) {
                feedItemsContainer.innerHTML = '';
            }

            // Hide end of feed message
            const endOfFeed = document.getElementById('endOfFeed');
            if (endOfFeed) {
                endOfFeed.style.display = 'none';
            }

            // Show loading indicator
            const loadingIndicator = document.getElementById('feedLoading');
            if (loadingIndicator) {
                loadingIndicator.style.display = 'flex';
            }

            // Load fresh items
            loadFeedItems();
        });
    }

    // Event delegation for feed item interactions
    document.addEventListener('click', function (event) {
        // Like button
        if (event.target.closest('.action-like')) {
            const likeButton = event.target.closest('.action-like');
            likeButton.classList.toggle('active');
        }

        // Comment button
        if (event.target.closest('.action-comment')) {
            const feedItem = event.target.closest('.feed-item');
            if (feedItem) {
                const reviewId = feedItem.dataset.reviewId;
                if (reviewId) {
                    console.log('Open comments for review:', reviewId);
                    // Future implementation: show comments modal
                }
            }
        }

        // Share button
        if (event.target.closest('.action-share')) {
            const feedItem = event.target.closest('.feed-item');
            if (feedItem) {
                const reviewId = feedItem.dataset.reviewId;
                const venueName = feedItem.querySelector('.venue-name').textContent;
                if (reviewId && venueName) {
                    shareReview(reviewId, venueName);
                }
            }
        }
    });
}

function setupInfiniteScroll() {
    // Detect when user has scrolled near the bottom
    window.addEventListener('scroll', () => {
        if (isLoading || !hasMoreItems) return;

        const scrollY = window.scrollY;
        const viewportHeight = window.innerHeight;
        const pageHeight = document.documentElement.scrollHeight;

        // Load more items when user scrolls to near bottom (200px from bottom)
        if (scrollY + viewportHeight >= pageHeight - 200) {
            loadFeedItems();
        }
    });
}

function loadFeedItems() {
    if (isLoading || !hasMoreItems) return;

    isLoading = true;

    // Show loading indicator
    const loadingIndicator = document.getElementById('feedLoading');
    if (loadingIndicator) {
        loadingIndicator.style.display = 'flex';
    }

    // Build API URL with pagination
    const apiUrl = `./api/feed.php?page=${currentPage}&limit=10`;

    console.log(`Loading feed items from: ${apiUrl}`);

    // Fetch feed items from API
    fetch(apiUrl)
        .then(response => {
            if (!response.ok) {
                throw new Error(`HTTP error! Status: ${response.status}`);
            }
            return response.json();
        })
        .then(data => {
            console.log('Feed data received:', data);

            // Hide loading indicator
            if (loadingIndicator) {
                loadingIndicator.style.display = 'none';
            }

            // Render feed items
            if (data.reviews && data.reviews.length > 0) {
                renderFeedItems(data.reviews);
                currentPage++;
            } else {
                hasMoreItems = false;
                showEndOfFeed();
            }

            isLoading = false;
        })
        .catch(error => {
            console.error('Error loading feed:', error);

            // Hide loading indicator
            if (loadingIndicator) {
                loadingIndicator.style.display = 'none';
            }

            // Show error message
            showNotification('Error loading feed. Please try again later.', 'error');

            isLoading = false;
        });
}

function renderFeedItems(reviews) {
    const feedItemsContainer = document.getElementById('feedItems');
    if (!feedItemsContainer) return;

    const template = document.getElementById('feedItemTemplate');
    const productTemplate = document.getElementById('productReviewTemplate');

    if (!template || !productTemplate) {
        console.error('Required templates not found!');
        return;
    }

    reviews.forEach(review => {
        // Clone the template
        const clone = document.importNode(template.content, true);
        const feedItem = clone.querySelector('.feed-item');

        // Set data attributes
        feedItem.dataset.reviewId = review.ReviewID;

        // Fill in user information
        const username = feedItem.querySelector('.username');
        if (username) username.textContent = review.UserID || 'Anonymous';

        // Format timestamp
        const timestamp = feedItem.querySelector('.timestamp');
        if (timestamp) timestamp.textContent = formatTimestamp(review.CreatedAt);

        // Venue information
        const venueName = feedItem.querySelector('.venue-name');
        if (venueName) venueName.textContent = review.CompanyName || review.DBA;

        const venueLocation = feedItem.querySelector('.venue-location');
        if (venueLocation) {
            venueLocation.textContent = `${review.City}, ${review.State}`;
        }

        // Venue photo
        const venuePhoto = feedItem.querySelector('.feed-item-venue-photo img');
        if (venuePhoto) {
            if (review.PhotoID) {
                // Check if PhotoID is already a path
                if (review.PhotoID.startsWith('/')) {
                    venuePhoto.src = review.PhotoID;
                } else {
                    venuePhoto.src = `/TastingTrail/uploads/photos/venue/${review.PhotoID}`;
                }
                venuePhoto.alt = `Photo of ${review.CompanyName || review.DBA}`;
            } else {
                // Use default placeholder from the correct path
                venuePhoto.src = '/TastingTrail/uploads/photos/venue/placeholder-venue.jpg';
                venuePhoto.alt = 'No photo available';
            }
        }

        // Venue rating
        const venueRating = feedItem.querySelector('.venue-rating');
        if (venueRating) venueRating.textContent = `${review.Rating.toFixed(1)}/5.0`;

        // Venue review text
        const venueReview = feedItem.querySelector('.venue-review');
        if (venueReview) venueReview.textContent = review.ReviewText;

        // Product reviews
        const productsContainer = feedItem.querySelector('.feed-item-products');
        if (productsContainer && review.Products && review.Products.length > 0) {
            review.Products.forEach(product => {
                // Clone the product template
                const productClone = document.importNode(productTemplate.content, true);
                const productReview = productClone.querySelector('.product-review');

                // Product photo
                const productPhoto = productReview.querySelector('.product-photo img');
                if (productPhoto) {
                    if (product.PhotoID) {
                        // Check if PhotoID is already a path
                        if (product.PhotoID.startsWith('/')) {
                            productPhoto.src = product.PhotoID;
                        } else {
                            productPhoto.src = `/TastingTrail/uploads/photos/product/${product.PhotoID}`;
                        }
                        productPhoto.alt = `Photo of ${product.Name}`;
                    } else {
                        // Use default placeholder from the correct path
                        productPhoto.src = '/TastingTrail/uploads/photos/product/placeholder-product.jpg';
                        productPhoto.alt = 'No photo available';
                    }
                }

                // Product name
                const productName = productReview.querySelector('.product-name');
                if (productName) productName.textContent = product.Name;

                // Product category
                const productCategory = productReview.querySelector('.product-category');
                if (productCategory) {
                    productCategory.textContent = product.Style ?
                        `${product.Category} - ${product.Style}` :
                        product.Category;
                }

                // Product rating
                const productRating = productReview.querySelector('.product-rating');
                if (productRating) productRating.textContent = `${product.Rating.toFixed(1)}/5.0`;

                // Product quick take
                const productQuickTake = productReview.querySelector('.product-quick-take');
                if (productQuickTake) productQuickTake.textContent = product.QuickTake || '';

                // Add to products container
                productsContainer.appendChild(productReview);
            });
        } else if (productsContainer) {
            productsContainer.style.display = 'none';
        }

        // Comment count
        const commentCount = feedItem.querySelector('.comment-count');
        if (commentCount) {
            // For now, just use a placeholder
            commentCount.textContent = 'View all comments';
        }

        // Add the feed item to the container
        feedItemsContainer.appendChild(feedItem);
    });
}

function formatTimestamp(timestamp) {
    if (!timestamp) return 'Unknown time';

    const date = new Date(timestamp);
    const now = new Date();

    // Check if date is valid
    if (isNaN(date.getTime())) return 'Invalid date';

    const diffMs = now - date;
    const diffSecs = Math.floor(diffMs / 1000);
    const diffMins = Math.floor(diffSecs / 60);
    const diffHours = Math.floor(diffMins / 60);
    const diffDays = Math.floor(diffHours / 24);

    if (diffSecs < 60) {
        return 'Just now';
    } else if (diffMins < 60) {
        return `${diffMins} ${diffMins === 1 ? 'minute' : 'minutes'} ago`;
    } else if (diffHours < 24) {
        return `${diffHours} ${diffHours === 1 ? 'hour' : 'hours'} ago`;
    } else if (diffDays < 7) {
        return `${diffDays} ${diffDays === 1 ? 'day' : 'days'} ago`;
    } else {
        // Format date: Month Day, Year
        return date.toLocaleDateString('en-US', {
            year: 'numeric',
            month: 'short',
            day: 'numeric'
        });
    }
}

function showEndOfFeed() {
    const endOfFeed = document.getElementById('endOfFeed');
    if (endOfFeed) {
        endOfFeed.style.display = 'block';
    }
}

function shareReview(reviewId, venueName) {
    // Check if Web Share API is available
    if (navigator.share) {
        navigator.share({
            title: `Craft Review: ${venueName}`,
            text: `Check out this review of ${venueName} on Craft Review!`,
            url: `${window.location.origin}/review-details.html?id=${reviewId}`
        })
            .then(() => console.log('Successfully shared'))
            .catch((error) => console.error('Error sharing:', error));
    } else {
        // Fallback for browsers that don't support Web Share API
        // Copy link to clipboard
        const url = `${window.location.origin}/review-details.html?id=${reviewId}`;
        navigator.clipboard.writeText(url)
            .then(() => {
                showNotification('Link copied to clipboard!', 'success');
            })
            .catch(err => {
                console.error('Failed to copy link:', err);
                showNotification('Failed to copy link', 'error');
            });
    }
}

function showNotification(message, type = 'info') {
    const notification = document.getElementById('notificationModal');
    if (!notification) return;

    notification.textContent = message;
    notification.className = 'notification';
    notification.classList.add(type);
    notification.style.display = 'block';

    // Hide notification after 3 seconds
    setTimeout(() => {
        notification.style.display = 'none';
    }, 3000);
}