// location.js - Handles location and geocoding functionality

let map;
let marker;
let geocoder;

// Initialize location functionality
document.addEventListener('DOMContentLoaded', initLocationFunctions);

function initLocationFunctions() {
    // Set up event listeners for location features
    const getCurrentLocationBtn = document.getElementById('getCurrentLocation');
    const locationInput = document.getElementById('locationSearch');

    if (getCurrentLocationBtn) {
        getCurrentLocationBtn.addEventListener('click', getUserLocation);
    }

    if (locationInput) {
        // Simple debounce for location search
        let timeout = null;
        locationInput.addEventListener('input', function () {
            clearTimeout(timeout);
            timeout = setTimeout(() => {
                const query = this.value.trim();
                if (query.length > 2) {
                    searchLocations(query);
                }
            }, 500);
        });
    }

    // Initialize map
    initMap();
}

function initMap() {
    const mapContainer = document.getElementById('map');
    
    if (!mapContainer) return;
    
    // Initialize Leaflet map with a default view
    map = L.map('map').setView([33.6846, -117.8265], 13); // Huntington Beach coordinates
    
    L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
        attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
    }).addTo(map);
    
    // Add a marker that can be dragged
    marker = L.marker([33.6846, -117.8265], {
        draggable: true
    }).addTo(map);
    
    // When marker is dragged, update coordinates
    marker.on('dragend', function(event) {
        const position = marker.getLatLng();
        updateCoordinates(position.lat, position.lng);
        reverseGeocode(position.lat, position.lng);
    });
    
    // When map is clicked, move marker and update coordinates
    map.on('click', function(e) {
        marker.setLatLng(e.latlng);
        updateCoordinates(e.latlng.lat, e.latlng.lng);
        reverseGeocode(e.latlng.lat, e.latlng.lng);
    });
}

function getUserLocation() {
    if (!navigator.geolocation) {
        showNotification('Geolocation is not supported by your browser', 'error');
        return;
    }
    
    showNotification('Fetching your location...', 'info');
    
    navigator.geolocation.getCurrentPosition(
        position => {
            const { latitude, longitude } = position.coords;
            
            // Update coordinates and map position
            updateCoordinates(latitude, longitude);
            updateMapPosition(latitude, longitude);
            
            // Check if user is at a craft producer location
            checkNearbyProducers(latitude, longitude);
            
            // Also perform reverse geocoding to get address as fallback
            reverseGeocode(latitude, longitude);
        },
        error => {
            console.error('Geolocation error:', error);
            let errorMessage = 'Unable to get your location';
            
            switch (error.code) {
                case error.PERMISSION_DENIED:
                    errorMessage = 'You denied the request for geolocation';
                    break;
                case error.POSITION_UNAVAILABLE:
                    errorMessage = 'Location information is unavailable';
                    break;
                case error.TIMEOUT:
                    errorMessage = 'The request to get your location timed out';
                    break;
            }
            
            showNotification(errorMessage, 'error');
        }
    );
}

function checkNearbyProducers(latitude, longitude) {
    // Set a smaller radius (e.g., 0.2 km) to check if user is at a specific establishment
    const establishmentRadius = 0.2; // 200 meters

    // Limit coordinate precision to match database field size
    const formattedLat = parseFloat(latitude.toFixed(10));
    const formattedLong = parseFloat(longitude.toFixed(10));
    
    showNotification('Checking if you\'re at a craft producer...', 'info');
    
    // Use absolute path from domain root
    const apiUrl = `./api/nearby-producers.php?latitude=${formattedLat}&longitude=${formattedLong}&radius=${establishmentRadius}`;
    
    console.log('Calling API:', apiUrl); // Debug log
    
    // Call the API to find nearby producers
    fetch(apiUrl)
        .then(response => {
            console.log('API response status:', response.status); // Debug log
            if (!response.ok) {
                throw new Error(`Network response was not ok: ${response.status}`);
            }
            return response.json();
        })
        .then(data => {
            console.log('API data:', data); // Debug log
            if (data.producers && data.producers.length > 0) {
                // Sort by distance to get the closest producer
                data.producers.sort((a, b) => a.distance - b.distance);
                const closestProducer = data.producers[0];
                
                // If user is very close to a producer (within 200m), assume they're at that location
                if (closestProducer.distance <= establishmentRadius) {
                    // They're at an establishment - select and display it
                    selectProducerLocation(closestProducer);
                    showNotification(`You appear to be at ${closestProducer.DBA || closestProducer.CompanyName}`, 'success');
                } else {
                    // Show nearby producers for selection
                    displayLocationResults(data.producers.map(producer => ({
                        name: producer.DBA || producer.CompanyName,
                        address: `${producer.Street}, ${producer.City}, ${producer.State} ${producer.Zip}`,
                        latitude: producer.GeoLat,
                        longitude: producer.GeoLong,
                        distance: producer.distance.toFixed(1),
                        producer: producer // Pass the full producer object
                    })));
                }
            } else {
                showNotification('No craft producers found nearby', 'info');
            }
        })
        .catch(error => {
            console.error('Error checking nearby producers:', error);
            showNotification(`Error checking nearby locations: ${error.message}. Please check the console for details.`, 'error');
        });
}

function updateMapPosition(latitude, longitude) {
    // Update the map with the new coordinates
    if (map && marker) {
        // Update the map center and zoom
        map.setView([latitude, longitude], 15);
        
        // Move the marker to the new position
        marker.setLatLng([latitude, longitude]);
        
        // Update the coordinates in the form
        updateCoordinates(latitude, longitude);
    } else {
        console.error('Map or marker not initialized');
    }
}

// Helper function to update coordinate fields
function updateCoordinates(latitude, longitude) {
    document.getElementById('latitude').value = latitude;
    document.getElementById('longitude').value = longitude;
    
    // Optionally display coordinates in a readable format
    const selectedLocationElement = document.getElementById('selectedLocation');
    if (selectedLocationElement && !selectedLocationElement.classList.contains('producer-selected')) {
        selectedLocationElement.innerHTML = `
            <div>Coordinates: ${latitude.toFixed(6)}, ${longitude.toFixed(6)}</div>
        `;
    }
}

function searchLocations(query) {
    // Get the current map center or user's location
    let latitude, longitude;
    
    if (map && marker) {
        const position = marker.getLatLng();
        latitude = position.lat;
        longitude = position.lng;
    } else {
        // Use Huntington Beach coordinates as default
        latitude = 42.6846;
        longitude = -420.8265;
    }

     // Limit coordinate precision to match database field size
    const formattedLat = parseFloat(latitude.toFixed(10));
    const formattedLong = parseFloat(longitude.toFixed(10));
    
    showNotification('Searching nearby producers...', 'info');
    
    // Use absolute path from domain root
    const apiUrl = `./api/nearby-producers.php?latitude=${formattedLat}&longitude=${formattedLong}&radius=10&query=${encodeURIComponent(query)}`;
    
    console.log('Searching locations with:', apiUrl); // Debug log
    
    // Call the API
    fetch(apiUrl)
        .then(response => {
            console.log('Search response status:', response.status); // Debug log
            if (!response.ok) {
                throw new Error(`Network response was not ok: ${response.status}`);
            }
            return response.json();
        })
        .then(data => {
            console.log('Search results:', data); // Debug log
            if (data.producers && data.producers.length > 0) {
                displayLocationResults(data.producers.map(producer => ({
                    name: producer.DBA || producer.CompanyName,
                    address: `${producer.Street}, ${producer.City}, ${producer.State} ${producer.Zip}`,
                    latitude: producer.GeoLat,
                    longitude: producer.GeoLong,
                    distance: producer.distance.toFixed(1),
                    producer: producer // Pass the full producer object
                })));
            } else {
                showNotification('No producers found nearby', 'warning');
            }
        })
        .catch(error => {
            console.error('Error searching for locations:', error);
            showNotification(`Error searching for locations: ${error.message}`, 'error');
        });
}

function displayLocationResults(results) {
    // Create or get results container
    let resultsContainer = document.querySelector('.location-results');

    if (!resultsContainer) {
        resultsContainer = document.createElement('div');
        resultsContainer.className = 'location-results';

        // Insert after the location input
        const locationInput = document.getElementById('locationSearch');
        locationInput.parentNode.insertBefore(resultsContainer, locationInput.nextSibling);
    }

    // Clear previous results
    resultsContainer.innerHTML = '';

    // If no results, show message and hide container
    if (!results || results.length === 0) {
        resultsContainer.style.display = 'none';
        return;
    }

    // Display results
    results.forEach(result => {
        const resultItem = document.createElement('div');
        resultItem.className = 'location-result-item';
        resultItem.innerHTML = `
            <div class="location-name">${result.name}</div>
            <div class="location-address">${result.address}</div>
            <div class="location-distance">${result.distance} km away</div>
        `;

        // Add click event to select this location
        resultItem.addEventListener('click', () => {
            if (result.producer) {
                selectProducerLocation(result.producer);
            } else {
                selectLocation(result);
            }
            resultsContainer.style.display = 'none';
        });

        resultsContainer.appendChild(resultItem);
    });

    // Show the results container
    resultsContainer.style.display = 'block';
}

function selectProducerLocation(producer) {
    // Update the input with the selected producer name
    const locationInput = document.getElementById('locationSearch');
    locationInput.value = producer.DBA || producer.CompanyName;

    // Update hidden coordinates inputs
    document.getElementById('latitude').value = producer.GeoLat;
    document.getElementById('longitude').value = producer.GeoLong;

    // Create a rich display of the establishment data
    const selectedLocationElement = document.getElementById('selectedLocation');
    if (selectedLocationElement) {
        const socialLinks = [];
        
        if (producer.FB) socialLinks.push(`<a href="${formatSocialUrl('facebook', producer.FB)}" target="_blank">Facebook</a>`);
        if (producer.IG) socialLinks.push(`<a href="${formatSocialUrl('instagram', producer.IG)}" target="_blank">Instagram</a>`);
        if (producer.X) socialLinks.push(`<a href="${formatSocialUrl('twitter', producer.X)}" target="_blank">X/Twitter</a>`);
        
        const socialLinksHtml = socialLinks.length > 0 
            ? `<div class="producer-social">${socialLinks.join(' | ')}</div>` 
            : '';
            
        const amenities = [];
        if (producer.TastingRoom === 'Y') amenities.push('Tasting Room');
        if (producer.Tours === 'Y') amenities.push('Tours Available');
        if (producer.Appointments === 'Y') amenities.push('Appointments Required');
        if (producer.Accessible) amenities.push(producer.Accessible);
        
        const amenitiesHtml = amenities.length > 0 
            ? `<div class="producer-amenities"><strong>Amenities:</strong> ${amenities.join(', ')}</div>` 
            : '';
            
        const hoursHtml = producer.Hours 
            ? `<div class="producer-hours"><strong>Hours:</strong> ${producer.Hours}</div>` 
            : '';
            
        const websiteHtml = producer.WebSite 
            ? `<div class="producer-website"><a href="${formatWebsiteUrl(producer.WebSite)}" target="_blank">Visit Website</a></div>` 
            : '';
            
        const phoneHtml = producer.Phone 
            ? `<div class="producer-phone"><strong>Phone:</strong> <a href="tel:${producer.Phone}">${formatPhoneNumber(producer.Phone)}</a></div>` 
            : '';
        
        selectedLocationElement.innerHTML = `
            <div class="producer-info">
                <div class="producer-name">${producer.DBA || producer.CompanyName}</div>
                <div class="producer-address">${producer.Street}, ${producer.City}, ${producer.State} ${producer.Zip}</div>
                ${phoneHtml}
                ${websiteHtml}
                ${hoursHtml}
                ${amenitiesHtml}
                ${socialLinksHtml}
            </div>
        `;
        
        // Add a special class to style the producer info nicely
        selectedLocationElement.classList.add('producer-selected');
    }

    // Update map position
    updateMapPosition(producer.GeoLat, producer.GeoLong);
}

function selectLocation(location) {
    // Update the input with the selected location name
    const locationInput = document.getElementById('locationSearch');
    locationInput.value = location.name;

    // Update hidden coordinates inputs
    document.getElementById('latitude').value = location.latitude;
    document.getElementById('longitude').value = location.longitude;

    // Update the selected location display
    const selectedLocationElement = document.getElementById('selectedLocation');
    if (selectedLocationElement) {
        selectedLocationElement.textContent = `Selected: ${location.name} (${location.address})`;
        // Remove producer-selected class if it exists
        selectedLocationElement.classList.remove('producer-selected');
    }

    // Update map position
    updateMapPosition(location.latitude, location.longitude);
}

// Helper function to format social media URLs
function formatSocialUrl(platform, handle) {
    if (!handle) return '';
    
    handle = handle.replace('@', '').trim();
    
    switch(platform) {
        case 'facebook':
            return `https://facebook.com/${handle}`;
        case 'instagram':
            return `https://instagram.com/${handle}`;
        case 'twitter':
            return `https://twitter.com/${handle}`;
        default:
            return `https://${platform}.com/${handle}`;
    }
}

// Helper function to format website URL
function formatWebsiteUrl(url) {
    if (!url) return '';
    
    // Add https:// if missing
    if (!url.startsWith('http://') && !url.startsWith('https://')) {
        return `https://${url}`;
    }
    
    return url;
}

// Helper function to format phone number
function formatPhoneNumber(phone) {
    if (!phone) return '';
    
    // Simple formatting - could be enhanced based on your needs
    return phone;
}

function reverseGeocode(latitude, longitude) {
    showNotification('Looking up address...', 'info');
    
    // Use OpenStreetMap's Nominatim service for reverse geocoding
    fetch(`https://nominatim.openstreetmap.org/reverse?format=json&lat=${latitude}&lon=${longitude}&zoom=18&addressdetails=1`)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            if (data && data.display_name) {
                // Create a location object from the response
                const location = {
                    name: extractLocationName(data),
                    address: formatAddress(data),
                    latitude: latitude,
                    longitude: longitude
                };
                
                // Only update the UI if we haven't found a producer yet
                const selectedLocationElement = document.getElementById('selectedLocation');
                if (selectedLocationElement && !selectedLocationElement.classList.contains('producer-selected')) {
                    selectLocation(location);
                }
                
                // Optional: Search for producers near this location
                const locationInput = document.getElementById('locationSearch');
                if (locationInput && locationInput.value.trim() === '') {
                    searchLocations(''); // Search with empty query to find all nearby producers
                }
            } else {
                showNotification('Could not find address for this location', 'warning');
            }
        })
        .catch(error => {
            console.error('Error reverse geocoding:', error);
            showNotification('Error getting address for this location', 'error');
        });
}

// Helper function to extract a meaningful name from Nominatim response
function extractLocationName(nominatimData) {
    // Try to get the most relevant name for the location
    if (nominatimData.address) {
        // Check for establishment names first
        if (nominatimData.address.attraction) return nominatimData.address.attraction;
        if (nominatimData.address.amenity) return nominatimData.address.amenity;
        if (nominatimData.address.shop) return nominatimData.address.shop;
        
        // If no establishment name, use the address components
        if (nominatimData.address.road) {
            let name = nominatimData.address.road;
            if (nominatimData.address.house_number) {
                name = `${nominatimData.address.house_number} ${name}`;
            }
            return name;
        }
        
        // Fall back to neighborhood or locality
        if (nominatimData.address.neighbourhood) return nominatimData.address.neighbourhood;
        if (nominatimData.address.suburb) return nominatimData.address.suburb;
        if (nominatimData.address.village) return nominatimData.address.village;
        if (nominatimData.address.town) return nominatimData.address.town;
        if (nominatimData.address.city) return nominatimData.address.city;
    }
    
    // If we can't extract a meaningful name, use a generic one with coordinates
    return `Location (${nominatimData.lat.toFixed(4)}, ${nominatimData.lon.toFixed(4)})`;
}

// Format a nicely structured address from Nominatim data
function formatAddress(nominatimData) {
    if (!nominatimData.address) return nominatimData.display_name;
    
    const addr = nominatimData.address;
    let parts = [];
    
    // Street address
    if (addr.house_number && addr.road) {
        parts.push(`${addr.house_number} ${addr.road}`);
    } else if (addr.road) {
        parts.push(addr.road);
    }
    
    // City/town, state, postal code
    let cityPart = '';
    if (addr.city || addr.town || addr.village) {
        cityPart = addr.city || addr.town || addr.village;
    }
    
    if (cityPart && addr.state) {
        if (addr.postcode) {
            parts.push(`${cityPart}, ${addr.state} ${addr.postcode}`);
        } else {
            parts.push(`${cityPart}, ${addr.state}`);
        }
    } else if (cityPart) {
        parts.push(cityPart);
    }
    
    // Country
    if (addr.country) {
        parts.push(addr.country);
    }
    
    return parts.join(', ');
}