// photos.js - Handles photo uploads and geocoding verification

// Initialize photo handling when DOM is loaded
document.addEventListener('DOMContentLoaded', initPhotoHandling);

function initPhotoHandling() {
    // Set up the establishment photo upload
    const establishmentPhotoInput = document.getElementById('establishmentPhoto');
    const establishmentPhotoUpload = document.getElementById('establishmentPhotoUpload');

    if (establishmentPhotoInput && establishmentPhotoUpload) {
        establishmentPhotoUpload.addEventListener('click', () => {
            establishmentPhotoInput.click();
        });

        establishmentPhotoInput.addEventListener('change', (e) => {
            handlePhotoUpload(e, 'establishment');
        });
    }

    // Product photo uploads are now handled in products.js
}

function handlePhotoUpload(event, type) {
    const file = event.target.files[0];
    if (!file) return;

    // Check if file is an image
    if (!file.type.startsWith('image/')) {
        showNotification('Please select an image file', 'error');
        return;
    }

    // Check file size (limit to 10MB)
    if (file.size > 10 * 1024 * 1024) {
        showNotification('Image size should be less than 10MB', 'error');
        return;
    }

    // Show preview of the image
    displayPhotoPreview(file, type);

    // If it's an establishment photo, verify the geocoding
    if (type === 'establishment') {
        verifyPhotoLocation(file);
    }
}

function displayPhotoPreview(file, type) {
    const reader = new FileReader();
    const previewElement = document.getElementById(`${type}PhotoPreview`);

    if (!previewElement) return;

    reader.onload = (e) => {
        previewElement.innerHTML = `
            <img src="${e.target.result}" alt="${type} photo">
            <div class="photo-info">
                <span class="photo-name">${file.name}</span>
                <span class="photo-size">${formatFileSize(file.size)}</span>
            </div>
        `;
    };

    reader.readAsDataURL(file);
}

function formatFileSize(bytes) {
    if (bytes < 1024) return bytes + ' bytes';
    else if (bytes < 1024 * 1024) return (bytes / 1024).toFixed(1) + ' KB';
    else return (bytes / (1024 * 1024)).toFixed(1) + ' MB';
}

function verifyPhotoLocation(file) {
    // In a real implementation, this would check the photo's EXIF data
    // for location and compare with the selected location

    // First, get the current selected location
    const latitude = document.getElementById('latitude').value;
    const longitude = document.getElementById('longitude').value;

    if (!latitude || !longitude) {
        showNotification('Please select a location before uploading an establishment photo', 'warning');
        return;
    }

    // Show a loading message
    showNotification('Verifying photo location...', 'info');

    // For this demo, we'll simulate reading EXIF data
    setTimeout(() => {
        extractExifData(file)
            .then(exifData => {
                if (exifData && verifyCoordinatesMatch(exifData, latitude, longitude)) {
                    showNotification('Photo location verified successfully!', 'success');
                } else {
                    showNotification('Warning: Photo location does not match selected location', 'warning');
                }
            })
            .catch(error => {
                console.error('Error extracting EXIF data:', error);
                showNotification('Could not verify photo location', 'error');
            });
    }, 1500);
}

function extractExifData(file) {
    // In a real implementation, this would extract EXIF data including GPS coordinates
    // For this demo, we'll simulate the process

    return new Promise((resolve, reject) => {
        try {
            // Simulate extracting EXIF data
            // In a real implementation, you would use a library like exif-js

            // 80% chance the photo has EXIF data with GPS in this simulation
            const hasGpsData = Math.random() < 0.8;

            if (hasGpsData) {
                // Get the selected location coordinates
                const selectedLatitude = parseFloat(document.getElementById('latitude').value);
                const selectedLongitude = parseFloat(document.getElementById('longitude').value);

                // Add a small random offset to simulate a photo taken nearby (within ~100 meters)
                const latOffset = (Math.random() - 0.5) * 0.002; // ~100m in latitude
                const lngOffset = (Math.random() - 0.5) * 0.002; // ~100m in longitude

                const exifData = {
                    latitude: selectedLatitude + latOffset,
                    longitude: selectedLongitude + lngOffset,
                    timestamp: new Date().toISOString()
                };

                resolve(exifData);
            } else {
                // Simulate a photo without GPS data
                resolve(null);
            }
        } catch (error) {
            reject(error);
        }
    });
}

function verifyCoordinatesMatch(exifData, selectedLatitude, selectedLongitude) {
    // Convert string coordinates to numbers
    const lat1 = parseFloat(exifData.latitude);
    const lng1 = parseFloat(exifData.longitude);
    const lat2 = parseFloat(selectedLatitude);
    const lng2 = parseFloat(selectedLongitude);

    // Calculate distance between the two points (Haversine formula)
    const distance = calculateDistance(lat1, lng1, lat2, lng2);

    // Consider it a match if within 500 meters
    return distance <= 0.5; // 0.5 km = 500 meters
}

function calculateDistance(lat1, lon1, lat2, lon2) {
    // Haversine formula to calculate distance between two points on Earth
    const R = 6371; // Radius of Earth in kilometers
    const dLat = (lat2 - lat1) * Math.PI / 180;
    const dLon = (lon2 - lon1) * Math.PI / 180;

    const a =
        Math.sin(dLat / 2) * Math.sin(dLat / 2) +
        Math.cos(lat1 * Math.PI / 180) * Math.cos(lat2 * Math.PI / 180) *
        Math.sin(dLon / 2) * Math.sin(dLon / 2);

    const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    const distance = R * c; // Distance in kilometers

    return distance;
}

// Function to save the photo (would connect to backend in real implementation)
async function savePhoto(photoFile, type) {
    if (!photoFile) return null;

    // In a real implementation, this would upload the photo to a server
    // For this demo, we'll simulate saving and return a mock ID

    return new Promise((resolve) => {
        setTimeout(() => {
            // Generate a random ID for the photo
            const photoId = `${type}-${Date.now()}-${Math.floor(Math.random() * 1000)}`;
            console.log(`Photo ${photoId} saved successfully`);
            resolve(photoId);
        }, 800);
    });
}