﻿// products.js - Handles dynamic product reviews functionality

// Initialize product functionality when DOM is loaded
document.addEventListener('DOMContentLoaded', initProductFunctionality);

function initProductFunctionality() {
    // Set up the tasting count selector
    const tastingCountSelect = document.getElementById('tastingCount');
    if (tastingCountSelect) {
        tastingCountSelect.addEventListener('change', updateProductForms);

        // Initialize with the default value (1 product)
        updateProductForms();
    }

    // Set up character counts for text inputs
    setupCharacterCounts();

    // Set up rating sliders
    setupRatingSliders();

    // Set up product collapse toggles
    setupProductCollapseToggles();

    // IMPORTANT FIX: Initialize the photo upload for the first product
    // which is always present on the page load
    setupProductPhotoUpload(1);
}

function updateProductForms() {
    const tastingCount = parseInt(document.getElementById('tastingCount').value);
    const productContainer = document.getElementById('productReviewsContainer');

    // Get the current number of product forms
    const currentProductForms = productContainer.querySelectorAll('.product-review').length;

    // If we need to add more forms
    if (tastingCount > currentProductForms) {
        for (let i = currentProductForms + 1; i <= tastingCount; i++) {
            addProductForm(i);
        }
    }
    // If we need to remove forms
    else if (tastingCount < currentProductForms) {
        for (let i = currentProductForms; i > tastingCount; i--) {
            const formToRemove = productContainer.querySelector(`.product-review[data-product-index="${i}"]`);
            if (formToRemove) {
                productContainer.removeChild(formToRemove);
            }
        }
    }

    // Update character counts and sliders for all visible forms
    setupCharacterCounts();
    setupRatingSliders();
    setupProductCollapseToggles();
}

function addProductForm(index) {
    const productContainer = document.getElementById('productReviewsContainer');

    // Create the new product form
    const newProductForm = document.createElement('div');
    newProductForm.className = 'product-review';
    newProductForm.setAttribute('data-product-index', index);

    newProductForm.innerHTML = `
        <div class="product-header">
            <h4>Product ${index} <span class="required">*</span></h4>
            <div class="product-collapse-toggle" data-product-index="${index}">
                <span class="toggle-icon">−</span>
            </div>
        </div>
        
        <div class="product-body">
            <!-- Product Name -->
            <div class="form-subsection">
                <label for="productName${index}">Product Name</label>
                <input type="text" id="productName${index}" class="product-name" placeholder="Enter product name">
            </div>
            
            <!-- Category Section -->
            <div class="form-subsection">
                <label for="category${index}">Category</label>
                <select id="category${index}" class="product-category">
                    <option value="">Select a category</option>
                    <option value="Beer">Beer</option>
                    <option value="Wine">Wine</option>
                    <option value="Mead">Mead</option>
                    <option value="Cider">Cider</option>
                    <option value="Distilled Spirit">Distilled Spirit</option>
                    <option value="Seltzer">Seltzer</option>
                    <option value="Kombucha">Kombucha</option>
                    <option value="Other">Other</option>
                </select>
            </div>
            
            <!-- Style Section -->
            <div class="form-subsection">
                <label for="style${index}">Style</label>
                <select id="style${index}" class="product-style">
                    <option value="">Select a style</option>
                    <!-- Dynamic options will be loaded based on category -->
                </select>
            </div>
            
            <!-- Product Rating -->
            <div class="form-subsection">
                <label for="productRating${index}">Rating</label>
                <div class="rating-slider">
                    <input type="range" id="productRating${index}" class="product-rating" min="1" max="5" step="0.1" value="3">
                    <div class="rating-value"><span id="productRatingValue${index}">3.0</span>/5.0</div>
                </div>
            </div>
            
            <!-- Quick Take -->
            <div class="form-subsection">
                <label for="quickTake${index}">Quick Take</label>
                <input type="text" id="quickTake${index}" class="product-quick-take" placeholder="Sum it up in a short phrase..." maxlength="50">
                <div class="char-count"><span id="quickTakeCount${index}">0</span>/50</div>
            </div>
            
            <!-- Product Photo -->
            <div class="form-subsection">
                <label for="productPhoto${index}">Product Photo</label>
                <div class="photo-upload" id="productPhotoUpload${index}">
                    <img src="images/camera.svg" alt="Camera icon">
                    <span>Add product photo</span>
                    <input type="file" id="productPhoto${index}" class="product-photo" accept="image/*" capture="environment">
                </div>
                <div id="productPhotoPreview${index}" class="photo-preview"></div>
            </div>
        </div>
    `;

    productContainer.appendChild(newProductForm);

    // Set up category change listeners for the new form
    setupCategoryListeners(index);

    // Set up photo upload for the new form
    setupProductPhotoUpload(index);
}

function setupCategoryListeners(index) {
    const categorySelect = document.getElementById(`category${index}`);
    const styleSelect = document.getElementById(`style${index}`);

    if (categorySelect && styleSelect) {
        // Define styles for each category (copied from app.js)
        const styleOptions = {
            'Beer': ['IPA', 'Stout', 'Pilsner', 'Lager', 'Wheat Beer', 'Sour', 'Porter', 'Pale Ale', 'Other'],
            'Wine': ['Red', 'White', 'Rosé', 'Sparkling', 'Dessert', 'Fortified', 'Orange', 'Natural', 'Other'],
            'Mead': ['Traditional', 'Melomel', 'Metheglin', 'Braggot', 'Cyser', 'Pyment', 'Other'],
            'Cider': ['Dry', 'Semi-Dry', 'Sweet', 'Perry', 'Fruit', 'Hopped', 'Spiced', 'Other'],
            'Distilled Spirit': ['Whiskey', 'Bourbon', 'Rum', 'Vodka', 'Gin', 'Tequila', 'Brandy', 'Liqueur', 'Other'],
            'Seltzer': ['Fruit', 'Berry', 'Citrus', 'Tropical', 'Herbal', 'Unflavored', 'Other'],
            'Kombucha': ['Original', 'Fruit', 'Flower', 'Herb', 'Spiced', 'Other'],
            'Other': ['Non-Alcoholic', 'Coffee', 'Tea', 'Mixed Drink', 'Cocktail', 'Other']
        };

        // Update styles when category changes
        categorySelect.addEventListener('change', () => {
            // Clear existing options
            styleSelect.innerHTML = '<option value="">Select a style</option>';

            // Get selected category
            const selectedCategory = categorySelect.value;

            // Add new options if category is selected
            if (selectedCategory && styleOptions[selectedCategory]) {
                styleOptions[selectedCategory].forEach(style => {
                    const option = document.createElement('option');
                    option.value = style;
                    option.textContent = style;
                    styleSelect.appendChild(option);
                });
            }
        });
    }
}

function setupProductPhotoUpload(index) {
    const photoInput = document.getElementById(`productPhoto${index}`);
    const photoUpload = document.getElementById(`productPhotoUpload${index}`);

    if (photoInput && photoUpload) {
        // First, remove any existing event listeners to prevent duplicates
        const newPhotoUpload = photoUpload.cloneNode(true);
        photoUpload.parentNode.replaceChild(newPhotoUpload, photoUpload);

        // Then add our event listeners
        newPhotoUpload.addEventListener('click', () => {
            photoInput.click();
        });

        photoInput.addEventListener('change', (e) => {
            handleProductPhotoUpload(e, index);
        });

        // Add debugging to verify
        console.log(`Product photo upload #${index} initialized`);
    } else {
        console.error(`Could not initialize product photo upload #${index} - elements not found`);
    }
}

function handleProductPhotoUpload(event, index) {
    const file = event.target.files[0];
    if (!file) return;

    // Check if file is an image
    if (!file.type.startsWith('image/')) {
        showNotification('Please select an image file', 'error');
        return;
    }

    // Check file size (limit to 10MB)
    if (file.size > 10 * 1024 * 1024) {
        showNotification('Image size should be less than 10MB', 'error');
        return;
    }

    // Show preview of the image
    displayProductPhotoPreview(file, index);
}

function displayProductPhotoPreview(file, index) {
    const reader = new FileReader();
    const previewElement = document.getElementById(`productPhotoPreview${index}`);

    if (!previewElement) return;

    reader.onload = (e) => {
        previewElement.innerHTML = `
            <img src="${e.target.result}" alt="Product photo">
            <div class="photo-info">
                <span class="photo-name">${file.name}</span>
                <span class="photo-size">${formatFileSize(file.size)}</span>
            </div>
        `;
    };

    reader.readAsDataURL(file);
}

// Reuse the formatFileSize function from photos.js
function formatFileSize(bytes) {
    if (bytes < 1024) return bytes + ' bytes';
    else if (bytes < 1024 * 1024) return (bytes / 1024).toFixed(1) + ' KB';
    else return (bytes / (1024 * 1024)).toFixed(1) + ' MB';
}

function setupCharacterCounts() {
    // Establishment review character count
    const establishmentReview = document.getElementById('establishmentReview');
    const establishmentReviewCount = document.getElementById('establishmentReviewCount');

    if (establishmentReview && establishmentReviewCount) {
        establishmentReview.addEventListener('input', function () {
            establishmentReviewCount.textContent = this.value.length;
        });
    }

    // Quick take character counts for all products
    const quickTakes = document.querySelectorAll('.product-quick-take');

    quickTakes.forEach(quickTake => {
        const index = quickTake.id.replace('quickTake', '');
        const countElement = document.getElementById(`quickTakeCount${index}`);

        if (countElement) {
            quickTake.addEventListener('input', function () {
                countElement.textContent = this.value.length;
            });

            // Set initial count
            countElement.textContent = quickTake.value.length;
        }
    });
}

function setupRatingSliders() {
    // Establishment rating slider
    const establishmentRating = document.getElementById('establishmentRating');
    const establishmentRatingValue = document.getElementById('establishmentRatingValue');

    if (establishmentRating && establishmentRatingValue) {
        establishmentRating.addEventListener('input', function () {
            establishmentRatingValue.textContent = parseFloat(this.value).toFixed(1);
        });
    }

    // Product rating sliders
    const productRatings = document.querySelectorAll('.product-rating');

    productRatings.forEach(rating => {
        const index = rating.id.replace('productRating', '');
        const valueElement = document.getElementById(`productRatingValue${index}`);

        if (valueElement) {
            rating.addEventListener('input', function () {
                valueElement.textContent = parseFloat(this.value).toFixed(1);
            });
        }
    });
}

function setupProductCollapseToggles() {
    const toggles = document.querySelectorAll('.product-collapse-toggle');

    toggles.forEach(toggle => {
        toggle.addEventListener('click', function () {
            const productIndex = this.getAttribute('data-product-index');
            const productBody = this.closest('.product-review').querySelector('.product-body');
            const toggleIcon = this.querySelector('.toggle-icon');

            if (productBody.classList.contains('collapsed')) {
                productBody.classList.remove('collapsed');
                toggleIcon.textContent = '−';
            } else {
                productBody.classList.add('collapsed');
                toggleIcon.textContent = '+';
            }
        });
    });
}