// wizard-app.js - Main controller for the multi-step review wizard

// State management for the review data
const reviewState = {
    location: {
        producerId: null,
        venueName: '',
        venueAddress: '',
        latitude: null,
        longitude: null,
        isNewVenue: false
    },
    establishment: {
        rating: 3.5,
        review: '',
        photoFile: null,
        photoUrl: null
    },
    products: [],
    currentStep: 1,
    productCount: 1
};

// DOM content loaded - initialize the wizard
document.addEventListener('DOMContentLoaded', initWizard);

function initWizard() {
    setupEventListeners();
    updateProgressBar();

    // Set initial rating value display
    document.getElementById('establishmentRatingValue').textContent = reviewState.establishment.rating;

    // Initialize product count display
    updateProductCount(reviewState.productCount);
}

function setupEventListeners() {
    // Navigation buttons
    document.getElementById('toVenueStep').addEventListener('click', () => navigateToStep(2));
    document.getElementById('toProductsStep').addEventListener('click', () => navigateToStep(3));
    document.getElementById('toSubmitStep').addEventListener('click', () => navigateToStep(4));

    document.getElementById('backToLocationStep').addEventListener('click', () => navigateToStep(1));
    document.getElementById('backToVenueStep').addEventListener('click', () => navigateToStep(2));
    document.getElementById('backToProductsStep').addEventListener('click', () => navigateToStep(3));

    // Establishment review listeners
    document.getElementById('establishmentRating').addEventListener('input', handleEstablishmentRatingChange);
    document.getElementById('establishmentReview').addEventListener('input', handleEstablishmentReviewChange);

    // Product count buttons
    document.getElementById('increaseProductCount').addEventListener('click', () => updateProductCount(reviewState.productCount + 1));
    document.getElementById('decreaseProductCount').addEventListener('click', () => {
        if (reviewState.productCount > 1) {
            updateProductCount(reviewState.productCount - 1);
        }
    });

    // Submit review button
    document.getElementById('submitReview').addEventListener('click', handleSubmitReview);

    // Edit buttons
    document.getElementById('editVenue').addEventListener('click', () => navigateToStep(2));
}

// Navigation functions
function navigateToStep(stepNumber) {
    // Validate before proceeding to the next step
    if (stepNumber > reviewState.currentStep && !validateCurrentStep()) {
        return;
    }

    // Hide all steps
    const steps = document.querySelectorAll('.wizard-step');
    steps.forEach(step => step.classList.remove('active'));

    // Show the target step
    const targetStep = document.querySelector(`#step-${getStepId(stepNumber)}`);
    if (targetStep) {
        targetStep.classList.add('active');
        reviewState.currentStep = stepNumber;
        updateProgressBar();

        // Special handling for specific steps
        if (stepNumber === 3) {
            setupProductEntryForms();
        } else if (stepNumber === 4) {
            updateReviewSummary();
        }
    }
}

function getStepId(stepNumber) {
    switch (stepNumber) {
        case 1: return 'location';
        case 2: return 'venue';
        case 3: return 'product-count';
        case 4: return 'submit';
        default: return 'location';
    }
}

function updateProgressBar() {
    // Update the progress steps
    const progressSteps = document.querySelectorAll('.progress-step');
    const progressLabels = document.querySelectorAll('.progress-label');

    progressSteps.forEach((step, index) => {
        const stepNum = index + 1;
        step.classList.remove('active', 'completed');

        if (stepNum === reviewState.currentStep) {
            step.classList.add('active');
        } else if (stepNum < reviewState.currentStep) {
            step.classList.add('completed');
        }
    });

    progressLabels.forEach((label, index) => {
        const stepNum = index + 1;
        label.classList.remove('active');

        if (stepNum === reviewState.currentStep) {
            label.classList.add('active');
        }
    });
}

// Validation functions
function validateCurrentStep() {
    switch (reviewState.currentStep) {
        case 1:
            return validateLocationStep();
        case 2:
            return validateVenueStep();
        case 3:
            return validateProductsStep();
        default:
            return true;
    }
}

function validateLocationStep() {
    if (!reviewState.location.producerId && !reviewState.location.isNewVenue) {
        showNotification('Please select a venue or suggest a new one', 'error');
        return false;
    }
    return true;
}

function validateVenueStep() {
    if (!reviewState.establishment.photoFile) {
        showNotification('Please take a photo of the venue', 'error');
        return false;
    }

    if (!reviewState.establishment.review.trim()) {
        showNotification('Please provide a brief review of the venue', 'error');
        return false;
    }

    return true;
}

function validateProductsStep() {
    if (reviewState.products.length === 0) {
        showNotification('Please add at least one product', 'error');
        return false;
    }

    const missingInfo = [];

    for (let i = 0; i < reviewState.products.length; i++) {
        const product = reviewState.products[i];

        if (!product.name) missingInfo.push(`Product ${i + 1} name`);
        if (!product.category) missingInfo.push(`Product ${i + 1} category`);
        if (!product.rating) missingInfo.push(`Product ${i + 1} rating`);
    }

    if (missingInfo.length > 0) {
        showNotification(`Please complete: ${missingInfo.join(', ')}`, 'error');
        return false;
    }

    return true;
}

// Event handlers
function handleEstablishmentRatingChange(e) {
    const rating = parseFloat(e.target.value);
    reviewState.establishment.rating = rating;
    document.getElementById('establishmentRatingValue').textContent = rating.toFixed(1);
}

function handleEstablishmentReviewChange(e) {
    const review = e.target.value;
    reviewState.establishment.review = review;
    document.getElementById('establishmentReviewCount').textContent = review.length;
}

function updateProductCount(count) {
    // Limit to 5 products maximum
    count = Math.min(Math.max(count, 1), 5);

    reviewState.productCount = count;
    document.getElementById('productCount').textContent = count;

    // Ensure we have the right number of products in our state
    while (reviewState.products.length < count) {
        reviewState.products.push({
            id: null,
            name: '',
            category: '',
            style: '',
            rating: 3.0,
            quickTake: '',
            photoFile: null,
            photoUrl: null,
            isNewProduct: true
        });
    }

    // Remove extra products if count decreased
    if (reviewState.products.length > count) {
        reviewState.products = reviewState.products.slice(0, count);
    }
}

function setupProductEntryForms() {
    const container = document.getElementById('productEntryContainer');
    container.innerHTML = '';

    const template = document.getElementById('productEntryTemplate').innerHTML;

    // Create a form for each product
    for (let i = 0; i < reviewState.productCount; i++) {
        const productIndex = i + 1;
        const productHtml = template.replace(/{index}/g, productIndex);

        const tempDiv = document.createElement('div');
        tempDiv.innerHTML = productHtml;
        const productElement = tempDiv.firstElementChild;

        container.appendChild(productElement);

        // Set up event listeners for this product
        setupProductFormListeners(productIndex);
    }
}

function setupProductFormListeners(index) {
    // Product search input
    const searchInput = document.querySelector(`.product-card[data-product-index="${index}"] .product-search`);
    if (searchInput) {
        searchInput.addEventListener('input', (e) => handleProductSearch(e, index));
        searchInput.addEventListener('focus', (e) => showProductList(index));
    }

    // Product rating slider
    const ratingSlider = document.querySelector(`.product-card[data-product-index="${index}"] .product-rating`);
    if (ratingSlider) {
        ratingSlider.addEventListener('input', (e) => handleProductRatingChange(e, index));
    }

    // Product quick take
    const quickTakeInput = document.querySelector(`.product-card[data-product-index="${index}"] .product-quick-take`);
    if (quickTakeInput) {
        quickTakeInput.addEventListener('input', (e) => handleProductQuickTakeChange(e, index));
    }

    // Category select
    const categorySelect = document.querySelector(`.product-card[data-product-index="${index}"] .category-select`);
    if (categorySelect) {
        categorySelect.addEventListener('change', (e) => handleCategoryChange(e, index));
    }
}

function handleProductSearch(e, index) {
    const query = e.target.value.trim();

    // Show product list and details section based on input
    const productCard = document.querySelector(`.product-card[data-product-index="${index}"]`);
    const productList = productCard.querySelector('.product-list');
    const productDetails = productCard.querySelector('.product-details');

    if (query.length < 2) {
        // Hide product list for short queries
        productList.classList.add('hidden');
        return;
    }

    // Show the product list
    productList.classList.remove('hidden');

    // In a real implementation, this would fetch products from the database
    // For now, we'll simulate with dummy data
    setTimeout(() => {
        if (reviewState.location.producerId) {
            // Simulate fetching products for this venue
            const dummyProducts = [
                { id: 1, name: 'Hoppy Trails IPA', category: 'Beer', style: 'IPA' },
                { id: 2, name: 'Midnight Stout', category: 'Beer', style: 'Stout' },
                { id: 3, name: 'Hazy Days', category: 'Beer', style: 'Hazy IPA' },
                { id: 4, name: 'Summer Wheat', category: 'Beer', style: 'Wheat Beer' },
                { id: 5, name: 'Barrel Aged Porter', category: 'Beer', style: 'Porter' }
            ];

            // Filter based on search query
            const filteredProducts = dummyProducts.filter(p =>
                p.name.toLowerCase().includes(query.toLowerCase()));

            // Display filtered products
            displayProductList(filteredProducts, index, productList);
        }
    }, 300);

    // If it's a new product name
    if (query.length > 0) {
        reviewState.products[index - 1].name = query;
        reviewState.products[index - 1].isNewProduct = true;

        // Show the details section for a new product
        productDetails.classList.remove('hidden');
    }
}

function displayProductList(products, index, listElement) {
    listElement.innerHTML = '';

    if (products.length === 0) {
        const noResults = document.createElement('div');
        noResults.className = 'product-list-item';
        noResults.textContent = 'No matching products found. Enter details for a new product.';
        listElement.appendChild(noResults);
        return;
    }

    products.forEach(product => {
        const item = document.createElement('div');
        item.className = 'product-list-item';
        item.innerHTML = `
            <div class="product-name">${product.name} 
                <span class="product-tag">${product.category} - ${product.style}</span>
            </div>
        `;

        // Add click handler to select this product
        item.addEventListener('click', () => selectExistingProduct(product, index));

        listElement.appendChild(item);
    });
}

function selectExistingProduct(product, index) {
    const productIndex = index - 1;

    // Update the state
    reviewState.products[productIndex] = {
        ...reviewState.products[productIndex],
        id: product.id,
        name: product.name,
        category: product.category,
        style: product.style,
        isNewProduct: false
    };

    // Update the UI
    const productCard = document.querySelector(`.product-card[data-product-index="${index}"]`);
    const searchInput = productCard.querySelector('.product-search');
    const productList = productCard.querySelector('.product-list');
    const productDetails = productCard.querySelector('.product-details');
    const categorySelect = productCard.querySelector('.category-select');
    const styleSelect = productCard.querySelector('.style-select');

    // Update the search input
    searchInput.value = product.name;

    // Hide the product list
    productList.classList.add('hidden');

    // Show the details section
    productDetails.classList.remove('hidden');

    // Set the category and style
    if (categorySelect) {
        categorySelect.value = product.category;
        populateStyleOptions(styleSelect, product.category);
        styleSelect.value = product.style;
    }
}

function showProductList(index) {
    const query = document.querySelector(`.product-card[data-product-index="${index}"] .product-search`).value.trim();

    if (query.length >= 2) {
        const productList = document.querySelector(`.product-card[data-product-index="${index}"] .product-list`);
        productList.classList.remove('hidden');
    }
}

function handleProductRatingChange(e, index) {
    const productIndex = index - 1;
    const rating = parseFloat(e.target.value);

    // Update state
    reviewState.products[productIndex].rating = rating;

    // Update UI
    const ratingDisplay = document.querySelector(`.product-card[data-product-index="${index}"] .product-rating-value`);
    if (ratingDisplay) {
        ratingDisplay.textContent = rating.toFixed(1);
    }
}

function handleProductQuickTakeChange(e, index) {
    const productIndex = index - 1;
    const quickTake = e.target.value;

    // Update state
    reviewState.products[productIndex].quickTake = quickTake;

    // Update UI
    const countDisplay = document.querySelector(`.product-card[data-product-index="${index}"] .product-quick-take-count`);
    if (countDisplay) {
        countDisplay.textContent = quickTake.length;
    }
}

function handleCategoryChange(e, index) {
    const productIndex = index - 1;
    const category = e.target.value;

    // Update state
    reviewState.products[productIndex].category = category;

    // Update UI - populate style options
    const styleSelect = document.querySelector(`.product-card[data-product-index="${index}"] .style-select`);
    if (styleSelect) {
        populateStyleOptions(styleSelect, category);
    }
}

function populateStyleOptions(styleSelect, category) {
    // Clear existing options except the first one
    while (styleSelect.options.length > 1) {
        styleSelect.remove(1);
    }

    // Define styles for each category
    const styleOptions = {
        'Beer': ['IPA', 'Stout', 'Pilsner', 'Lager', 'Wheat Beer', 'Sour', 'Porter', 'Pale Ale', 'Other'],
        'Wine': ['Red', 'White', 'Ros', 'Sparkling', 'Dessert', 'Fortified', 'Orange', 'Natural', 'Other'],
        'Mead': ['Traditional', 'Melomel', 'Metheglin', 'Braggot', 'Cyser', 'Pyment', 'Other'],
        'Cider': ['Dry', 'Semi-Dry', 'Sweet', 'Perry', 'Fruit', 'Hopped', 'Spiced', 'Other'],
        'Distilled Spirit': ['Whiskey', 'Bourbon', 'Rum', 'Vodka', 'Gin', 'Tequila', 'Brandy', 'Liqueur', 'Other'],
        'Seltzer': ['Fruit', 'Berry', 'Citrus', 'Tropical', 'Herbal', 'Unflavored', 'Other'],
        'Kombucha': ['Original', 'Fruit', 'Flower', 'Herb', 'Spiced', 'Other'],
        'Other': ['Non-Alcoholic', 'Coffee', 'Tea', 'Mixed Drink', 'Cocktail', 'Other']
    };

    // Add new options if category is selected
    if (category && styleOptions[category]) {
        styleOptions[category].forEach(style => {
            const option = document.createElement('option');
            option.value = style;
            option.textContent = style;
            styleSelect.appendChild(option);
        });
    }
}

function updateReviewSummary() {
    // Update venue information
    document.getElementById('reviewVenueName').textContent = reviewState.location.venueName;
    document.getElementById('reviewVenueAddress').textContent = reviewState.location.venueAddress;
    document.getElementById('reviewVenueRating').textContent = reviewState.establishment.rating.toFixed(1);
    document.getElementById('reviewVenueComment').textContent = reviewState.establishment.review;

    // Display venue photo if available
    const venuePhotoPreview = document.getElementById('venuePhotoPreview');
    if (reviewState.establishment.photoUrl) {
        venuePhotoPreview.style.backgroundImage = `url(${reviewState.establishment.photoUrl})`;
        venuePhotoPreview.innerHTML = '';
    } else {
        venuePhotoPreview.textContent = 'No photo available';
    }

    // Update product count
    document.getElementById('reviewProductCount').textContent = reviewState.products.length;

    // Update product cards
    const productsContainer = document.getElementById('reviewProductsContainer');
    productsContainer.innerHTML = '';

    const template = document.getElementById('productReviewTemplate').innerHTML;

    reviewState.products.forEach((product, i) => {
        const productIndex = i + 1;
        const productHtml = template.replace(/{index}/g, productIndex);

        const tempDiv = document.createElement('div');
        tempDiv.innerHTML = productHtml;
        const productElement = tempDiv.firstElementChild;

        // Fill in product details
        productElement.querySelector('.review-product-name').textContent = product.name;
        productElement.querySelector('.review-product-category').textContent = `${product.category} - ${product.style || 'N/A'}`;
        productElement.querySelector('.review-product-rating').textContent = product.rating.toFixed(1);

        if (product.quickTake) {
            productElement.querySelector('.review-product-quick-take').textContent = product.quickTake;
        } else {
            productElement.querySelector('.review-product-quick-take').textContent = 'No description provided';
        }

        // Add status badge (new or established)
        const statusBadge = document.createElement('span');
        statusBadge.className = product.isNewProduct ? 'product-tag' : 'established-flag';
        statusBadge.textContent = product.isNewProduct ? 'New' : 'Established';
        productElement.querySelector('.product-status').appendChild(statusBadge);

        // Product photo
        const photoElement = productElement.querySelector('.review-product-photo');
        if (product.photoUrl) {
            photoElement.style.backgroundImage = `url(${product.photoUrl})`;
        } else {
            photoElement.textContent = 'No photo';
        }

        productsContainer.appendChild(productElement);
    });
}

function handleSubmitReview() {
    // Show loading state
    document.getElementById('submitReview').textContent = 'Posting...';
    document.getElementById('submitReview').disabled = true;

    // Call the saveReview function in wizard-db.js with the current review state
    saveReview(reviewState)
        .then(response => {
            console.log('Review submission successful:', response);
            showNotification('Review posted successfully!', 'success');

            // Reset form after delay
            setTimeout(() => {
                resetWizard();
            }, 2000);
        })
        .catch(error => {
            console.error('Review submission failed:', error);
            showNotification('Error posting review: ' + error.message, 'error');

            // Re-enable submit button
            document.getElementById('submitReview').textContent = 'Post Review';
            document.getElementById('submitReview').disabled = false;
        });
}

function resetWizard() {
    // Reset state
    reviewState.location = {
        producerId: null,
        venueName: '',
        venueAddress: '',
        latitude: null,
        longitude: null,
        isNewVenue: false
    };

    reviewState.establishment = {
        rating: 3.5,
        review: '',
        photoFile: null,
        photoUrl: null
    };

    reviewState.products = [];
    reviewState.productCount = 1;

    // Navigate to first step
    navigateToStep(1);

    // Reset form elements
    document.getElementById('establishmentRating').value = 3.5;
    document.getElementById('establishmentRatingValue').textContent = '3.5';
    document.getElementById('establishmentReview').value = '';
    document.getElementById('establishmentReviewCount').textContent = '0';
    document.getElementById('establishmentPhotoPreview').innerHTML = '';
    document.getElementById('establishmentPhotoPreview').classList.remove('has-image');

    // Reset product count
    document.getElementById('productCount').textContent = '1';

    // Reset submit button
    document.getElementById('submitReview').textContent = 'Post Review';
    document.getElementById('submitReview').disabled = false;

    // Clear location
    document.getElementById('locationResults').innerHTML = '';

    // Reset location selection button
    document.getElementById('toVenueStep').disabled = true;
}

// Utility functions
function showNotification(message, type = 'info') {
    const notification = document.getElementById('notification');
    notification.textContent = message;
    notification.className = 'notification';

    if (type) {
        notification.classList.add(type);
    }

    notification.style.display = 'block';

    // Hide after 3 seconds
    setTimeout(() => {
        notification.style.display = 'none';
    }, 3000);
}// JavaScript source code
