// wizard-db.js - Handles database operations for the review wizard

// Initialize IndexedDB for offline storage
let db;

document.addEventListener('DOMContentLoaded', initDatabase);

function initDatabase() {
    // Open/create an IndexedDB database for offline storage
    const request = indexedDB.open('CraftReviewDB', 1);

    request.onerror = function (event) {
        console.error('IndexedDB error:', event.target.error);
        showNotification('Error initializing offline storage', 'error');
    };

    request.onupgradeneeded = function (event) {
        db = event.target.result;

        // Create object stores (tables)
        if (!db.objectStoreNames.contains('pendingReviews')) {
            db.createObjectStore('pendingReviews', { keyPath: 'id', autoIncrement: true });
        }
    };

    request.onsuccess = function (event) {
        db = event.target.result;
        console.log('Database opened successfully');

        // Check for pending reviews to sync
        checkPendingReviews();
    };
}

// Function to save a review to the backend
function saveReview(reviewData) {
    return new Promise((resolve, reject) => {
        // Check if we're online
        if (navigator.onLine) {
            // Prepare the data for submission
            const formData = new FormData();

            // Add establishment data
            formData.append('producerId', reviewData.location.producerId);
            formData.append('venueName', reviewData.location.venueName);
            formData.append('venueAddress', reviewData.location.venueAddress);
            formData.append('latitude', reviewData.location.latitude);
            formData.append('longitude', reviewData.location.longitude);
            formData.append('isNewVenue', reviewData.location.isNewVenue);

            formData.append('rating', reviewData.establishment.rating);
            formData.append('review', reviewData.establishment.review);

            // Add establishment photo
            if (reviewData.establishment.photoFile) {
                formData.append('establishmentPhoto', reviewData.establishment.photoFile);
            }

            // Add products data
            formData.append('productCount', reviewData.products.length);

            reviewData.products.forEach((product, index) => {
                formData.append(`product[${index}][id]`, product.id);
                formData.append(`product[${index}][name]`, product.name);
                formData.append(`product[${index}][category]`, product.category);
                formData.append(`product[${index}][style]`, product.style);
                formData.append(`product[${index}][rating]`, product.rating);
                formData.append(`product[${index}][quickTake]`, product.quickTake);
                formData.append(`product[${index}][isNewProduct]`, product.isNewProduct);

                // Add product photo if available
                if (product.photoFile) {
                    formData.append(`product${index}photo`, product.photoFile);
                }
            });

            // Make the actual API call to submit the review
            console.log('Submitting review to server...');

            fetch('./api/submit-review.php', {
                method: 'POST',
                body: formData
            })
                .then(response => {
                    if (!response.ok) {
                        throw new Error(`Network response was not ok: ${response.status}`);
                    }
                    return response.json();
                })
                .then(data => {
                    console.log('Review submission successful:', data);
                    if (data.success) {
                        resolve(data);
                    } else if (data.error) {
                        reject(new Error(data.error));
                    } else {
                        resolve(data);
                    }
                })
                .catch(error => {
                    console.error('Error submitting review:', error);
                    reject(error);
                });
        } else {
            // We're offline, save to IndexedDB
            savePendingReview(reviewData)
                .then(() => {
                    showNotification('Review saved offline. It will be uploaded when you reconnect.', 'info');
                    resolve('pending-' + Date.now());
                })
                .catch(error => {
                    console.error('Error saving offline:', error);
                    reject(error);
                });
        }
    });
}


// Function to save a pending review to IndexedDB
function savePendingReview(reviewData) {
    return new Promise((resolve, reject) => {
        if (!db) {
            reject(new Error('Database not initialized'));
            return;
        }

        // Add timestamp
        reviewData.timestamp = new Date().toISOString();

        // Start a transaction
        const transaction = db.transaction(['pendingReviews'], 'readwrite');
        const store = transaction.objectStore('pendingReviews');

        // Add the review
        const request = store.add(reviewData);

        request.onsuccess = function (event) {
            resolve(event.target.result);
        };

        request.onerror = function (event) {
            console.error('Error saving to IndexedDB:', event.target.error);
            reject(event.target.error);
        };
    });
}

// Function to check for pending reviews and sync them
function checkPendingReviews() {
    if (!navigator.onLine || !db) return;

    const transaction = db.transaction(['pendingReviews'], 'readonly');
    const store = transaction.objectStore('pendingReviews');
    const request = store.getAll();

    request.onsuccess = function (event) {
        const pendingReviews = event.target.result;

        if (pendingReviews.length > 0) {
            showNotification(`Syncing ${pendingReviews.length} pending reviews...`, 'info');

            // Sync each pending review
            syncPendingReviews(pendingReviews);
        }
    };
}

// Function to sync pending reviews
function syncPendingReviews(pendingReviews) {
    console.log('Syncing pending reviews:', pendingReviews);

    // Process each review one by one
    let syncPromises = pendingReviews.map(review => {
        return new Promise((resolve, reject) => {
            saveReview(review)
                .then(result => {
                    // Remove from pending queue
                    const transaction = db.transaction(['pendingReviews'], 'readwrite');
                    const store = transaction.objectStore('pendingReviews');
                    store.delete(review.id);
                    console.log('Successfully synced review:', review.id);
                    resolve(result);
                })
                .catch(error => {
                    console.error('Error syncing review:', review.id, error);
                    reject(error);
                });
        });
    });

    Promise.allSettled(syncPromises)
        .then(results => {
            const successCount = results.filter(r => r.status === 'fulfilled').length;
            const failCount = results.filter(r => r.status === 'rejected').length;

            if (failCount === 0) {
                showNotification(`All ${successCount} reviews synced successfully`, 'success');
            } else {
                showNotification(`${successCount} reviews synced, ${failCount} failed`, 'warning');
            }
        });
}

// Function to fetch products for a producer
function fetchProducerProducts(producerId) {
    return new Promise((resolve, reject) => {
        if (!producerId) {
            resolve([]);
            return;
        }

        // Call the API to get products for this producer
        fetch(`./api/producer-products.php?producerId=${producerId}`)
            .then(response => {
                if (!response.ok) {
                    throw new Error(`Network response was not ok: ${response.status}`);
                }
                return response.json();
            })
            .then(data => {
                if (data.products) {
                    resolve(data.products);
                } else {
                    resolve([]);
                }
            })
            .catch(error => {
                console.error('Error fetching products:', error);

                // Fall back to a simulated response for demonstration
                // This will be replaced with the actual API in production
                const fallbackProducts = simulateFallbackProducts(producerId);
                resolve(fallbackProducts);
            });
    });
}

// Function to search products
function searchProducts(producerId, query) {
    return new Promise((resolve, reject) => {
        if (!producerId) {
            resolve([]);
            return;
        }

        // Call the API to search products
        fetch(`./api/search-products.php?producerId=${producerId}&query=${encodeURIComponent(query)}`)
            .then(response => {
                if (!response.ok) {
                    throw new Error(`Network response was not ok: ${response.status}`);
                }
                return response.json();
            })
            .then(data => {
                if (data.products) {
                    resolve(data.products);
                } else {
                    resolve([]);
                }
            })
            .catch(error => {
                console.error('Error searching products:', error);

                // Fall back to a local search in case of API failure
                fetchProducerProducts(producerId)
                    .then(products => {
                        // Filter by search query
                        if (query && query.length > 0) {
                            const filteredProducts = products.filter(p =>
                                p.Name.toLowerCase().includes(query.toLowerCase()) ||
                                p.Category.toLowerCase().includes(query.toLowerCase()) ||
                                p.Style.toLowerCase().includes(query.toLowerCase())
                            );
                            resolve(filteredProducts);
                        } else {
                            resolve(products);
                        }
                    });
            });
    });
}

// Temporary fallback function for products if API is not yet available
function simulateFallbackProducts(producerId) {
    // We'll use the actual database structure but simulate the data
    const mockProductsByProducer = {
        // These would be replaced with real data from your database
        1: [
            { ProductID: 101, Name: 'Hoppy Trails IPA', Category: 'Beer', Style: 'IPA', ABV: 6.5 },
            { ProductID: 102, Name: 'Midnight Stout', Category: 'Beer', Style: 'Stout', ABV: 7.2 },
            { ProductID: 103, Name: 'Hazy Days', Category: 'Beer', Style: 'Hazy IPA', ABV: 6.8 },
            { ProductID: 104, Name: 'Summer Wheat', Category: 'Beer', Style: 'Wheat Beer', ABV: 5.2 }
        ],
        2: [
            { ProductID: 201, Name: 'Chardonnay Reserve', Category: 'Wine', Style: 'White', ABV: 13.5 },
            { ProductID: 202, Name: 'Cabernet Sauvignon', Category: 'Wine', Style: 'Red', ABV: 14.2 },
            { ProductID: 203, Name: 'Ros Sunset', Category: 'Wine', Style: 'Ros', ABV: 12.8 }
        ],
        3: [
            { ProductID: 301, Name: 'Single Malt Whiskey', Category: 'Distilled Spirit', Style: 'Whiskey', ABV: 40.0 },
            { ProductID: 302, Name: 'Botanical Gin', Category: 'Distilled Spirit', Style: 'Gin', ABV: 42.5 },
            { ProductID: 303, Name: 'Aged Rum', Category: 'Distilled Spirit', Style: 'Rum', ABV: 38.5 }
        ]
    };

    return mockProductsByProducer[producerId] || [];
}

// Listen for online/offline events
window.addEventListener('online', () => {
    showNotification('You are now online', 'success');
    checkPendingReviews();
});

window.addEventListener('offline', () => {
    showNotification('You are now offline. Reviews will be saved locally.', 'warning');
});
