// wizard-location.js - Handles location functionality for the review wizard

let map;
let marker;
let nearbyProducers = [];

// Initialize location functionality when DOM is loaded
document.addEventListener('DOMContentLoaded', initLocationFunctionality);

function initLocationFunctionality() {
    // Initialize map
    initMap();

    // Set up event listener for geolocation button
    const getLocationBtn = document.getElementById('getCurrentLocation');
    if (getLocationBtn) {
        getLocationBtn.addEventListener('click', getUserLocation);
    }

    // Set up event listener for new venue suggestion
    const suggestVenueBtn = document.getElementById('suggestNewVenue');
    if (suggestVenueBtn) {
        suggestVenueBtn.addEventListener('click', showSuggestVenueForm);
    }

    // Hide the suggest venue section initially
    const suggestVenueSection = document.getElementById('suggestVenueSection');
    if (suggestVenueSection) {
        suggestVenueSection.classList.add('hidden');
    }

    // Set up event listeners for suggested venue form
    setupSuggestedVenueFormListeners();

    // Hide the continue button initially
    const continueBtn = document.getElementById('toVenueStep');
    if (continueBtn) {
        continueBtn.style.display = 'none';
        continueBtn.disabled = true;
    }

    
}

function initMap() {
    const mapContainer = document.getElementById('map');

    if (!mapContainer) return;

    // Initialize Leaflet map with a default view (Huntington Beach, CA)
    map = L.map('map').setView([33.6846, -117.8265], 14);

    L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
        attribution: '&copy; <a href="https://www.openstreetmap.org/copyright">OpenStreetMap</a> contributors'
    }).addTo(map);

    // Add a marker that can be dragged
    marker = L.marker([33.6846, -117.8265], {
        draggable: true
    }).addTo(map);

    // When marker is dragged, update coordinates
    marker.on('dragend', function (event) {
        const position = marker.getLatLng();
        reverseGeocode(position.lat, position.lng);
    });

    // When map is clicked, move marker and update coordinates
    map.on('click', function (e) {
        marker.setLatLng(e.latlng);
        reverseGeocode(e.latlng.lat, e.latlng.lng);
    });
}

function getUserLocation() {
    if (!navigator.geolocation) {
        showNotification('Geolocation is not supported by your browser', 'error');
        return;
    }

    showNotification('Fetching your location...', 'info');

    navigator.geolocation.getCurrentPosition(
        position => {
            const { latitude, longitude } = position.coords;

            // Update map position
            updateMapPosition(latitude, longitude);

            // Check for nearby producers
            checkNearbyProducers(latitude, longitude);
        },
        error => {
            console.error('Geolocation error:', error);
            let errorMessage = 'Unable to get your location';

            switch (error.code) {
                case error.PERMISSION_DENIED:
                    errorMessage = 'You denied the request for geolocation';
                    break;
                case error.POSITION_UNAVAILABLE:
                    errorMessage = 'Location information is unavailable';
                    break;
                case error.TIMEOUT:
                    errorMessage = 'The request to get your location timed out';
                    break;
            }

            showNotification(errorMessage, 'error');
        }
    );
}

function updateMapPosition(latitude, longitude) {
    // Update the map with the new coordinates
    if (map && marker) {
        // Update the map center and zoom
        map.setView([latitude, longitude], 16);

        // Move the marker to the new position
        marker.setLatLng([latitude, longitude]);
    } else {
        console.error('Map or marker not initialized');
    }
}

function checkNearbyProducers(latitude, longitude) {
    // Set a small radius (e.g., 0.3 km) to check if user is at a specific establishment
    const establishmentRadius = 0.3; // 300 meters

    showNotification('Checking if you\'re at a craft producer...', 'info');

    // Call the real API endpoint to find nearby producers
    // Use the path from the root of your domain
    const apiUrl = `./api/nearby-producers.php?latitude=${latitude}&longitude=${longitude}&radius=${establishmentRadius}`;

    console.log('Calling API:', apiUrl);

    fetch(apiUrl)
        .then(response => {
            if (!response.ok) {
                throw new Error(`Network response was not ok: ${response.status}`);
            }
            return response.json();
        })
        .then(data => {
            if (data.producers && data.producers.length > 0) {
                nearbyProducers = data.producers;

                // Sort by distance
                nearbyProducers.sort((a, b) => a.distance - b.distance);

                // Clear the location results container
                const resultsContainer = document.getElementById('locationResults');
                resultsContainer.innerHTML = '';

                // If user is very close to a producer, assume they're at that location
                const closestProducer = nearbyProducers[0];

                if (closestProducer.distance <= establishmentRadius) {
                    // They're at an establishment - display it prominently
                    displaySelectedProducer(closestProducer);
                    showNotification(`You appear to be at ${closestProducer.DBA || closestProducer.CompanyName}`, 'success');

                    // Hide the suggest venue section
                    hideElement('suggestVenueSection');

                    // Hide the continue button since we'll use the producer card to navigate
                    document.getElementById('toVenueStep').style.display = 'none';
                } else {
                    // Show nearby producers for selection
                    displayNearbyProducers(nearbyProducers);
                    showNotification('Select the venue you\'re visiting', 'info');

                    // Hide the suggest venue section
                    hideElement('suggestVenueSection');

                    // Show the continue button but keep it disabled until a producer is selected
                    document.getElementById('toVenueStep').style.display = 'block';
                    document.getElementById('toVenueStep').disabled = true;
                }
            } else {
                // No producers found - show suggest venue section
                showNotification('No craft producers found nearby', 'warning');
                clearLocationResults();
                showElement('suggestVenueSection');

                // Show the continue button for the suggest venue flow
                document.getElementById('toVenueStep').style.display = 'block';
                document.getElementById('toVenueStep').disabled = true;
            }
        })
        .catch(error => {
            console.error('Error checking nearby producers:', error);
            showNotification(`Error: ${error.message}. Please try again.`, 'error');

            // Show suggest venue section on error
            showElement('suggestVenueSection');

            // Show the continue button for the suggest venue flow
            document.getElementById('toVenueStep').style.display = 'block';
            document.getElementById('toVenueStep').disabled = true;
        });
}

function clearLocationResults() {
    const resultsContainer = document.getElementById('locationResults');
    if (resultsContainer) {
        resultsContainer.innerHTML = '';
    }
}

function displaySelectedProducer(producer) {
    const resultsContainer = document.getElementById('locationResults');

    // Create a more prominent producer card that acts as a button
    const producerCard = document.createElement('div');
    producerCard.className = 'producer-card selected';

    // Determine available amenities
    const amenities = [];
    if (producer.TastingRoom === 'Y') amenities.push('Tasting Room');
    if (producer.Tours === 'Y') amenities.push('Tours Available');
    if (producer.Hours) amenities.push(producer.Hours);

    const amenitiesText = amenities.length > 0
        ? `<div class="producer-amenities">${amenities.join('  ')}</div>`
        : '';

    producerCard.innerHTML = `
        <div class="producer-header">
            <h3 class="producer-title">${producer.DBA || producer.CompanyName}</h3>
            <div class="producer-distance">${producer.distance.toFixed(1)} km away</div>
        </div>
        <div class="producer-address">${producer.Street}, ${producer.City}, ${producer.State} ${producer.Zip}</div>
        ${amenitiesText}
        <button class="producer-select-btn">Review This Venue</button>
    `;

    // Add click event to select this producer and proceed
    const selectButton = producerCard.querySelector('.producer-select-btn');
    selectButton.addEventListener('click', () => {
        selectProducer(producer);

        // Proceed to the next step
        navigateToStep(2);
    });

    // Add the card to the results container
    resultsContainer.appendChild(producerCard);

    // Pre-select this producer
    selectProducer(producer);
}

function displayNearbyProducers(producers) {
    const resultsContainer = document.getElementById('locationResults');
    resultsContainer.innerHTML = '';

    if (producers.length === 0) {
        // No results message is now handled by the suggest venue section
        return;
    }

    producers.forEach(producer => {
        const resultElement = document.createElement('div');
        resultElement.className = 'location-result';
        resultElement.innerHTML = `
            <div class="location-name">${producer.DBA || producer.CompanyName}</div>
            <div class="location-address">${producer.Street}, ${producer.City}, ${producer.State} ${producer.Zip}</div>
            <div class="location-distance">${producer.distance.toFixed(1)} km away</div>
        `;

        // Add click event to select this producer
        resultElement.addEventListener('click', () => {
            // Remove selected class from all results
            document.querySelectorAll('.location-result').forEach(el => {
                el.classList.remove('selected');
            });

            // Add selected class to this result
            resultElement.classList.add('selected');

            // Select this producer
            selectProducer(producer);

            // Enable the continue button
            document.getElementById('toVenueStep').disabled = false;
        });

        resultsContainer.appendChild(resultElement);
    });
}

function selectProducer(producer) {
    // Update state
    reviewState.location.producerId = producer.ProducerID;
    reviewState.location.venueName = producer.DBA || producer.CompanyName;
    reviewState.location.venueAddress = `${producer.Street}, ${producer.City}, ${producer.State} ${producer.Zip}`;
    reviewState.location.latitude = producer.GeoLat;
    reviewState.location.longitude = producer.GeoLong;
    reviewState.location.isNewVenue = false;

    // Update map position
    updateMapPosition(producer.GeoLat, producer.GeoLong);

    // Pre-fill venue information for next step
    document.getElementById('venueName').textContent = producer.DBA || producer.CompanyName;
    document.getElementById('venueAddress').textContent = `${producer.Street}, ${producer.City}, ${producer.State} ${producer.Zip}`;

    // Hide any venue suggestion form
    hideElement('venueFormSection');
    hideElement('suggestVenueSection');
}

function showSuggestVenueForm() {
    // Hide the suggest venue button section
    hideElement('suggestVenueSection');

    // Show the venue form section
    showElement('venueFormSection');

    // Focus on the venue name input
    document.getElementById('suggestedVenueName').focus();
}

function setupSuggestedVenueFormListeners() {
    // Listen for input on suggested venue name and address
    const nameInput = document.getElementById('suggestedVenueName');
    const addressInput = document.getElementById('suggestedVenueAddress');
    const continueButton = document.getElementById('toVenueStep');

    if (nameInput && addressInput) {
        // Enable/disable continue button based on inputs
        const checkInputs = () => {
            const nameValue = nameInput.value.trim();
            const addressValue = addressInput.value.trim();

            continueButton.disabled = !(nameValue && addressValue);
        };

        nameInput.addEventListener('input', checkInputs);
        addressInput.addEventListener('input', checkInputs);

        // Handle form submission
        continueButton.addEventListener('click', () => {
            if (!continueButton.disabled && nameInput.classList.contains('active')) {
                handleSuggestNewVenue();
            }
        });
    }
}

function handleSuggestNewVenue() {
    // Get venue details from form
    const venueName = document.getElementById('suggestedVenueName').value.trim();
    const venueAddress = document.getElementById('suggestedVenueAddress').value.trim();

    if (!venueName || !venueAddress) {
        showNotification('Please enter both venue name and address', 'error');
        return;
    }

    // Get current marker position
    const position = marker.getLatLng();

    // Update state with new venue info
    reviewState.location.producerId = null; // No ID for new venues yet
    reviewState.location.venueName = venueName;
    reviewState.location.venueAddress = venueAddress;
    reviewState.location.latitude = position.lat;
    reviewState.location.longitude = position.lng;
    reviewState.location.isNewVenue = true;

    // Update UI
    const resultsContainer = document.getElementById('locationResults');
    resultsContainer.innerHTML = `
        <div class="producer-card selected">
            <div class="producer-header">
                <h3 class="producer-title">${venueName} <span class="product-tag">New</span></h3>
            </div>
            <div class="producer-address">${venueAddress}</div>
            <div class="producer-note">You are suggesting a new venue. This will be verified by our team.</div>
        </div>
    `;

    // Pre-fill venue information for next step
    document.getElementById('venueName').textContent = venueName;
    document.getElementById('venueAddress').textContent = venueAddress;

    // Enable the continue button
    document.getElementById('toVenueStep').disabled = false;

    showNotification('New venue added for review. This will be verified by our team.', 'success');

    // Hide the form
    hideElement('venueFormSection');
}

function reverseGeocode(latitude, longitude) {
    // Use OpenStreetMap's Nominatim service for reverse geocoding
    fetch(`https://nominatim.openstreetmap.org/reverse?format=json&lat=${latitude}&lon=${longitude}&zoom=18&addressdetails=1`)
        .then(response => {
            if (!response.ok) {
                throw new Error('Network response was not ok');
            }
            return response.json();
        })
        .then(data => {
            if (data && data.display_name) {
                // Format the address nicely
                let address = '';

                if (data.address) {
                    const addr = data.address;

                    // Build street address
                    if (addr.house_number && addr.road) {
                        address += `${addr.house_number} ${addr.road}, `;
                    } else if (addr.road) {
                        address += `${addr.road}, `;
                    }

                    // Add city, state, zip
                    if (addr.city || addr.town || addr.village) {
                        address += `${addr.city || addr.town || addr.village}, `;
                    }

                    if (addr.state) {
                        address += `${addr.state} `;
                    }

                    if (addr.postcode) {
                        address += addr.postcode;
                    }
                } else {
                    // Use the display name as fallback
                    address = data.display_name;
                }

                // Update the suggested venue address if form is visible
                if (!document.getElementById('venueFormSection').classList.contains('hidden')) {
                    document.getElementById('suggestedVenueAddress').value = address;
                }

                // Update state with address
                reviewState.location.venueAddress = address;
            }
        })
        .catch(error => {
            console.error('Error reverse geocoding:', error);
        });
}

// Helper functions to show/hide elements
function showElement(elementId) {
    const element = document.getElementById(elementId);
    if (element) {
        element.classList.remove('hidden');

        // Mark inputs as active if in the venue form
        if (elementId === 'venueFormSection') {
            const inputs = element.querySelectorAll('input');
            inputs.forEach(input => input.classList.add('active'));
        }
    }
}

function hideElement(elementId) {
    const element = document.getElementById(elementId);
    if (element) {
        element.classList.add('hidden');

        // Mark inputs as inactive if in the venue form
        if (elementId === 'venueFormSection') {
            const inputs = element.querySelectorAll('input');
            inputs.forEach(input => input.classList.remove('active'));
        }
    }
}