// wizard-photos.js - Handles photo uploads for the review wizard

// Initialize photo handling when DOM is loaded
document.addEventListener('DOMContentLoaded', initPhotoHandling);

function initPhotoHandling() {
    // Set up the establishment photo upload
    setupEstablishmentPhotoUpload();

    // Set up retake button functionality
    setupRetakeButton();

    // Product photo uploads will be set up dynamically when the product forms are created
    setupProductPhotoEventDelegation();
}

function setupEstablishmentPhotoUpload() {
    const photoUploadArea = document.getElementById('establishmentPhotoUpload');
    const photoInput = document.getElementById('establishmentPhoto');
    const photoPreview = document.getElementById('establishmentPhotoPreview');

    if (photoUploadArea && photoInput) {
        photoUploadArea.addEventListener('click', () => {
            photoInput.click();
        });

        photoInput.addEventListener('change', (e) => {
            handlePhotoUpload(e, 'establishment');
        });
    }
}

function setupProductPhotoEventDelegation() {
    // Use event delegation to handle product photo uploads
    // This allows us to attach events to dynamically created elements
    document.addEventListener('click', function (e) {
        // Check if the clicked element or its parent is a product photo upload area
        const uploadArea = e.target.closest('.product-photo-upload');
        if (uploadArea) {
            const photoInput = uploadArea.querySelector('.product-photo');
            if (photoInput) {
                photoInput.click();
            }
        }
    });

    // Handle file input change events through delegation as well
    document.addEventListener('change', function (e) {
        if (e.target.classList.contains('product-photo')) {
            // Get the product index from the container
            const productCard = e.target.closest('.product-card');
            if (productCard) {
                const productIndex = productCard.getAttribute('data-product-index');
                handleProductPhotoUpload(e, parseInt(productIndex));
            }
        }
    });
}

function setupRetakeButton() {
    // Use event delegation for the retake button
    document.addEventListener('click', function (e) {
        // Find if the clicked element is a retake button
        if (e.target.classList.contains('retake-button') ||
            e.target.closest('.retake-button')) {

            // Find the closest photo preview
            const previewArea = e.target.closest('.photo-preview');
            if (!previewArea) return;

            // Determine the type (establishment or product)
            const type = previewArea.id.replace('PhotoPreview', '');

            // Hide preview and show upload area again
            previewArea.style.display = 'none';
            const uploadArea = document.getElementById(`${type}PhotoUpload`);
            if (uploadArea) uploadArea.style.display = 'flex';

            // Clear the image preview
            previewArea.innerHTML = '';
            previewArea.classList.remove('has-image');

            // Reset the state
            if (type === 'establishment') {
                reviewState.establishment.photoFile = null;
                reviewState.establishment.photoUrl = null;
            } else {
                // Handle product photos - extract index from the type
                const matches = type.match(/product(\d+)/);
                if (matches && matches.length > 1) {
                    const productIndex = parseInt(matches[1]) - 1;
                    if (reviewState.products[productIndex]) {
                        reviewState.products[productIndex].photoFile = null;
                        reviewState.products[productIndex].photoUrl = null;
                    }
                }
            }
        }
    });
}

function handlePhotoUpload(event, type) {
    const file = event.target.files[0];
    if (!file) return;

    // Check if file is an image
    if (!file.type.startsWith('image/')) {
        showNotification('Please select an image file', 'error');
        return;
    }

    // Check file size (limit to 5MB)
    if (file.size > 5 * 1024 * 1024) {
        showNotification('Image size should be less than 5MB', 'error');
        return;
    }

    // Hide the upload area and show the preview area
    const uploadArea = document.getElementById(`${type}PhotoUpload`);
    const previewArea = document.getElementById(`${type}PhotoPreview`);

    if (uploadArea) uploadArea.style.display = 'none';
    if (previewArea) previewArea.style.display = 'block';

    // Show preview of the image
    displayPhotoPreview(file, type);

    // Update state with the photo
    reviewState.establishment.photoFile = file;

    // Create a temporary URL for the photo preview
    reviewState.establishment.photoUrl = URL.createObjectURL(file);

    // If it's an establishment photo, verify the geocoding
    if (type === 'establishment') {
        verifyPhotoLocation(file);
    }
}

function handleProductPhotoUpload(event, productIndex) {
    const file = event.target.files[0];
    if (!file) return;

    // Check if file is an image
    if (!file.type.startsWith('image/')) {
        showNotification('Please select an image file', 'error');
        return;
    }

    // Check file size (limit to 5MB)
    if (file.size > 5 * 1024 * 1024) {
        showNotification('Image size should be less than 5MB', 'error');
        return;
    }

    // Get the related elements
    const productCard = document.querySelector(`.product-card[data-product-index="${productIndex}"]`);
    if (!productCard) return;

    const uploadArea = productCard.querySelector('.product-photo-upload');
    const previewArea = productCard.querySelector('.product-photo-preview');

    // Hide upload area and show preview
    if (uploadArea) uploadArea.style.display = 'none';
    if (previewArea) previewArea.style.display = 'block';

    // Adjust index for zero-based array
    const stateIndex = productIndex - 1;

    // Show preview of the image
    displayProductPhotoPreview(file, productIndex);

    // Update state with the photo
    if (reviewState.products[stateIndex]) {
        reviewState.products[stateIndex].photoFile = file;

        // Create a temporary URL for the photo preview
        reviewState.products[stateIndex].photoUrl = URL.createObjectURL(file);
    }
}

function displayPhotoPreview(file, type) {
    const reader = new FileReader();
    const previewElement = document.getElementById(`${type}PhotoPreview`);

    if (!previewElement) return;

    reader.onload = (e) => {
        previewElement.innerHTML = `
            <img src="${e.target.result}" alt="${type} photo">
            <button class="retake-button">Retake</button>
        `;
        previewElement.classList.add('has-image');
    };

    reader.readAsDataURL(file);
}

function displayProductPhotoPreview(file, productIndex) {
    const reader = new FileReader();
    const previewElement = document.querySelector(`.product-card[data-product-index="${productIndex}"] .product-photo-preview`);

    if (!previewElement) return;

    reader.onload = (e) => {
        previewElement.innerHTML = `
            <img src="${e.target.result}" alt="Product photo">
            <button class="retake-button">Retake</button>
        `;
        previewElement.classList.add('has-image');
    };

    reader.readAsDataURL(file);
}

function verifyPhotoLocation(file) {
    // In a real implementation, this would check the photo's EXIF data
    // for location and compare with the selected location

    // Check if we have a selected location
    if (!reviewState.location.latitude || !reviewState.location.longitude) {
        showNotification('Please select a location before uploading an establishment photo', 'warning');
        return;
    }

    // Show a loading message
    showNotification('Verifying photo location...', 'info');

    // For this demo, we'll simulate reading EXIF data
    setTimeout(() => {
        const hasGpsData = Math.random() < 0.8; // 80% chance of having GPS data

        if (hasGpsData) {
            // Create "nearby" coordinates (within 100m)
            const latOffset = (Math.random() - 0.5) * 0.002; // ~100m in latitude
            const lngOffset = (Math.random() - 0.5) * 0.002; // ~100m in longitude

            const exifLat = reviewState.location.latitude + latOffset;
            const exifLng = reviewState.location.longitude + lngOffset;

            // Calculate distance
            const distance = calculateDistance(
                exifLat, exifLng,
                reviewState.location.latitude, reviewState.location.longitude
            );

            if (distance <= 0.3) { // Within 300 meters
                showNotification('Photo location verified successfully!', 'success');
            } else {
                showNotification('Warning: Photo location does not match selected venue', 'warning');
            }
        } else {
            showNotification('No location data found in photo. Please ensure you\'re at the venue.', 'warning');
        }
    }, 1500);
}

// Calculate distance between two coordinates using Haversine formula
function calculateDistance(lat1, lon1, lat2, lon2) {
    const R = 6371; // Radius of Earth in kilometers
    const dLat = (lat2 - lat1) * Math.PI / 180;
    const dLon = (lon2 - lon1) * Math.PI / 180;

    const a =
        Math.sin(dLat / 2) * Math.sin(dLat / 2) +
        Math.cos(lat1 * Math.PI / 180) * Math.cos(lat2 * Math.PI / 180) *
        Math.sin(dLon / 2) * Math.sin(dLon / 2);

    const c = 2 * Math.atan2(Math.sqrt(a), Math.sqrt(1 - a));
    const distance = R * c; // Distance in kilometers

    return distance;
}