// wizard-products.js - Handles product-specific functionality for the review wizard

// Initialize product functionality when DOM is loaded
document.addEventListener('DOMContentLoaded', initProductFunctionality);

// Define style options for different product categories
const styleOptions = {
    'Beer': ['IPA', 'Stout', 'Pilsner', 'Lager', 'Wheat Beer', 'Sour', 'Porter', 'Pale Ale', 'Other'],
    'Wine': ['Red', 'White', 'Ros', 'Sparkling', 'Dessert', 'Fortified', 'Orange', 'Natural', 'Other'],
    'Mead': ['Traditional', 'Melomel', 'Metheglin', 'Braggot', 'Cyser', 'Pyment', 'Other'],
    'Cider': ['Dry', 'Semi-Dry', 'Sweet', 'Perry', 'Fruit', 'Hopped', 'Spiced', 'Other'],
    'Distilled Spirit': ['Whiskey', 'Bourbon', 'Rum', 'Vodka', 'Gin', 'Tequila', 'Brandy', 'Liqueur', 'Other'],
    'Seltzer': ['Fruit', 'Berry', 'Citrus', 'Tropical', 'Herbal', 'Unflavored', 'Other'],
    'Kombucha': ['Original', 'Fruit', 'Flower', 'Herb', 'Spiced', 'Other'],
    'Other': ['Non-Alcoholic', 'Coffee', 'Tea', 'Mixed Drink', 'Cocktail', 'Other']
};

function initProductFunctionality() {
    // Set up event delegation for product search and selection
    setupProductSearch();

    // Set up event listener for the "Next Product" button
    const nextProductBtn = document.querySelector('.button-primary');
    if (nextProductBtn) {
        nextProductBtn.addEventListener('click', handleNextProduct);
    }

    // Set up event listener for the "Skip to Review" button
    const skipToReviewBtn = document.querySelector('.button-outline');
    if (skipToReviewBtn) {
        skipToReviewBtn.addEventListener('click', () => navigateToStep(4));
    }

    // Set up event delegation for category changes
    document.addEventListener('change', function (e) {
        if (e.target.classList.contains('category-select')) {
            const productCard = e.target.closest('.product-card');
            if (productCard) {
                const productIndex = parseInt(productCard.getAttribute('data-product-index'));
                handleCategoryChange(e.target.value, productIndex);
            }
        }

        if (e.target.classList.contains('style-select')) {
            const productCard = e.target.closest('.product-card');
            if (productCard) {
                const productIndex = parseInt(productCard.getAttribute('data-product-index'));
                handleStyleChange(e.target.value, productIndex);
            }
        }
    });
}

function setupProductSearch() {
    // Make sure the document exists before adding event listeners
    if (!document) return;

    // Use event delegation for product search inputs
    document.addEventListener('input', function (e) {
        if (e.target && e.target.classList && e.target.classList.contains('product-search')) {
            // Debounce search to avoid too many API calls
            clearTimeout(e.target.searchTimeout);

            e.target.searchTimeout = setTimeout(() => {
                const productCard = e.target.closest('.product-card');
                if (productCard) {
                    const productIndex = parseInt(productCard.getAttribute('data-product-index'));
                    // Make sure we have a valid product index and input value
                    if (!isNaN(productIndex) && e.target.value) {
                        handleProductSearch(e.target.value, productIndex);
                    }
                }
            }, 300);
        }
    });

    // Event delegation for selecting a product from search results
    document.addEventListener('click', function (e) {
        const listItem = e.target.closest('.product-list-item');
        if (listItem) {
            const productCard = listItem.closest('.product-card');

            if (productCard) {
                const productIndex = parseInt(productCard.getAttribute('data-product-index'));
                if (isNaN(productIndex)) {
                    console.error('Invalid product index');
                    return;
                }

                const productNameElement = listItem.querySelector('.product-name');
                if (!productNameElement) {
                    console.error('Product name element not found');
                    return;
                }

                const productName = productNameElement.textContent.trim();

                // Extract category and style from the tag text
                const tagElement = listItem.querySelector('.product-tag');
                let category = '';
                let style = '';

                if (tagElement) {
                    const tagText = tagElement.textContent.trim();
                    const tagParts = tagText.split(' - ');

                    if (tagParts.length >= 2) {
                        category = tagParts[0];
                        style = tagParts[1];
                    }
                }

                // Get product ID if available (stored as data attribute)
                const productId = listItem.getAttribute('data-product-id') || null;

                selectProduct(productName, category, style, productId, productIndex);
            }
        }
    });
}

// Update this function in your wizard-products.js file
function handleProductSearch(query, productIndex) {
    // Make sure query is a string before using trim
    if (!query || typeof query !== 'string' || query.trim().length < 2) return;

    const productCard = document.querySelector(`.product-card[data-product-index="${productIndex}"]`);
    // If we can't find the product card, log an error and return early
    if (!productCard) {
        console.error(`Product card with index ${productIndex} not found`);
        return;
    }

    const productList = productCard.querySelector('.product-list');
    // Check if elements exist before using them
    const noResultsMessage = productCard.querySelector('.no-results-message');
    const productDetails = productCard.querySelector('.product-details');

    // Make sure all elements exist before proceeding
    if (!productList) {
        console.error(`Product list not found in product card ${productIndex}`);
        return;
    }

    // Show the product list and loading state
    productList.classList.remove('hidden');
    productList.innerHTML = '<div class="product-list-item">Searching...</div>';

    // Get the producer ID from the state
    const producerId = reviewState.location.producerId;

    if (!producerId) {
        productList.innerHTML = '<div class="product-list-item">Please select a venue first</div>';
        return;
    }

    // Search for products
    searchProducts(producerId, query)
        .then(products => {
            // Make sure elements exist before using them
            if (!productList) return;

            if (!products || products.length === 0) {
                // Show no results message
                productList.classList.add('hidden');
                if (noResultsMessage) {
                    noResultsMessage.classList.remove('hidden');
                }

                // Show the details section for a new product
                if (productDetails) {
                    productDetails.classList.remove('hidden');
                }
            } else {
                // Show search results
                if (noResultsMessage) {
                    noResultsMessage.classList.add('hidden');
                }
                displayProductSearchResults(products, productList);
            }
        })
        .catch(error => {
            console.error('Error searching products:', error);
            if (productList) {
                productList.innerHTML = '<div class="product-list-item">Error searching products</div>';
            }
        });

    // Update the state with the product name
    const stateIndex = productIndex - 1;
    if (reviewState.products && reviewState.products[stateIndex]) {
        reviewState.products[stateIndex].name = query;
        reviewState.products[stateIndex].isNewProduct = true;
    }

    // Show the details section for a new product if it exists
    if (productDetails) {
        productDetails.classList.remove('hidden');
    }
}



// Add this function to your wizard-products.js file
function displayProductSearchResults(products, listElement) {
    // Make sure the list element exists
    if (!listElement) {
        console.error('List element is null or undefined');
        return;
    }

    listElement.innerHTML = '';

    if (!products || products.length === 0) {
        listElement.innerHTML = `
            <div class="product-list-item">
                No matching products found. Enter details for a new product.
            </div>
        `;
        return;
    }

    products.forEach(product => {
        const item = document.createElement('div');
        item.className = 'product-list-item';

        // Make sure product has a ProductID before setting attribute
        if (product.ProductID) {
            item.setAttribute('data-product-id', product.ProductID);
        }

        item.innerHTML = `
            <div class="product-name">${product.Name || 'Unknown Product'} 
                <span class="product-tag">${product.Category || 'Uncategorized'} - ${product.Style || 'Other'}</span>
            </div>
        `;

        listElement.appendChild(item);
    });
}

// Update this function in your wizard-products.js file
function selectProduct(name, category, style, productId, productIndex) {
    // Update the state
    const stateIndex = productIndex - 1;

    // Make sure the state array exists and has this index
    if (!reviewState.products || !reviewState.products[stateIndex]) {
        console.error(`Product state at index ${stateIndex} does not exist`);
        return;
    }

    reviewState.products[stateIndex].id = productId;
    reviewState.products[stateIndex].name = name;
    reviewState.products[stateIndex].category = category;
    reviewState.products[stateIndex].style = style;
    reviewState.products[stateIndex].isNewProduct = !productId;

    // Update the UI
    const productCard = document.querySelector(`.product-card[data-product-index="${productIndex}"]`);
    if (!productCard) {
        console.error(`Product card with index ${productIndex} not found`);
        return;
    }

    const searchInput = productCard.querySelector('.product-search');
    const productList = productCard.querySelector('.product-list');
    const noResultsMessage = productCard.querySelector('.no-results-message');
    const productDetails = productCard.querySelector('.product-details');
    const productStatus = productCard.querySelector('.product-status');

    // Update the search input if it exists
    if (searchInput) {
        searchInput.value = name;
    }

    // Add status badge (new or established) if status element exists
    if (productStatus) {
        productStatus.innerHTML = productId ?
            '<span class="established-flag">Established</span>' :
            '<span class="product-tag">New</span>';
    }

    // Hide the product list and no results message if they exist
    if (productList) {
        productList.classList.add('hidden');
    }
    if (noResultsMessage) {
        noResultsMessage.classList.add('hidden');
    }

    // Show the details section if it exists
    if (productDetails) {
        productDetails.classList.remove('hidden');
    }

    // Set the category and style if they're available
    if (category) {
        const categorySelect = productCard.querySelector('.category-select');
        if (categorySelect) {
            categorySelect.value = category;

            // Update style options
            updateStyleOptions(categorySelect, productCard, style);
        }
    }
}



function updateStyleOptions(categorySelect, productCard, selectedStyle = '') {
    const category = categorySelect.value;
    const styleSelect = productCard.querySelector('.style-select');

    if (!styleSelect) return;

    // Clear existing options
    styleSelect.innerHTML = '<option value="" disabled selected>Select a style</option>';

    // Add new options based on category
    if (category && styleOptions[category]) {
        styleOptions[category].forEach(style => {
            const option = document.createElement('option');
            option.value = style;
            option.textContent = style;
            styleSelect.appendChild(option);
        });

        // Set the selected style if provided
        if (selectedStyle && styleOptions[category].includes(selectedStyle)) {
            styleSelect.value = selectedStyle;
        }
    }
}

function handleCategoryChange(category, productIndex) {
    // Update the state
    const stateIndex = productIndex - 1;
    reviewState.products[stateIndex].category = category;

    // Update the UI
    const productCard = document.querySelector(`.product-card[data-product-index="${productIndex}"]`);
    const categorySelect = productCard.querySelector('.category-select');

    // Update style options
    updateStyleOptions(categorySelect, productCard);
}

function handleStyleChange(style, productIndex) {
    // Update the state
    const stateIndex = productIndex - 1;
    reviewState.products[stateIndex].style = style;
}

function handleNextProduct() {
    // Validate current product
    const currentProductIndex = getCurrentProductIndex();

    if (!validateProduct(currentProductIndex)) {
        return;
    }

    // If it's the last product, go to the review step
    if (currentProductIndex >= reviewState.productCount) {
        navigateToStep(4);
        return;
    }

    // Otherwise, show the next product
    showProductForm(currentProductIndex + 1);
}

function getCurrentProductIndex() {
    // Get the currently visible product form
    const visibleForm = document.querySelector('.product-card');
    if (!visibleForm) return 1;

    return parseInt(visibleForm.getAttribute('data-product-index')) || 1;
}

function validateProduct(productIndex) {
    // Get the form elements
    const productCard = document.querySelector(`.product-card[data-product-index="${productIndex}"]`);
    if (!productCard) return false;

    const productName = productCard.querySelector('.product-search').value.trim();
    const category = productCard.querySelector('.category-select').value;

    // Check required fields
    if (!productName) {
        showNotification('Please enter a product name', 'error');
        return false;
    }

    if (!category) {
        showNotification('Please select a category', 'error');
        return false;
    }

    return true;
}

function showProductForm(productIndex) {
    // Hide all product forms
    const productForms = document.querySelectorAll('.product-card');
    productForms.forEach(form => {
        form.remove();
    });

    // Update the product counter display
    const counterValue = document.querySelector('.counter-value');
    if (counterValue) {
        counterValue.textContent = `${productIndex} of ${reviewState.productCount}`;
    }

    // Show the form for the specified product
    const container = document.getElementById('productEntryContainer');

    // Get the product template
    const template = document.getElementById('productEntryTemplate').innerHTML;
    const productHtml = template.replace(/{index}/g, productIndex);

    // Create and append the new form
    const tempDiv = document.createElement('div');
    tempDiv.innerHTML = productHtml;
    const productElement = tempDiv.firstElementChild;

    container.appendChild(productElement);

    // Set up event listeners
    setupProductFormListeners(productIndex);

    // Populate the form if we have data for this product
    populateProductForm(productIndex);
}

function populateProductForm(productIndex) {
    const stateIndex = productIndex - 1;
    if (!reviewState.products[stateIndex]) return;

    const product = reviewState.products[stateIndex];
    const productCard = document.querySelector(`.product-card[data-product-index="${productIndex}"]`);
    if (!productCard) return;

    // Set product name
    const nameInput = productCard.querySelector('.product-search');
    if (nameInput && product.name) {
        nameInput.value = product.name;
    }

    // Set category
    const categorySelect = productCard.querySelector('.category-select');
    if (categorySelect && product.category) {
        categorySelect.value = product.category;

        // Update style options
        updateStyleOptions(categorySelect, productCard, product.style);
    }

    // Set rating
    const ratingInput = productCard.querySelector('.product-rating');
    const ratingDisplay = productCard.querySelector('.product-rating-value');
    if (ratingInput && ratingDisplay && product.rating) {
        ratingInput.value = product.rating;
        ratingDisplay.textContent = product.rating.toFixed(1);
    }

    // Set quick take
    const quickTakeInput = productCard.querySelector('.product-quick-take');
    const quickTakeCount = productCard.querySelector('.product-quick-take-count');
    if (quickTakeInput && quickTakeCount && product.quickTake) {
        quickTakeInput.value = product.quickTake;
        quickTakeCount.textContent = product.quickTake.length;
    }

    // Show photo if available
    if (product.photoUrl) {
        const photoPreview = productCard.querySelector('.product-photo-preview');
        if (photoPreview) {
            photoPreview.innerHTML = `<img src="${product.photoUrl}" alt="Product photo">`;
            photoPreview.classList.add('has-image');
        }
    }

    // Show product details section
    const productDetails = productCard.querySelector('.product-details');
    if (productDetails) {
        productDetails.classList.remove('hidden');
    }
}
