// service-worker.js - PWA Service Worker

const CACHE_NAME = 'craft-review-v1';
const ASSETS = [
    './',
    './index.html',
    './discover.html',
    './reviews.html',
    './css/style.css',
    './css/wizard-style.css',
    './js/app.js',
    './js/location.js',
    './js/photos.js',
    './js/db.js',
    './js/products.js',
    './js/discover.js',
    './js/wizard-app.js',
    './js/wizard-db.js',
    './js/wizard-location.js',
    './wizard-photos.js',
    './manifest.json',
    './images/icons/icon-72x72.png',
    './images/icons/icon-96x96.png',
    './images/icons/icon-128x128.png',
    './images/icons/icon-144x144.png',
    './images/icons/icon-152x152.png',
    './images/icons/icon-192x192.png',
    './images/icons/icon-384x384.png',
    './images/icons/icon-512x512.png',
    './images/icons/star-empty.svg',
    './images/icons/star-filled.svg',
    './images/icons/half-star.svg',
    './images/icons/location-marker.svg',
    './images/icons/text-message.svg',
    './images/icons/email.svg',
    './images/icons/feed.svg',
    './images/icons/share.svg',
    './images/icons/filter.svg',
    './images/icons/check.svg',
    './images/camera.svg',
    './images/location-camera.svg',
    './images/location-pin.svg',
    './images/star-empty.svg',
    './images/star-filled.svg'
];

// Install event - cache all necessary assets
self.addEventListener('install', event => {
    event.waitUntil(
        caches.open(CACHE_NAME)
            .then(cache => {
                console.log('Caching assets');
                return cache.addAll(ASSETS);
            })
            .then(() => self.skipWaiting())
    );
});

// Activate event - clean up old caches
self.addEventListener('activate', event => {
    event.waitUntil(
        caches.keys().then(cacheNames => {
            return Promise.all(
                cacheNames.map(cache => {
                    if (cache !== CACHE_NAME) {
                        console.log('Clearing old cache:', cache);
                        return caches.delete(cache);
                    }
                })
            );
        }).then(() => self.clients.claim())
    );
});

// Fetch event - serve from cache, fallback to network
self.addEventListener('fetch', event => {
    // Skip cross-origin requests
    if (!event.request.url.startsWith(self.location.origin)) {
        return;
    }

    // Skip POST or other non-GET requests - those should go straight to network
    if (event.request.method !== 'GET') {
        return;
    }

    event.respondWith(
        caches.match(event.request)
            .then(cachedResponse => {
                if (cachedResponse) {
                    return cachedResponse;
                }

                // Not in cache, get from network
                return fetch(event.request)
                    .then(response => {
                        // Make a copy of the response
                        const responseToCache = response.clone();

                        // Open the cache and add the new response
                        caches.open(CACHE_NAME)
                            .then(cache => {
                                cache.put(event.request, responseToCache);
                            });

                        return response;
                    })
                    .catch(() => {
                        // Network failed, try to return a cached fallback if it's an HTML request
                        if (event.request.headers.get('accept').includes('text/html')) {
                            return caches.match('/offline.html');
                        }

                        // For image requests, return a placeholder
                        if (event.request.url.match(/\.(jpg|jpeg|png|gif|svg)$/)) {
                            return caches.match('/images/placeholder.svg');
                        }

                        // Otherwise just return an error
                        return new Response('Network error occurred', {
                            status: 408,
                            headers: { 'Content-Type': 'text/plain' }
                        });
                    });
            })
    );
});

// Background sync for pending uploads
self.addEventListener('sync', event => {
    if (event.tag === 'sync-pending-reviews') {
        event.waitUntil(syncPendingReviews());
    }
});

// Push notification handler
self.addEventListener('push', event => {
    if (!event.data) return;

    const data = event.data.json();

    const options = {
        body: data.body || 'New notification from Craft Review',
        icon: '/images/icons/icon-192x192.png',
        badge: '/images/icons/badge-72x72.png',
        vibrate: [100, 50, 100],
        data: {
            url: data.url || '/'
        }
    };

    event.waitUntil(
        self.registration.showNotification(data.title || 'Craft Review', options)
    );
});

// Notification click handler
self.addEventListener('notificationclick', event => {
    event.notification.close();

    const url = event.notification.data.url;

    event.waitUntil(
        clients.matchAll({ type: 'window' })
            .then(windowClients => {
                // Check if there's already a window open
                for (const client of windowClients) {
                    if (client.url === url && 'focus' in client) {
                        return client.focus();
                    }
                }

                // If not, open a new window
                if (clients.openWindow) {
                    return clients.openWindow(url);
                }
            })
    );
});

// Function to sync pending reviews with the server
async function syncPendingReviews() {
    // This would normally call the API to sync reviews
    // For now, we'll just send a message to the main app
    const clients = await self.clients.matchAll();

    clients.forEach(client => {
        client.postMessage({
            type: 'SYNC_PENDING_REVIEWS'
        });
    });
}
